/*********************************************************************************

Copyright(c) 2019-21 Analog Devices, Inc. All Rights Reserved.

This software is proprietary.  By using this software you agree
to the terms of the associated Analog Devices License Agreement.

*********************************************************************************/
/*!
 * @file:    adi_emdma_2156x.h
 * @brief:   EMDMA Manager definitions and API for ADSP-2156x processor family
 * @version: $Revision: 68342 $
 * @date:    $Date: 2022-09-20 06:27:14 -0400 (Tue, 20 Sep 2022) $
 *
 * @details
 *           This is the primary header file for the EMDMA Manager service
 *           for ADSP-2156x processor family, which contains the API declarations,
 *           data and constant definitions used in the APIs
 */

/** @defgroup DMA_Manager DMA Manager
 *  @defgroup EMDMA_Manager EMDMA Manager
 *  @ingroup  DMA_Manager
 */
 
#ifndef __ADI_EMDMA_2156X_H__
#define __ADI_EMDMA_2156X_H__

#ifdef _MISRA_RULES
#pragma diag(push)
#pragma diag(suppress:misra_rule_5_1:"Allow identifiers to be more than 31 characters")
#pragma diag(suppress:misra_rule_5_7:"Allow identifier names to be reused")
#pragma diag(suppress:misra_rule_18_1:"Allow for forward declaration of handles")
#endif /* _MISRA_RULES */

/*=============  I N C L U D E S   =============*/


/*==============  D E F I N E S  ===============*/
/* IF (The processor family requires cache management) */
#if defined(__ADSP215xx__)
/* Enable DMA cache management */
#define __ADI_EMDMA_CACHE_MANAGEMENT    (1u)
#endif /* defined(__ADSP215xx__) */

#if !defined(_LANGUAGE_ASM)

/** @ingroup EMDMA_Manager
 *  Memory size (in bytes) required to handle an EMDMA Stream
 */
#if defined(__ADSPCORTEXA55__)
#define ADI_EMDMA_STREAM_REQ_MEMORY         (64u)
#else
#define ADI_EMDMA_STREAM_REQ_MEMORY         (32u)
#endif

/** @ingroup EMDMA_Manager
 * 	Enumeration to enable EMDMA done callback for each descriptor in the list
 */
#define ADI_EMDMA_UNMASK_DMA_INT  (BITM_EMDMA_CHNPTR_PCI)

/** @ingroup EMDMA_Manager
 * 	Enumeration to configure EMDMA direction to write to external memory on the fly for a particular descriptor
 */
#define ADI_EMDMA_OTFDIR_WRITE  (BITM_EMDMA_CHNPTR_CPDR)

/** @ingroup EMDMA_Manager
 * 	Enumeration to configure EMDMA direction to read from external memory on the fly for a particular descriptor
 */
#define ADI_EMDMA_OTFDIR_READ  (0u)

/** @ingroup EMDMA_Manager
 *  EMDMA Stream Handle
 */
typedef void* ADI_EMDMA_STREAM_HANDLE;

/** @ingroup EMDMA_Manager
 * EMDMA Stream IDs 
 */
typedef enum
{
    ADI_DMA_EMDMA_S0,        /*!< EMDMA Stream 0 */
    ADI_DMA_EMDMA_S1,        /*!< EMDMA Stream 1 */
	ADI_DMA_EMDMA_END_COUNT
} ADI_EMDMA_STREAM_ID;

/** @ingroup EMDMA_Manager
 * EMDMA transfer modes
 */
typedef enum
{
	ADI_EMDMA_STANDARD_WRITE_MODE=0,	/*!< Standard write mode				- Sequential writes to external memory for all the descriptors */
	ADI_EMDMA_STANDARD_READ_MODE,		/*!< Standard read mode	 				- Sequential reads from external memory for all the descriptors */
	ADI_EMDMA_STANDARD_OTFDIR_MODE,		/*!< Standard on the fly direction mode - Sequential writes/reads to/from external memory for each descriptor  */
	ADI_EMDMA_SCATTER_MODE,				/*!< Scatter mode 						- Non-sequential, tap-list based writes to external memory */
	ADI_EMDMA_GATHER_MODE,				/*!< Gather mode 						- Non-sequential, tap-list based reads from external memory */
	ADI_EMDMA_DELAYLINE_MODE			/*!< Delay-Line mode 					- Sequential writes followed by non-sequential, tap-list based reads from external memory */
} ADI_EMDMA_TRANSFER_MODE;

/** @ingroup EMDMA_Manager
 * EMDMA Descriptor Reuse Modes
 */
typedef enum
{
	ADI_EMDMA_DESC_REUSE_NONE=0, 			/*!< No reuse of the descriptor information  - default */
	ADI_EMDMA_DESC_REUSE_ALL,				/*!< Reuse all the descriptor information */
	ADI_EMDMA_DESC_REUSE_EXT_STARTADDR		/*!< Reuse only the external start address - useful when write-back is enabled for standard mode and for delay line mode (write back hardwired as enabled)*/
} ADI_EMDMA_DESC_REUSE_MODE;


/** @ingroup EMDMA_Manager
 * EMDMA Event codes posted with the callback function
 */
typedef enum
{
	ADI_EMDMA_EVENT_STANDARD_WRITE_DESC_PROCESSED,	/*!< Processed one/all descriptor(s) in standard write mode
														(pArg = Start address of the processed/first descriptor) */
	ADI_EMDMA_EVENT_STANDARD_READ_DESC_PROCESSED,	/*!< Processed one/all descriptor(s) in standard read mode
														(pArg = Start address of the processed/first descriptor) */
	ADI_EMDMA_EVENT_STANDARD_OTFDIR_DESC_PROCESSED,	/*!< Processed one/all descriptor(s) in standard on the fly direction change mode
														(pArg = Start address of the processed/first descriptor) */
	ADI_EMDMA_EVENT_SCATTER_DESC_PROCESSED,			/*!< Processed one/all descriptor(s) in scatter mode
														(pArg = Start address of the processed/first descriptor) */
	ADI_EMDMA_EVENT_GATHER_DESC_PROCESSED,			/*!< Processed one/all descriptor(s) in gather mode
														(pArg = Start address of the processed/first descriptor) */
	ADI_EMDMA_EVENT_DELAY_LINE_DESC_PROCESSED		/*!< Processed one/all descriptor(s) in delay-line mode
														(pArg = Start address of the processed/first descriptor) */
}ADI_EMDMA_EVENT;

/** @ingroup EMDMA_Manager
 * EMDMA API return codes
 */
typedef enum
{
    ADI_EMDMA_SUCCESS = 0,                /*!< 0x00 - Generic success */
    ADI_EMDMA_FAILURE,                    /*!< 0x01 - Generic failure */
    ADI_EMDMA_IN_USE,                     /*!< 0x02 - Supplied EMDMA Stream ID is already open and in use */
    ADI_EMDMA_NOT_OPEN,                   /*!< 0x03 - EMDMA Stream is not open to preform the requested task */
    ADI_EMDMA_ALREADY_RUNNING,            /*!< 0x04 - Function not permitted since EMDMA Stream is already running */
    ADI_EMDMA_BAD_HANDLE,                 /*!< 0x05 - Bad stream handle */
    ADI_EMDMA_BAD_STREAM_ID,              /*!< 0x06 - No such EMDMA Stream ID */
	ADI_EMDMA_TRANSFER_MODE_INVALID,	  /*!< 0x07 - Invalid EMDMA transfer mode */
    ADI_EMDMA_INT_REGISTER_FAILED,        /*!< 0x08 - Can't register an interrupt */
    ADI_EMDMA_INT_UNREGISTER_FAILED,      /*!< 0x09 - Can't unregister an interrupt */
    ADI_EMDMA_INVALID_NULL_POINTER        /*!< 0x0A -  Specified pointer is invalid and pointing to NULL. */
} ADI_EMDMA_RESULT;

/** @ingroup EMDMA_Manager
 * Structure to handle descriptor list for Standard EMDMA transfer modes with circular buffering disabled
 */
#pragma pack(4)
typedef struct _ADI_EMDMA_DESC_STANDARD
{
	struct _ADI_EMDMA_DESC_STANDARD		 *pNext;				/*!< Next descriptor in the chain, should be NULL for single descriptor chain */
	uint32_t							 Chnptr;				/*!< This field is used for two purposes:
																	 1. DMA done interrupt generation: write this field for all the descriptors either with the macro ADI_EMDMA_UNMASK_DMA_INT
																	 (interrupt/callback generated for each descriptor) or with zero (interrupt/callback generated for the last descriptor only). To avoid
																		race condition for short EMDMA work units, it is recommended to generate interrupt for the last descriptor only. 
																	 2. On the fly DMA direction change (only applicable for ADI_EMDMA_STANDARD_OTFDIR_MODE): write this field with the macro
																		ADI_EMDMA_OTFDIR_WRITE to write to external memory and ADI_EMDMA_OTFDIR_READ to read from external memory. */																
	int32_t	        				     ExtModify;				/*!< External modifier in 32 bits */
#if defined(__ADSPSC598_FAMILY__) && defined(__ADSPCORTEXA55__)
	uint32_t		 				     pExtStartAddress;		/*!< External start address(uint32_t) */
#else
	void		 						*pExtStartAddress;		/*!< External start address */
#endif /* defined(__ADSPSC598_FAMILY__) && defined(__ADSPCORTEXA55__) */

	uint32_t							 Count;					/*!< Total number of 32 bit words to be transferred */
	int32_t								 IntModify; 			/*!< Internal modifier in 32 bits */
#if defined(__ADSPSC598_FAMILY__) && defined(__ADSPCORTEXA55__)
	uint32_t 						     pIntStartAddress; 		/*!< Internal start address(uint32_t) */
#else
	void 								*pIntStartAddress; 		/*!< Internal start address */
#endif /* defined(__ADSPSC598_FAMILY__) && defined(__ADSPCORTEXA55__) */

} ADI_EMDMA_DESC_STANDARD;
#pragma pack()

/** @ingroup EMDMA_Manager
 * Structure to handle descriptor list for Standard EMDMA transfer modes with circular buffering enabled
 */
#pragma pack(4)
typedef struct _ADI_EMDMA_DESC_STANDARD_CIRCULAR
{
	struct  _ADI_EMDMA_DESC_STANDARD_CIRCULAR	*pNext;				/*!< Next descriptor in the chain, should be NULL for single descriptor chain */
	uint32_t									 Chnptr;			/*!< Write this field for all the descriptors either with the macro ADI_EMDMA_UNMASK_DMA_INT
																		(interrupt/callback generated for each descriptor) or with zero (interrupt/callback generated for the last descriptor only). To avoid
																		race condition for short EMDMA work units, it is recommended to generate interrupt for the last descriptor only. */
	uint32_t	 								 ExtLength;			/*!< Length of the external circular buffer in 32 bits */
#if defined(__ADSPSC598_FAMILY__) && defined(__ADSPCORTEXA55__)
	uint32_t									 pExtBaseAddress;	/*!< Base address of the external circular buffer in 32 bits(uint32_t) */
#else
	void										*pExtBaseAddress;	/*!< Base address of the external circular buffer in 32 bits */
#endif /* defined(__ADSPSC598_FAMILY__) && defined(__ADSPCORTEXA55__) */
	int32_t	        						     ExtModify;			/*!< External modifier in 32 bits */
#if defined(__ADSPSC598_FAMILY__) && defined(__ADSPCORTEXA55__)
	uint32_t		 						     pExtStartAddress;	/*!< External start address(uint32_t) */
#else
	void		 								*pExtStartAddress;	/*!< External start address */
#endif /* defined(__ADSPSC598_FAMILY__) && defined(__ADSPCORTEXA55__) */

	uint32_t									 Count;				/*!< Total number of 32 bit words to be transferred */
	int32_t										 IntModify; 		/*!< Internal modifier in 32 bits */
#if defined(__ADSPSC598_FAMILY__) && defined(__ADSPCORTEXA55__)
	uint32_t 									 pIntStartAddress; 	/*!< Internal start address(uint32_t) */
#else
	void 										*pIntStartAddress; 	/*!< Internal start address */
#endif /* defined(__ADSPSC598_FAMILY__) && defined(__ADSPCORTEXA55__) */
} ADI_EMDMA_DESC_STANDARD_CIRCULAR;
#pragma pack()

/** @ingroup EMDMA_Manager
 * Structure to handle descriptor list for Scatter/Gather EMDMA transfer mode with circular buffering disabled
 */
#pragma pack(4)
typedef struct _ADI_EMDMA_DESC_SCATTER_GATHER
{
	struct _ADI_EMDMA_DESC_SCATTER_GATHER  	*pNext;					/*!< Next descriptor in the chain, should be NULL for single descriptor chain */
	uint32_t								 Chnptr;				/*!< Write this field for all the descriptors either with the macro ADI_EMDMA_UNMASK_DMA_INT
																		(interrupt/callback generated for each descriptor) or with zero (interrupt/callback generated for the last descriptor only). To avoid
																		race condition for short EMDMA work units, it is recommended to generate interrupt for the last descriptor only. */
#if defined(__ADSPSC598_FAMILY__) && defined(__ADSPCORTEXA55__)
	uint32_t		 						 pTaplistPointer;		/*!< Start address of the tap list(uint32_t) */
#else
	void		 							*pTaplistPointer;		/*!< Start address of the tap list */
#endif /* defined(__ADSPSC598_FAMILY__) && defined(__ADSPCORTEXA55__) */
	uint32_t								 TapCount;				/*!< Number of elements in the tap list */
	int32_t	        					     ExtModify;				/*!< External modifier in 32 bits */
#if defined(__ADSPSC598_FAMILY__) && defined(__ADSPCORTEXA55__)
	uint32_t		 						 pExtStartAddress;		/*!< External start address(uint32_t) */
#else
	void		 							*pExtStartAddress;		/*!< External start address */
#endif /* defined(__ADSPSC598_FAMILY__) && defined(__ADSPCORTEXA55__) */
	uint32_t								 Count;					/*!< Total number of 32 bit words to be transferred */
	int32_t									 IntModify; 			/*!< Internal modifier in 32 bits */
#if defined(__ADSPSC598_FAMILY__) && defined(__ADSPCORTEXA55__)
	uint32_t 								 pIntStartAddress; 		/*!< Internal start address(uint32_t) */
#else
	void 									*pIntStartAddress; 		/*!< Internal start address */
#endif /* defined(__ADSPSC598_FAMILY__) && defined(__ADSPCORTEXA55__) */
} ADI_EMDMA_DESC_SCATTER_GATHER;
#pragma pack()

/** @ingroup EMDMA_Manager
 * Structure to handle descriptor list for Scatter/Gather EMDMA transfer mode with circular buffering enabled
 */
#pragma pack(4)
typedef struct _ADI_EMDMA_DESC_SCATTER_GATHER_CIRCULAR
{
	struct _ADI_EMDMA_DESC_SCATTER_GATHER_CIRCULAR  	*pNext;				/*!< Next descriptor in the chain, should be NULL for single descriptor chain */
	uint32_t										 Chnptr;			/*!< Write this field for all the descriptors either with the macro ADI_EMDMA_UNMASK_DMA_INT
																		(interrupt/callback generated for each descriptor) or with zero (interrupt/callback generated for the last descriptor only). To avoid
																		race condition for short EMDMA work units, it is recommended to generate interrupt for the last descriptor only. */
	uint32_t	 									 ExtLength;			/*!< Length of the external circular buffer in 32 bits */
#if defined(__ADSPSC598_FAMILY__) && defined(__ADSPCORTEXA55__)
	uint32_t										 pExtBaseAddress;	/*!< Base address of the external circular buffer in 32 bits(uint32_t) */
	uint32_t		 								 pTaplistPointer;	/*!< Start address of the tap list(uint32_t) */
#else
	void										 	*pExtBaseAddress;	/*!< Base address of the external circular buffer in 32 bits */
	void		 									*pTaplistPointer;	/*!< Start address of the tap list */
#endif /* defined(__ADSPSC598_FAMILY__) && defined(__ADSPCORTEXA55__) */
	uint32_t										 TapCount;			/*!< Number of elements in the tap list */	
	int32_t	        					  			 ExtModify;			/*!< External modifier in 32 bits */
#if defined(__ADSPSC598_FAMILY__) && defined(__ADSPCORTEXA55__)
	uint32_t		 								 pExtStartAddress;  /*!< External start address(uint32_t) */
#else
	void		 									*pExtStartAddress;  /*!< External start address */
#endif /* defined(__ADSPSC598_FAMILY__) && defined(__ADSPCORTEXA55__) */

	uint32_t										 Count;		    	/*!< Total number of 32 bit words to be transferred */
	int32_t											 IntModify; 	    /*!< Internal modifier in 32 bits */
#if defined(__ADSPSC598_FAMILY__) && defined(__ADSPCORTEXA55__)
	uint32_t 									     pIntStartAddress;  /*!< Internal start address(uint32_t) */
#else
	void 											*pIntStartAddress;  /*!< Internal start address */
#endif /* defined(__ADSPSC598_FAMILY__) && defined(__ADSPCORTEXA55__) */

} ADI_EMDMA_DESC_SCATTER_GATHER_CIRCULAR;
#pragma pack()

/** @ingroup EMDMA_Manager
 * Structure to handle descriptor list for Delay-Line transfer mode with/without circular buffering enabled. 
 */
#pragma pack(4)
typedef struct _ADI_EMDMA_DESC_DELAYLINE
{
	struct _ADI_EMDMA_DESC_DELAYLINE  	*pNext;					/*!< Next descriptor in the chain, should be NULL for single descriptor chain */
	uint32_t							 Chnptr;				/*!< Write this field for all the descriptors either with the macro ADI_EMDMA_UNMASK_DMA_INT
																		(interrupt/callback generated for each descriptor) or with zero (interrupt/callback generated for the last descriptor only). To avoid
																		race condition for short EMDMA work units, it is recommended to generate interrupt for the last descriptor only. */
#if defined(__ADSPSC598_FAMILY__) && defined(__ADSPCORTEXA55__)
	uint32_t		 					 pTaplistPointer;		/*!< Start address of the tap list(uint32_t) */
#else
	void		 						*pTaplistPointer;		/*!< Start address of the tap list */
#endif /* defined(__ADSPSC598_FAMILY__) && defined(__ADSPCORTEXA55__) */
	uint32_t							 TapCount;				/*!< Number of elements in the tap list */	
	int32_t	        				     ExtReadModify;			/*!< External read modifier in 32 bits */
	uint32_t							 ReadCount;				/*!< Total number of 32 bit words to be read from external memory for each tap */
#if defined(__ADSPSC598_FAMILY__) && defined(__ADSPCORTEXA55__)
	uint32_t 							 pIntReadStartAddress;  /*!< Internal read start address(uint32_t) */
#else
	void 								*pIntReadStartAddress;  /*!< Internal read start address */
#endif /* defined(__ADSPSC598_FAMILY__) && defined(__ADSPCORTEXA55__) */
	uint32_t	 						 ExtLength;				/*!< Length of the external circular buffer in 32 bits */
#if defined(__ADSPSC598_FAMILY__) && defined(__ADSPCORTEXA55__)
	uint32_t							 pExtBaseAddress;		/*!< Base address of the external circular buffer in 32 bits(uint32_t) */
#else
	void							    *pExtBaseAddress;		/*!< Base address of the external circular buffer in 32 bits */
#endif /* defined(__ADSPSC598_FAMILY__) && defined(__ADSPCORTEXA55__) */
	int32_t	        				     ExtWriteModify;		/*!< External write modifier in 32 bits */
#if defined(__ADSPSC598_FAMILY__) && defined(__ADSPCORTEXA55__)
	uint32_t		 					 pExtWriteStartAddress;	/*!< External write start address(uint32_t) */
#else
	void		 						*pExtWriteStartAddress;	/*!< External write start address */
#endif /* defined(__ADSPSC598_FAMILY__) && defined(__ADSPCORTEXA55__) */
	uint32_t							 WriteCount;			/*!< Total number of 32 bit words to be written to external memory*/
	int32_t								 IntModify; 			/*!< Internal write/read modifier in 32 bits */
#if defined(__ADSPSC598_FAMILY__) && defined(__ADSPCORTEXA55__)
	uint32_t 							 pIntWriteStartAddress; /*!< Internal write start address(uint32_t) */
#else
	void 								*pIntWriteStartAddress; /*!< Internal write start address */
#endif /* defined(__ADSPSC598_FAMILY__) && defined(__ADSPCORTEXA55__) */
} ADI_EMDMA_DESC_DELAYLINE;
#pragma pack()


/*=======  P U B L I C   P R O T O T Y P E S  ========*/
/*            (globally-scoped functions)             */

#ifdef __cplusplus
extern "C" {
#endif

/* Opens an EMDMA Stream for use */
ADI_EMDMA_RESULT adi_emdma_Open(
    ADI_EMDMA_STREAM_ID              eStreamID,
    void                            *pStreamMemory,
    ADI_EMDMA_STREAM_HANDLE          *phStream,
    ADI_CALLBACK                    pfCallback,
    void                            *pCBParam);

/* Closes an EMDMA Stream */
ADI_EMDMA_RESULT adi_emdma_Close(
    ADI_EMDMA_STREAM_HANDLE           hStream,
    bool                            bWaitFlag);

/* Gets the current data copy status of an EMDMA Stream  */
ADI_EMDMA_RESULT adi_emdma_IsCopyInProgress(
    ADI_EMDMA_STREAM_HANDLE           hStream,
    bool                              *pbCopyInProgress);

/* Disables/terminates EMDMA copy that is currently in progress */
ADI_EMDMA_RESULT adi_emdma_Disable(
    ADI_EMDMA_STREAM_HANDLE           hStream);

/* Configures the descriptor reuse mode for an EMDMA Stream */
ADI_EMDMA_RESULT adi_emdma_SetDescReuseMode(
    ADI_EMDMA_STREAM_HANDLE           hStream,
	ADI_EMDMA_DESC_REUSE_MODE		  eDescReuseMode);

/* Performs EMDMA copy operation as per the supplied descriptor list in various modes */
ADI_EMDMA_RESULT adi_emdma_Copy(
    ADI_EMDMA_STREAM_HANDLE           		hStream,
	ADI_EMDMA_TRANSFER_MODE					TransferMode,
	void						  		   *pDescList,
	uint32_t								DescNo,
	bool									bExtCircBuffEnable,
	bool									bWritebackEnable
);

		
#ifdef __cplusplus
}
#endif

#endif /*!defined(_LANGUAGE_ASM) */

#ifdef _MISRA_RULES
#pragma diag(pop)
#endif /* _MISRA_RULES */

#endif  /* __ADI_NEW_MDMA_SC57X_H__ */

/*****/

