/*********************************************************************************

Copyright(c) 2019-21 Analog Devices, Inc. All Rights Reserved.

This software is proprietary.  By using this software you agree
to the terms of the associated Analog Devices License Agreement.

*********************************************************************************/
/** @addtogroup EMDMA_Manager EMDMA Manager
 *  @{

EMDMA Manager

- EMDMA open/close/query/disable APIs
    - adi_emdma_Open()
    - adi_emdma_Close()
    - adi_emdma_IsCopyInProgress()
	- adi_emdma_Disable()
	
- Memory copy API
    - adi_emdma_Copy()
	
- Descriptor reuse mode set API
	- adi_emdma_SetDescReuseMode()

Following are some common important points to be noted when using EMDMA Manager APIs: 

- The EMDMA Manager code uses "internal" and "external" keywords because conventionally the terms such as DMA direction, tap list, and circular buffering are associated w.r.t. "external" memory. 
  However, all types of memories (L1/L2/L3) can be used as "internal" or "external" when calling EMDMA Manager APIs.
 
- The EMDMA Manager supports only chaining/descriptor based operations. To perform one-shot EMDMA copy operations, only one descriptor can be used. 
	
- When the processor cache is enabled, the EMDMA Manager takes care of cache flush operation of the descriptors internally. However, the cache flush of the source buffer and cache flush/invalidation of the 
  destination buffer should be done by the application external to the driver before calling adi_emdma_Copy() API.

- When the processor's cache is enabled, the receive data buffers must be aligned on cache line boundaries. If they are not then any data access by 
  the core may pull a buffer into the cache while the DMA is operating on the data buffer. Both the core and the DMA are masters on the bus. The core 
  will be performing cache operations. The DMA engine will not be. So it is important for the application to align data that will be operated on by the
  DMA and to separate this data from other application data.                                       

 */

/*!
 * @file      adi_emdma_2156x.c
 * @brief     EMDMA Manager for ADSP-2156x processor family.
 * @version:  $Revision: 68342 $
 * @date:     $Date: 2022-09-20 06:27:14 -0400 (Tue, 20 Sep 2022) $
 *
 * @details
 *            This is the the primary source file for EMDMA Manager Service
 *            for ADSP-2156x processor.
 *
 */
 
#ifdef _MISRA_RULES
#pragma diag(push)
#pragma diag(suppress:misra_rule_8_5:" ")
#pragma diag(suppress:misra_rule_10_1_a:" ")
#pragma diag(suppress:misra_rule_10_3:" ")
#pragma diag(suppress:misra_rule_17_4:" ")
#pragma diag(suppress:misra_rule_12_7:" ")
#pragma diag(suppress:misra_rule_14_7:" ")
#pragma diag(suppress:misra_rule_8_7:"Objects shall be defined at block scope if they are only accessed from within a single function")
#endif /* _MISRA_RULES */

/*=============  I N C L U D E S   =============*/
#include "adi_emdma_def_2156x.h"

#if defined(__ADSP215xx__)
#if defined(__ADSPARM__)
#include <runtime/cache/adi_cache.h>
#else
/* SHARC core */
#include <sys/cache.h>
#endif
#endif

/*==============  D E F I N E S  ===============*/

/*=============  L O C A L    F U N C T I O N    P R O T O T Y P E S =============*/
/* Resets EMDMA registers */
static void ResetEmdmaRegs (volatile ADI_EMDMA_REGS *pReg);

/* Disables EMDMA transfer */
static void DisableEmdmaXfer (ADI_EMDMA_STREAM *pStream);

/* Prepares TCBs for standard mode */
static void PrepareEmdmaTCBStd(ADI_EMDMA_STREAM_HANDLE hStream, ADI_EMDMA_DESC_STANDARD *pDescList, uint32_t DescNo, uint32_t DescOffset);

/* Prepares TCBs for standard circular mode */
static void PrepareEmdmaTCBStdCirc(ADI_EMDMA_STREAM_HANDLE hStream, ADI_EMDMA_DESC_STANDARD_CIRCULAR *pDescList, uint32_t DescNo, uint32_t DescOffset);

/* Prepares TCBs for scatter/gather mode */
static void PrepareEmdmaTCBSG(ADI_EMDMA_STREAM_HANDLE hStream, ADI_EMDMA_DESC_SCATTER_GATHER *pDescList, uint32_t DescNo,  uint32_t DescOffset);

/* Prepares TCBs for scatter/gather circular mode */
static void PrepareEmdmaTCBSGCirc(ADI_EMDMA_STREAM_HANDLE hStream, ADI_EMDMA_DESC_SCATTER_GATHER_CIRCULAR *pDescList, uint32_t DescNo, uint32_t DescOffset);

/* Prepares TCBs for delay line mode */
static void PrepareEmdmaTCBDL(ADI_EMDMA_STREAM_HANDLE hStream, ADI_EMDMA_DESC_DELAYLINE *pDescList, uint32_t DescNo, uint32_t DescOffset);

/* EMDMA data interrupt handler */
static void EmdmaDataIntHandler(uint32_t SID, void *pCBParam);


/*=============  D E B U G    F U N C T I O N    P R O T O T Y P E S  =============*/
#if defined(ADI_DEBUG)
/* Validates EMDMA stream handle */
static ADI_EMDMA_RESULT ValidateEmdmaStreamHandle (ADI_EMDMA_STREAM_HANDLE   hStream);
#endif /* defined(ADI_DEBUG) */
/*=============  C O D E  =============*/

/*=============  L O C A L   F U N C T I O N S =============*/
/*
 * Resets EMDMA Stream registers.
 *
 * Parameters
 *  - [in]  pReg   Pointer to EMDMA Stream registers to reset.
 *
 * Returns
 *  - None
 *
 */
static void ResetEmdmaRegs (volatile ADI_EMDMA_REGS *pReg)
{
    pReg->Index1			= 0;
	pReg->Modify1			= 0;
	pReg->Count1			= 0;
	pReg->Index0			= 0;
	pReg->Modify0			= 0;
	pReg->Count0			= 0;
	pReg->ChainPointer		= 0;
	pReg->Base1				= 0;
	pReg->TapListPointer	= 0;
	pReg->BuffLen1 			= 0;
	pReg->TapListCount 		= 0;
}

/*
 * Disables EMDMA transfer.
 *
 * Parameters
 *  - [in]  pStream  EMDMA Stream to work on.
 *
 * Returns
 *  - None
 *
 */
static void DisableEmdmaXfer (ADI_EMDMA_STREAM *pStream)
{
    /* Clear the control register */
	*pStream->pStreamInfo->pControlReg = 0;

    /* Set EMDMA transfer status as stopped */
	pStream->bIsXferInProgress = false;

    /* Clear EMDMA data transfer type */
	pStream->eXferMode  = ADI_EMDMA_XFER_NONE;
}

/*
 * Prepares TCBs for standard EMDMA non circular modes.
 *
 * Parameters
 *  - [in]  pWorkingDesc  Descriptor list 
 *	- [in]	DescNo		  Number of descriptors 
 *  - [in]  DescOffset	  Offset needed to initialize chain pointer register 
 *
 * Returns
 *  - None
 *
 */
static void PrepareEmdmaTCBStd(ADI_EMDMA_STREAM_HANDLE hStream, ADI_EMDMA_DESC_STANDARD *pDescList, uint32_t DescNo, uint32_t DescOffset)
{
	int i;

	/* To store chain pointer register value */
	uint32_t ChnptrValue;

	/* EMDMA Stream to work on */
    ADI_EMDMA_STREAM	*pStream = (ADI_EMDMA_STREAM *) hStream;
	
	/* Working descriptor */
	ADI_EMDMA_DESC_STANDARD *pWorkingDesc = pDescList;
	
	/* Loop DescNo times */
	for(i=0; i<DescNo; i++ )
	{
#if defined(__ADSPSC598_FAMILY__) && defined(__ADSPCORTEXA55__)
		/* Prepare chain pointer register value */
		ChnptrValue=(uint32_t)(uint64_t)pWorkingDesc->pNext;
#else
		/* Prepare chain pointer register value */
		ChnptrValue=(uint32_t)pWorkingDesc->pNext;
#endif /* defined(__ADSPSC598_FAMILY__) && defined(__ADSPCORTEXA55__) */

		/* If the next descriptor exists */
		if(ChnptrValue!=0)
		{
			/* Increment the descriptor address by right offset value */
			ChnptrValue += DescOffset;

			/* Convert the chain pointer address to system address space and shift right by 2 bits */
			ChnptrValue = adi_rtl_internal_to_system_addr(ChnptrValue, 1)>>2;
			
			/* Set the CPDR bit based on the direction of the next descriptor for OTFDIR mode */
			if((pStream->eXferMode  == ADI_EMDMA_XFER_STANDARD_OTFDIR)&&(pWorkingDesc->pNext->Chnptr&BITM_EMDMA_CHNPTR_CPDR))
			{
				ChnptrValue |= BITM_EMDMA_CHNPTR_CPDR;
			}
		}

		if(i==(DescNo-1))
		{
			/* Enable interrupt for the last descriptor */
			ChnptrValue |= BITM_EMDMA_CHNPTR_PCI;
		}

		/* Clear the CPDR bit of the current descriptor for OTFDIR mode as it will be decided based on the direction of the next descriptor */
		if(pStream->eXferMode  == ADI_EMDMA_XFER_STANDARD_OTFDIR)
		{
			pWorkingDesc->Chnptr &= ~BITM_EMDMA_CHNPTR_CPDR;
		}

		/* Initialize the chain pointer register */
		pWorkingDesc->Chnptr |= ChnptrValue;

		if(pStream->eDescReuseMode!=ADI_EMDMA_DESC_REUSE_EXT_STARTADDR)
		{
#if defined(__ADSPSC598_FAMILY__) && defined(__ADSPCORTEXA55__)
			/* Convert the external start address to system address space and shift right by 2 bits(uint32_t) */
			pWorkingDesc->pExtStartAddress = ((adi_rtl_internal_to_system_addr(pWorkingDesc->pExtStartAddress, 1))>>2);
#else
			/* Convert the external start address to system address space and shift right by 2 bits */
			pWorkingDesc->pExtStartAddress = (void*)(adi_rtl_internal_to_system_addr((uint32_t)pWorkingDesc->pExtStartAddress, 1)>>2);
#endif /* defined(__ADSPSC598_FAMILY__) && defined(__ADSPCORTEXA55__) */

		}

#if defined(__ADSPSC598_FAMILY__) && defined(__ADSPCORTEXA55__)
		/* Convert the internal start address to system address space and shift right by 2 bits(uint32_t) */
		pWorkingDesc->pIntStartAddress = ((adi_rtl_internal_to_system_addr(pWorkingDesc->pIntStartAddress, 1))>>2);
#else
		/* Convert the internal start address to system address space and shift right by 2 bits */
		pWorkingDesc->pIntStartAddress = (void*)(adi_rtl_internal_to_system_addr((uint32_t)pWorkingDesc->pIntStartAddress, 1)>>2);
#endif /* defined(__ADSPSC598_FAMILY__) && defined(__ADSPCORTEXA55__) */

		/* IF (DMA cache management enabled) */
		#if (__ADI_EMDMA_CACHE_MANAGEMENT == 1)

				/* Flush the descriptor */
				flush_data_buffer((void *)pWorkingDesc,
								(void *)((char *)pWorkingDesc + sizeof(ADI_EMDMA_DESC_STANDARD)),
								ADI_FLUSH_DATA_INV);
		#endif
	
		/* Moving to the next descriptor */
		pWorkingDesc = pWorkingDesc->pNext;
	}
}

/*
 * Prepares TCBs for standard circular EMDMA modes.
 *
 * Parameters
 *  - [in]  pWorkingDesc  Descriptor list 
 *	- [in]	DescNo		  Number of descriptors 
 *  - [in]  DescOffset	  Offset needed to initialize chain pointer register 
 *
 * Returns
 *  - None
 *
 */
static void PrepareEmdmaTCBStdCirc(ADI_EMDMA_STREAM_HANDLE hStream, ADI_EMDMA_DESC_STANDARD_CIRCULAR *pDescList, uint32_t DescNo, uint32_t DescOffset)
{
	int i;

	/* To store chain pointer register value */
	uint32_t ChnptrValue;

	/* EMDMA Stream to work on */
    ADI_EMDMA_STREAM	*pStream = (ADI_EMDMA_STREAM *) hStream;
	
	/* Working descriptor */
	ADI_EMDMA_DESC_STANDARD_CIRCULAR *pWorkingDesc = pDescList;
	
	/* Loop DescNo times */
	for(i=0; i<DescNo; i++ )
	{

#if defined(__ADSPSC598_FAMILY__) && defined(__ADSPCORTEXA55__)
		/* Prepare chain pointer register value */
		ChnptrValue=(uint32_t)(uint64_t)pWorkingDesc->pNext;
#else
		/* Prepare chain pointer register value */
		ChnptrValue=(uint32_t)pWorkingDesc->pNext;
#endif /* defined(__ADSPSC598_FAMILY__) && defined(__ADSPCORTEXA55__) */


		if(ChnptrValue!=0)
		{
			/* Increment the descriptor address by right offset value */
			ChnptrValue += DescOffset;

			/* Convert the chain pointer address to system address space and shift right by 2 bits */
			ChnptrValue = adi_rtl_internal_to_system_addr(ChnptrValue, 1)>>2;
		}

		if(i==(DescNo-1))
		{
			/*Enable interrupt for the last descriptor */
			ChnptrValue |= BITM_EMDMA_CHNPTR_PCI;
		}

		/* Initialize the chain pointer register */
		pWorkingDesc->Chnptr |= ChnptrValue;

#if defined(__ADSPSC598_FAMILY__) && defined(__ADSPCORTEXA55__)
		/* Convert the external base address to system address space and shift right by 2 bits(uint32_t) */
		pWorkingDesc->pExtBaseAddress = ((adi_rtl_internal_to_system_addr(pWorkingDesc->pExtBaseAddress, 1))>>2);
#else
		/* Convert the external base address to system address space and shift right by 2 bits */
		pWorkingDesc->pExtBaseAddress = (void*)(adi_rtl_internal_to_system_addr((uint32_t)pWorkingDesc->pExtBaseAddress, 1)>>2);
#endif /* defined(__ADSPSC598_FAMILY__) && defined(__ADSPCORTEXA55__) */



		if(pStream->eDescReuseMode!=ADI_EMDMA_DESC_REUSE_EXT_STARTADDR)
		{
#if defined(__ADSPSC598_FAMILY__) && defined(__ADSPCORTEXA55__)
			/* Convert the external start address to system address space and shift right by 2 bits(uint32_t) */
			pWorkingDesc->pExtStartAddress = ((adi_rtl_internal_to_system_addr(pWorkingDesc->pExtStartAddress, 1))>>2);
#else
			/* Convert the external start address to system address space and shift right by 2 bits */
			pWorkingDesc->pExtStartAddress = (void*)(adi_rtl_internal_to_system_addr((uint32_t)pWorkingDesc->pExtStartAddress, 1)>>2);
#endif /* defined(__ADSPSC598_FAMILY__) && defined(__ADSPCORTEXA55__) */

		}

#if defined(__ADSPSC598_FAMILY__) && defined(__ADSPCORTEXA55__)
		/* Convert the internal start address to system address space and shift right by 2 bits(uint32_t) */
		pWorkingDesc->pIntStartAddress = ((adi_rtl_internal_to_system_addr(pWorkingDesc->pIntStartAddress, 1))>>2);
#else
		/* Convert the internal start address to system address space and shift right by 2 bits */
		pWorkingDesc->pIntStartAddress = (void*)(adi_rtl_internal_to_system_addr((uint32_t)pWorkingDesc->pIntStartAddress, 1)>>2);
#endif /* defined(__ADSPSC598_FAMILY__) && defined(__ADSPCORTEXA55__) */


		/* IF (DMA cache management enabled) */
		#if (__ADI_EMDMA_CACHE_MANAGEMENT == 1)

				/* Flush the descriptor */
				flush_data_buffer((void *)pWorkingDesc,
								(void *)((char *)pWorkingDesc + sizeof(ADI_EMDMA_DESC_STANDARD)),
								ADI_FLUSH_DATA_INV);
		#endif

		/* Moving to the next descriptor */
		pWorkingDesc = pWorkingDesc->pNext;
	}
}

/*
 * Prepares TCBs for scatter/gather non circular EMDMA modes.
 *
 * Parameters
 *  - [in]  pWorkingDesc  Descriptor list 
 *	- [in]	DescNo		  Number of descriptors 
 *  - [in]  DescOffset	  Offset needed to initialize chain pointer register 
 *
 * Returns
 *  - None
 *
 */
static void PrepareEmdmaTCBSG(ADI_EMDMA_STREAM_HANDLE hStream, ADI_EMDMA_DESC_SCATTER_GATHER *pDescList, uint32_t DescNo,  uint32_t DescOffset)
{
	int i;

	/* To store chain pointer register value */
	uint32_t ChnptrValue;
	
	/* Working descriptor */
	ADI_EMDMA_DESC_SCATTER_GATHER *pWorkingDesc = pDescList;
	
	for(i=0; i<DescNo; i++ )
	{
#if defined(__ADSPSC598_FAMILY__) && defined(__ADSPCORTEXA55__)
		/* Prepare chain pointer register value */
		ChnptrValue=(uint32_t)(uint64_t)pWorkingDesc->pNext;
#else
		/* Prepare chain pointer register value */
		ChnptrValue=(uint32_t)pWorkingDesc->pNext;
#endif /* defined(__ADSPSC598_FAMILY__) && defined(__ADSPCORTEXA55__) */


		if(ChnptrValue!=0)
		{
			/* Increment the descriptor address by right offset value */
			ChnptrValue += DescOffset;

			/* Convert the chain pointer address to system address space and shift right by 2 bits */
			ChnptrValue = adi_rtl_internal_to_system_addr(ChnptrValue, 1)>>2;
		}

		if(i==(DescNo-1))
		{
			/*Enable interrupt for the last descriptor only */
			ChnptrValue |= BITM_EMDMA_CHNPTR_PCI;
		}

		/* Initialize the chain pointer register */
		pWorkingDesc->Chnptr |= ChnptrValue;

#if defined(__ADSPSC598_FAMILY__) && defined(__ADSPCORTEXA55__)
		/* Convert the tap list pointer to system address space and shift right by 2 bits(uint32_t) */
		pWorkingDesc->pTaplistPointer = ((adi_rtl_internal_to_system_addr(pWorkingDesc->pTaplistPointer, 1))>>2);

		/* Convert the external start address to system address space and shift right by 2 bits(uint32_t) */
		pWorkingDesc->pExtStartAddress = ((adi_rtl_internal_to_system_addr(pWorkingDesc->pExtStartAddress, 1))>>2);

		/* Convert the internal start address to system address space and shift right by 2 bits(uint32_t) */
		pWorkingDesc->pIntStartAddress = ((adi_rtl_internal_to_system_addr(pWorkingDesc->pIntStartAddress, 1))>>2);
#else
		/* Convert the tap list pointer to system address space and shift right by 2 bits */
		pWorkingDesc->pTaplistPointer = (void*)(adi_rtl_internal_to_system_addr((uint32_t)pWorkingDesc->pTaplistPointer, 1)>>2);

		/* Convert the external start address to system address space and shift right by 2 bits */
		pWorkingDesc->pExtStartAddress = (void*)(adi_rtl_internal_to_system_addr((uint32_t)pWorkingDesc->pExtStartAddress, 1)>>2);

		/* Convert the internal start address to system address space and shift right by 2 bits */
		pWorkingDesc->pIntStartAddress = (void*)(adi_rtl_internal_to_system_addr((uint32_t)pWorkingDesc->pIntStartAddress, 1)>>2);
#endif /* defined(__ADSPSC598_FAMILY__) && defined(__ADSPCORTEXA55__) */

		/* IF (DMA cache management enabled) */
		#if (__ADI_EMDMA_CACHE_MANAGEMENT == 1)

				/* Flush the descriptor */
				flush_data_buffer((void *)pWorkingDesc,
								(void *)((char *)pWorkingDesc + sizeof(ADI_EMDMA_DESC_SCATTER_GATHER)),
								ADI_FLUSH_DATA_INV);
		#endif

		/* Moving to the next descriptor */
		pWorkingDesc = pWorkingDesc->pNext;
	}
}

/*
 * Prepares TCBs for scatter/gather circular EMDMA modes.
 *
 * Parameters
 *  - [in]  pWorkingDesc  Descriptor list 
 *	- [in]	DescNo		  Number of descriptors 
 *  - [in]  DescOffset	  Offset needed to initialize chain pointer register 
 *
 * Returns
 *  - None
 *
 */
static void PrepareEmdmaTCBSGCirc(ADI_EMDMA_STREAM_HANDLE hStream, ADI_EMDMA_DESC_SCATTER_GATHER_CIRCULAR *pDescList, uint32_t DescNo, uint32_t DescOffset)
{
	int i;

	/* To store chain pointer register value */
	uint32_t ChnptrValue;
	
	/* Working descriptor */
	ADI_EMDMA_DESC_SCATTER_GATHER_CIRCULAR *pWorkingDesc = pDescList;
	
	for(i=0; i<DescNo; i++ )
	{
#if defined(__ADSPSC598_FAMILY__) && defined(__ADSPCORTEXA55__)
		/* Prepare chain pointer register value */
		ChnptrValue=(uint32_t)(uint64_t)pWorkingDesc->pNext;
#else
		/* Prepare chain pointer register value */
		ChnptrValue=(uint32_t)pWorkingDesc->pNext;
#endif /* defined(__ADSPSC598_FAMILY__) && defined(__ADSPCORTEXA55__) */


		if(ChnptrValue!=0)
		{
			/* Increment the descriptor address by right offset value */
			ChnptrValue += DescOffset;

			/* Convert the chain pointer address to system address space and shift right by 2 bits */
			ChnptrValue = adi_rtl_internal_to_system_addr(ChnptrValue, 1)>>2;
		}

		if(i==(DescNo-1))
		{
			/*Enable interrupt for the last descriptor only */
			ChnptrValue |= BITM_EMDMA_CHNPTR_PCI;
		}

		/* Initialize the chain pointer register */
		pWorkingDesc->Chnptr |= ChnptrValue;

#if defined(__ADSPSC598_FAMILY__) && defined(__ADSPCORTEXA55__)
		/* Convert the tap list pointer to system address space and shift right by 2 bits(uint32_t) */
		pWorkingDesc->pTaplistPointer = ((adi_rtl_internal_to_system_addr(pWorkingDesc->pTaplistPointer, 1))>>2);

		/* Convert the external base address to system address space and shift right by 2 bits(uint32_t) */
		pWorkingDesc->pExtBaseAddress = ((adi_rtl_internal_to_system_addr(pWorkingDesc->pExtBaseAddress, 1))>>2);

		/* Convert the external start address to system address space and shift right by 2 bits(uint32_t) */
		pWorkingDesc->pExtStartAddress = ((adi_rtl_internal_to_system_addr(pWorkingDesc->pExtStartAddress, 1))>>2);

		/* Convert the internal start address to system address space and shift right by 2 bits(uint32_t) */
		pWorkingDesc->pIntStartAddress = ((adi_rtl_internal_to_system_addr(pWorkingDesc->pIntStartAddress, 1))>>2);
#else
		/* Convert the tap list pointer to system address space and shift right by 2 bits */
		pWorkingDesc->pTaplistPointer = (void*)(adi_rtl_internal_to_system_addr((uint32_t)pWorkingDesc->pTaplistPointer, 1)>>2);

		/* Convert the external base address to system address space and shift right by 2 bits */
		pWorkingDesc->pExtBaseAddress = (void*)(adi_rtl_internal_to_system_addr((uint32_t)pWorkingDesc->pExtBaseAddress, 1)>>2);

		/* Convert the external start address to system address space and shift right by 2 bits */
		pWorkingDesc->pExtStartAddress = (void*)(adi_rtl_internal_to_system_addr((uint32_t)pWorkingDesc->pExtStartAddress, 1)>>2);

		/* Convert the internal start address to system address space and shift right by 2 bits */
		pWorkingDesc->pIntStartAddress = (void*)(adi_rtl_internal_to_system_addr((uint32_t)pWorkingDesc->pIntStartAddress, 1)>>2);
#endif /* defined(__ADSPSC598_FAMILY__) && defined(__ADSPCORTEXA55__) */


		/* IF (DMA cache management enabled) */
		#if (__ADI_EMDMA_CACHE_MANAGEMENT == 1)

				/* Flush the descriptor */
				flush_data_buffer((void *)pWorkingDesc,
								(void *)((char *)pWorkingDesc + sizeof(ADI_EMDMA_DESC_SCATTER_GATHER_CIRCULAR)),
								ADI_FLUSH_DATA_INV);
		#endif

		/* Moving to the next descriptor */
		pWorkingDesc = pWorkingDesc->pNext;
	}
}

/*
 * Prepares TCBs for delay line EMDMA mode.
 *
 * Parameters
 *  - [in]  pWorkingDesc  Descriptor list 
 *	- [in]	DescNo		  Number of descriptors 
 *  - [in]  DescOffset	  Offset needed to initialize chain pointer register 
 *
 * Returns
 *  - None
 *
 */
static void PrepareEmdmaTCBDL(ADI_EMDMA_STREAM_HANDLE hStream, ADI_EMDMA_DESC_DELAYLINE *pDescList, uint32_t DescNo, uint32_t DescOffset)
{
	int i;

	uint32_t ChnptrValue;

	/* EMDMA Stream to work on */
    ADI_EMDMA_STREAM	*pStream = (ADI_EMDMA_STREAM *) hStream;
	
	/* Working descriptor */
	ADI_EMDMA_DESC_DELAYLINE *pWorkingDesc = pDescList;
	
	for(i=0; i<DescNo; i++ )
	{
#if defined(__ADSPSC598_FAMILY__) && defined(__ADSPCORTEXA55__)
		/* Prepare chain pointer register value */
		ChnptrValue=(uint32_t)(uint64_t)pWorkingDesc->pNext;
#else
		/* Prepare chain pointer register value */
		ChnptrValue=(uint32_t)pWorkingDesc->pNext;
#endif /* defined(__ADSPSC598_FAMILY__) && defined(__ADSPCORTEXA55__) */


		if(ChnptrValue!=0)
		{
			/* Increment the descriptor address by right offset value */
			ChnptrValue += DescOffset;

			/* Convert the chain pointer address to system address space and shift right by 2 bits */
			ChnptrValue = adi_rtl_internal_to_system_addr(ChnptrValue, 1)>>2;
		}

		if(i==(DescNo-1))
		{
			/* Enable interrupt for the last descriptor */
			ChnptrValue |= BITM_EMDMA_CHNPTR_PCI;
		}

		/* Initialize the chain pointer register */
		pWorkingDesc->Chnptr |= ChnptrValue;

#if defined(__ADSPSC598_FAMILY__) && defined(__ADSPCORTEXA55__)
		/* Convert the tap list pointer to system address space and shift right by 2 bits(uint32_t) */
		pWorkingDesc->pTaplistPointer = ((adi_rtl_internal_to_system_addr(pWorkingDesc->pTaplistPointer, 1))>>2);

		/* Convert the internal read start address to system address space and shift right by 2 bits(uint32_t) */
		pWorkingDesc->pIntReadStartAddress = ((adi_rtl_internal_to_system_addr(pWorkingDesc->pIntReadStartAddress, 1))>>2);

		/* Convert the external base address to system address space and shift right by 2 bits(uint32_t) */
		pWorkingDesc->pExtBaseAddress = ((adi_rtl_internal_to_system_addr(pWorkingDesc->pExtBaseAddress, 1))>>2);
#else
		/* Convert the tap list pointer to system address space and shift right by 2 bits */
		pWorkingDesc->pTaplistPointer = (void*)(adi_rtl_internal_to_system_addr((uint32_t)pWorkingDesc->pTaplistPointer, 1)>>2);

		/* Convert the internal read start address to system address space and shift right by 2 bits */
		pWorkingDesc->pIntReadStartAddress = (void*)(adi_rtl_internal_to_system_addr((uint32_t)pWorkingDesc->pIntReadStartAddress, 1)>>2);

		/* Convert the external base address to system address space and shift right by 2 bits */
		pWorkingDesc->pExtBaseAddress = (void*)(adi_rtl_internal_to_system_addr((uint32_t)pWorkingDesc->pExtBaseAddress, 1)>>2);
#endif /* defined(__ADSPSC598_FAMILY__) && defined(__ADSPCORTEXA55__) */


		if(pStream->eDescReuseMode!=ADI_EMDMA_DESC_REUSE_EXT_STARTADDR)
		{
#if defined(__ADSPSC598_FAMILY__) && defined(__ADSPCORTEXA55__)
			/* Convert the external write start address to system address space and shift right by 2 bits(uint32_t) */
			pWorkingDesc->pExtWriteStartAddress = ((adi_rtl_internal_to_system_addr(pWorkingDesc->pExtWriteStartAddress, 1))>>2);
#else
			/* Convert the external write start address to system address space and shift right by 2 bits */
			pWorkingDesc->pExtWriteStartAddress = (void*)(adi_rtl_internal_to_system_addr((uint32_t)pWorkingDesc->pExtWriteStartAddress, 1)>>2);
#endif /* defined(__ADSPSC598_FAMILY__) && defined(__ADSPCORTEXA55__) */

		}

#if defined(__ADSPSC598_FAMILY__) && defined(__ADSPCORTEXA55__)
		/* Convert the internal write start address to system address space and shift right by 2 bits(uint32_t) */
		pWorkingDesc->pIntWriteStartAddress = ((adi_rtl_internal_to_system_addr(pWorkingDesc->pIntWriteStartAddress, 1))>>2);
#else
		/* Convert the internal write start address to system address space and shift right by 2 bits */
		pWorkingDesc->pIntWriteStartAddress = (void*)(adi_rtl_internal_to_system_addr((uint32_t)pWorkingDesc->pIntWriteStartAddress, 1)>>2);
#endif /* defined(__ADSPSC598_FAMILY__) && defined(__ADSPCORTEXA55__) */

		/* IF (DMA cache management enabled) */
		#if (__ADI_EMDMA_CACHE_MANAGEMENT == 1)

				/* Flush the descriptor */
				flush_data_buffer((void *)pWorkingDesc,
								(void *)((char *)pWorkingDesc + sizeof(ADI_EMDMA_DESC_DELAYLINE)),
								ADI_FLUSH_DATA_INV);
		#endif

		/* Moving to the next descriptor */
		pWorkingDesc = pWorkingDesc->pNext;
	}
}

/*
 * EMDMA data interrupt handler.
 *
 * Parameters
 *  - [in]   SID         System interrupt ID.
 *  - [in]   pCBParam    Callback parameter provided by Memory DMA manager
 *                       while installing interrupt handler.
 *
 * Returns
 *  None
 *
 */
static void EmdmaDataIntHandler(uint32_t SID, void *pCBParam)
{
	/* EMDMA Stream to work on */
	ADI_EMDMA_STREAM     *pStream = (ADI_EMDMA_STREAM *) pCBParam;

	/* If  we've a valid callback function */
	if (pStream->pfCallback != NULL)
	{
		/* Call the callback function */
		(pStream->pfCallback ) (pStream->pCBParam,
								pStream->eXferMode,
								pStream->pCallbackArg);
	}

	/* If last descriptor or if the interrupt per descriptor is disabled, disable EMDMA */
	if((((ADI_EMDMA_DESC_STANDARD*)pStream->pCallbackArg)->pNext == NULL)||(pStream->bIntPerDesc==false))
	{
		/* Disable EMDMA transfer */
		DisableEmdmaXfer (pStream);
	}
	/* Else continue to next descriptor */
	else
	{
		pStream->pCallbackArg = ((ADI_EMDMA_DESC_STANDARD*)pStream->pCallbackArg)->pNext;
	}
}

/*=============  D E B U G   F U N C T I O N S =============*/
#if defined(ADI_DEBUG)
/* Validates EMDMA Stream handle */
static ADI_EMDMA_RESULT ValidateEmdmaStreamHandle (ADI_EMDMA_STREAM_HANDLE   hStream)
{
    /* Loop variable */
    uint32_t    i;

	/* EMDMA Stream to work on */
    ADI_EMDMA_STREAM     *pStream = (ADI_EMDMA_STREAM *) hStream;
	
	/* Check if the handle is NULL */
	if (pStream ==NULL)
	{
		return (ADI_EMDMA_FAILURE);
	}
	
    /* FOR (All EMDMA Streams) */
    for (i = 0; i < ADI_EMDMA_TOTAL_STREAMS; i++)
    {
        /* IF (Stream handle is valid) */
        if(pStream->pStreamInfo == &EMDMAStreamInfo[i])
        {
			if(pStream->pStreamInfo->pEmdmaStream == (ADI_EMDMA_STREAM*)(&EMDMAStreamInfo[i])->pEmdmaStream)
			{
				/* Stream handle valid, return success */
				return (ADI_EMDMA_SUCCESS);
			}
        }
    }

    /* Channel invalid, return failure */
    return (ADI_EMDMA_FAILURE);
}
#endif /* defined(ADI_DEBUG) */
/*=============  P U B L I C   F U N C T I O N S =============*/
/**
 * @brief Opens an EMDMA stream for use.
 *
 * @param [in]  eStreamID       EMDMA Stream ID to open.
 * @param [in]  pStreamMemory   Pointer to ADI_EMDMA_STREAM_REQ_MEMORY sized memory
 *                              location to handle the DMA stream.
 * @param [out] phStream        Address where the EMDMA stream handle will be stored.
 * @param [in]  pfCallback      Address of application callback function.
 * @param [in]  pCBParam        Parameter passed back to application callback.
 *
 * @return  Status
 *          - #ADI_EMDMA_SUCCESS: Successfully opened an EMDMA Stream.
 *          - #ADI_EMDMA_FAILURE: Supplied pointer parameters are invalid.
 *          - #ADI_EMDMA_IN_USE: Supplied EMDMA Stream ID is already in use.
 *          - #ADI_EMDMA_BAD_STREAM_ID: Supplied EMDMA Stream ID is invalid.
 *          - #ADI_EMDMA_INT_REGISTER_FAILED: Failed to register DMA interrupt handler.
 *			- #ADI_EMDMA_INT_UNREGISTER_FAILED: Failed to unregister DMA interrupt handler.
 */
ADI_EMDMA_RESULT adi_emdma_Open(
    ADI_EMDMA_STREAM_ID              eStreamID,
    void                            *pStreamMemory,
    ADI_EMDMA_STREAM_HANDLE          *phStream,
    ADI_CALLBACK                    pfCallback,
    void                            *pCBParam)
{
    /* Return Code */
    ADI_EMDMA_RESULT      eResult = ADI_EMDMA_SUCCESS;

    /* Pointer to memory allocated to handle EMDMA Stream */
    ADI_EMDMA_STREAM     *pStream = (ADI_EMDMA_STREAM *) pStreamMemory;

    /* Pointer to the processor specific stream information */
    ADI_EMDMA_STREAM_INFO *pStreamInfo = &EMDMAStreamInfo[eStreamID];

   /* If debug information enabled) */
   #if defined(ADI_DEBUG)

	   /* IF (This is not a valid DMA stream) */
	   if (eStreamID >= ADI_DMA_EMDMA_END_COUNT)
	   {
		   /* Report failure as bad stream id */
		   return (ADI_EMDMA_BAD_STREAM_ID);
	   }

	   /* Check if the given pointer parameters are valid */
	   if((pStreamMemory == NULL) || (phStream == NULL))
	   {
		   return (ADI_EMDMA_FAILURE);
	   }
   #endif  /* ADI_DEBUG */



    /* Pointer to the stream info */
    pStream->pStreamInfo = pStreamInfo;

	/* If EMDMA stream already in use) */
	if ((pStream->pStreamInfo->pEmdmaStream != NULL))
	{
		/* Report failure as the stream is already in use */
		return (ADI_EMDMA_IN_USE);
	}

	/* Entering critical region, disable interrupts */
	adi_osal_EnterCriticalRegion();

		/* Save the Memory address allocated to handle this EMDMA stream */
		pStream->pStreamInfo->pEmdmaStream=pStream;
		
		/* Data transfer is not in progress */
		pStream->bIsXferInProgress  = false;

		/* Reset the DMA registers  */
		*pStream->pStreamInfo->pControlReg = 0;
		ResetEmdmaRegs(pStream->pStreamInfo->pReg);
		
	/* Re-enable interrupts */
	adi_osal_ExitCriticalRegion();	

    /* Save callback parameters */
	pStream->pfCallback = pfCallback;
	pStream->pCBParam   = pCBParam;
 
	/*Reset callback argument  */
	pStream->pCallbackArg = NULL;

	/* Reset the descriptor reuse mode to ADI_EMDMA_DESC_REUSE_NONE */
	pStream->eDescReuseMode = ADI_EMDMA_DESC_REUSE_NONE;

	/* Clear DMA Channel data transfer type */
	pStream->eXferMode  = ADI_EMDMA_XFER_NONE;

	/* Keep the interrupt per descriptor disabled by default*/
	pStream->bIntPerDesc = false;

#if !defined(__ADSPARM__)
	/* Configure EMDMA interrupt as edge sensitive */
	adi_sec_EnableEdgeSense((uint32_t)pStream->pStreamInfo->DataIntSID, true);

#else
#if defined(__ADSPCORTEXA55__)
	adi_gic_ConfigInt((uint32_t)pStream->pStreamInfo->DataIntSID,ADI_GIC_INT_EDGE_SENSITIVE);
#else
	adi_gic_ConfigInt((uint32_t)pStream->pStreamInfo->DataIntSID,ADI_GIC_INT_EDGE_SENSITIVE, ADI_GIC_INT_HANDLING_MODEL_N_N);
#endif /* defined(__ADSPCORTEXA55__) */
#endif
	/* Install DMA done Interrupt handler */
	if (adi_int_InstallHandler ((uint32_t)pStream->pStreamInfo->DataIntSID, EmdmaDataIntHandler, pStream, true) != ADI_INT_SUCCESS)
	{
		/* Clear the memory address allocated to handle the EMDMA stream */
		pStream->pStreamInfo->pEmdmaStream=NULL;
		
		/* Failed to install DMA data interrupt handler */
		return (ADI_EMDMA_INT_REGISTER_FAILED);
	}

	/* Pass a valid handle to this MDMA Stream */
	*phStream = (ADI_EMDMA_STREAM_HANDLE)pStream;

	/* Return */
	return (eResult);
}

/**
 * @brief Closes an EMDMA stream.
 *
 * @param [in]  hStream     EMDMA Stream handle to be closed.
 * @param [in]  bWaitFlag   Wait for transfers to complete flag
 *                          (TRUE to wait, FALSE to force close).
 *
 * @return  Status
 *          - #ADI_EMDMA_SUCCESS: Successfully closed an EMDMA Stream.
 *          - #ADI_EMDMA_NOT_OPEN: The EMDMA Stream is not open yet.
 *          - #ADI_EMDMA_BAD_HANDLE: Supplied EMDMA Stream handle is invalid.
 *          - #ADI_EMDMA_INT_UNREGISTER_FAILED: Failed to unregister interrupt.
 *
 */
ADI_EMDMA_RESULT adi_emdma_Close(
    ADI_EMDMA_STREAM_HANDLE           hStream,
    bool                            bWaitFlag)
{
    /* Return Code */
    ADI_EMDMA_RESULT  eResult = ADI_EMDMA_SUCCESS;

    /* EMDMA Stream to work on */
    ADI_EMDMA_STREAM     *pStream = (ADI_EMDMA_STREAM *) hStream;

/* If debug information enabled) */
#if defined(ADI_DEBUG)

	/* If EMDMA Stream Handle is invalid */
	if (ValidateEmdmaStreamHandle (hStream) != ADI_EMDMA_SUCCESS)
	{
		return (ADI_EMDMA_BAD_HANDLE);
	}
	
	/* If EMDMA Stream is not open */
	if (pStream->pStreamInfo->pEmdmaStream == NULL)
	{
		return (ADI_EMDMA_NOT_OPEN);
	}

#endif /* ADI_DEBUG */

    /* If wait for DMA to complete transfer */
    if (bWaitFlag)
    {
        /* wait until DMA is idle */
        while (((*pStream->pStreamInfo->pControlReg) & (BITM_EMDMA_CTL_DMAS0|BITM_EMDMA_CTL_DMAS1)) != 0)
        {
        }
    }

 	/* Un-register DMA data interrupt handler */
	if (adi_int_UninstallHandler ((uint32_t)pStream->pStreamInfo->DataIntSID) != ADI_INT_SUCCESS)
	{
		/* Return failure as we failed to un-register the interrupt */
		return (ADI_EMDMA_INT_UNREGISTER_FAILED);
	}

   /* Entering critical region, disable interrupts */
    adi_osal_EnterCriticalRegion();

		/* Reset the DMA registers  */
		*pStream->pStreamInfo->pControlReg = 0;
		ResetEmdmaRegs(pStream->pStreamInfo->pReg);

		/* Clear the memory address allocated to handle the EMDMA Stream */
		pStream->pStreamInfo->pEmdmaStream=NULL;

		/* Clear the stream info pointer */
		pStream->pStreamInfo = NULL;

    /* Re-enable interrupts */
    adi_osal_ExitCriticalRegion();

    return (eResult);
}

/**
 * @brief Gets the current data copy status of an EMDMA stream.
 *
 * @param [in]  hStream             EMDMA Stream handle to query.
 * @param [out] pbCopyInProgress    Location to store current data copy status.
 *
 * @return  Status
 *          - #ADI_EMDMA_SUCCESS: Successfully queried EMDMA dataflow status.
 *          - #ADI_EMDMA_NOT_OPEN: The EMDMA Stream is not open yet.
 *          - #ADI_EMDMA_BAD_HANDLE: Supplied EMDMA Stream handle is invalid.
 *
 */
ADI_EMDMA_RESULT adi_emdma_IsCopyInProgress(
    ADI_EMDMA_STREAM_HANDLE           hStream,
    bool                              *pbCopyInProgress)
{
    /* EMDMA Stream to work on */
	ADI_EMDMA_STREAM     *pStream = (ADI_EMDMA_STREAM *) hStream;

	 /* If debug information enabled */
	#if defined(ADI_DEBUG)
		/* Validate stream handle */
		if (ValidateEmdmaStreamHandle (hStream) != ADI_EMDMA_SUCCESS)
		{
			return (ADI_EMDMA_BAD_HANDLE);
		}
		
		/* Check if stream is not open  */
		if (pStream->pStreamInfo->pEmdmaStream == NULL)
		{
			return (ADI_EMDMA_NOT_OPEN);
		}
	#endif /* ADI_DEBUG */


    /* Get the current data transfer status of the DMA channel */
    *pbCopyInProgress = pStream->bIsXferInProgress;

    /* Report success */
    return (ADI_EMDMA_SUCCESS);
}

/**
 * @brief Configures the descriptor reuse mode for an EMDMA Stream.
 *
 * @param [in]  hStream             EMDMA Stream handle to query.
 * @param [in]  eDescReuseMode   	Descriptor reuse mode. 
 *
 * @return  Status\
 *          - #ADI_EMDMA_SUCCESS: Successfully configured the descriptor reuse mode for EMDMA copy operation.
 *          - #ADI_EMDMA_NOT_OPEN: The EMDMA Stream is not open yet.
 *          - #ADI_EMDMA_BAD_HANDLE: Supplied EMDMA Stream handle is invalid.
 *
 */
ADI_EMDMA_RESULT adi_emdma_SetDescReuseMode(
    ADI_EMDMA_STREAM_HANDLE           hStream,
	ADI_EMDMA_DESC_REUSE_MODE		  eDescReuseMode)
{
	/* EMDMA Stream to work on */
	ADI_EMDMA_STREAM     *pStream = (ADI_EMDMA_STREAM *) hStream;

	 /* If debug information enabled */
	#if defined(ADI_DEBUG)
		/* Validate stream handle */
		if (ValidateEmdmaStreamHandle (hStream) != ADI_EMDMA_SUCCESS)
		{
			return (ADI_EMDMA_BAD_HANDLE);
		}
		
		/* Check if stream is not open  */
		if (pStream->pStreamInfo->pEmdmaStream == NULL)
		{
			return (ADI_EMDMA_NOT_OPEN);
		}
	#endif /* ADI_DEBUG */

	/* Set descriptor reuse mode */
	pStream->eDescReuseMode = eDescReuseMode;
	
	 /* Report success */
    return (ADI_EMDMA_SUCCESS);
}

/**
 * @brief Disables/terminates EMDMA copy that is currently in progress.
 *
 * @param [in]  hStream     MDMA stream handle to work on.
 *
 * @return  Status
 *          - #ADI_EMDMA_SUCCESS: Successfully disabled EMDMA dataflow.
 *          - #ADI_EMDMA_NOT_OPEN: The EMDMA Stream is not open yet.
 *          - #ADI_EMDMA_BAD_HANDLE: Supplied EMDMA Stream handle is invalid.
 *
 */
ADI_EMDMA_RESULT adi_emdma_Disable (
    ADI_EMDMA_STREAM_HANDLE           hStream)
{
    /* EMDMA Stream to work on */
	ADI_EMDMA_STREAM     *pStream = (ADI_EMDMA_STREAM *) hStream;

	 /* If debug information enabled */
	#if defined(ADI_DEBUG)
		/* Validate stream handle */
		if (ValidateEmdmaStreamHandle (hStream) != ADI_EMDMA_SUCCESS)
		{
			return (ADI_EMDMA_BAD_HANDLE);
		}
		
		/* Check if stream is not open  */
		if (pStream->pStreamInfo->pEmdmaStream == NULL)
		{
			return (ADI_EMDMA_NOT_OPEN);
		}
	#endif /* ADI_DEBUG */

    /* Disable EMDMA transfer */
    DisableEmdmaXfer (pStream);

    /* Report success */
    return (ADI_EMDMA_SUCCESS);
}
	
/**
 * @brief Copies data between memory in different EMDMA modes.
 *
 * @param [in]  hStream        		EMDMA Stream handle to work on.
 * @param [in]  TransferMode   		EMDMA transfer mode. 
 * @param [in]  pDescList       	Descriptor list start address.
 * @param [in]  DescNo   			Number of descriptors in the list.
 * @param [in]  bExtCircBuffEnable  "true" to enable circular buffering for external buffer.
 * @param [in]	bWritebackEnable	"true" to enable index write back for standard modes. Write back is not supported for scatter/gather modes and is hardwired to "true" for delay line mode. 
 *
 * @return  Status
 *          - #ADI_EMDMA_SUCCESS: Successfully initiated EMDMA transfer.
 *          - #ADI_EMDMA_NOT_OPEN: The EMDMA Stream is not open yet.
 *          - #ADI_EMDMA_BAD_HANDLE: Supplied EMDMA Stream handle is invalid.
 *          - #ADI_EMDMA_ALREADY_RUNNING: Can't submit new buffer when the EMDMA dataflow is already enabled.
 *
 * @note    By default, the API processes the descriptor information and updates the descriptor list with the hardware usable descriptors. In case the complete descriptor list information or the external start address information
 *			need to be reused, passing the hardware updated descriptor list as is may result in unpredictable results. To allow reuse of the complete descriptor list or only the external start address (useful when write back is enabled),
 *			the API adi_emdma_SetDescReuseMode() should be called before the adi_emdma_Copy() API to change the default behavior.

 */
ADI_EMDMA_RESULT adi_emdma_Copy(
    ADI_EMDMA_STREAM_HANDLE           		hStream,
	ADI_EMDMA_TRANSFER_MODE					TransferMode,
	void						  		   *pDescList,
	uint32_t								DescNo,
	bool									bExtCircBuffEnable,
	bool									bWritebackEnable
)
{

	/* Variable to store the control register value */
	uint32_t CtrlValue = BITM_EMDMA_CTL_EN | BITM_EMDMA_CTL_CHEN;

	/* Variable to store the chain pointer register value */
	uint32_t ChnptrValue = 0;

	/* Holds the chain pointer offset for different descriptor modes */
	uint32_t DescOffset = 0;
	
	/* Holds the direction information for the first descriptor to be written into the chain pointer register */
	uint32_t OTFDir =0;

	/* Return code */
	ADI_EMDMA_RESULT	eResult = ADI_EMDMA_SUCCESS;

	/* EMDMA Stream to work on */
    ADI_EMDMA_STREAM	*pStream = (ADI_EMDMA_STREAM *) hStream;

    /* If debug information enabled */
    #if defined(ADI_DEBUG)
    	/* Validate stream handle */
    	if (ValidateEmdmaStreamHandle (hStream) != ADI_EMDMA_SUCCESS)
    	{
    		return (ADI_EMDMA_BAD_HANDLE);
    	}
		
	    /* Check if stream is not open  */
		if (pStream->pStreamInfo->pEmdmaStream == NULL)
		{
			return (ADI_EMDMA_NOT_OPEN);
		}
		
		/* If descriptor list is NULL */
		if(pDescList == NULL)
		{
			return (ADI_EMDMA_INVALID_NULL_POINTER);
		}
    #endif /* ADI_DEBUG */

    /* Check if an EMDMA transfer is already running */
    if (pStream->bIsXferInProgress)
    {
    	return (ADI_EMDMA_ALREADY_RUNNING);
    }

    /* Check if the interrupt is enabled for all the descriptors (BITM_EMDMA_CHNPTR_PCI bit set for the first descriptor) */
	if(pStream->eDescReuseMode!=ADI_EMDMA_DESC_REUSE_ALL)
	{
		if((((ADI_EMDMA_DESC_STANDARD*)pDescList)->Chnptr)&BITM_EMDMA_CHNPTR_PCI)
		{
			pStream->bIntPerDesc = true;
		}
	}
	
    /* Standard mode */
    if((TransferMode == ADI_EMDMA_STANDARD_WRITE_MODE)||(TransferMode == ADI_EMDMA_STANDARD_READ_MODE)||(TransferMode == ADI_EMDMA_STANDARD_OTFDIR_MODE))
    {
    	/* Common code */

		/* Enable write back if needed */
		if(bWritebackEnable)
		{
			CtrlValue |= BITM_EMDMA_CTL_WRBEN;
		}

		/* Enable on the fly direction change if needed, else select the DMA direction based on write/read mode */
    	if(TransferMode == ADI_EMDMA_STANDARD_OTFDIR_MODE)
		{
			CtrlValue |= BITM_EMDMA_CTL_OFCEN;

	    	pStream->eXferMode  = ADI_EMDMA_XFER_STANDARD_OTFDIR;
			
			OTFDir	= (((ADI_EMDMA_DESC_STANDARD*)pDescList)->Chnptr)&BITM_EMDMA_CHNPTR_CPDR;
		}
		else
		{
			if(TransferMode == ADI_EMDMA_STANDARD_WRITE_MODE)
			{
				pStream->eXferMode  = ADI_EMDMA_XFER_STANDARD_WRITE;

				CtrlValue |= BITM_EMDMA_CTL_TRAN;
			}
			else
			{
				pStream->eXferMode  = ADI_EMDMA_XFER_STANDARD_READ;
			}
		}

    	/* Code specific to non circular mode */
    	if (!bExtCircBuffEnable)
    	{
			/* Set descriptor offset to the right value*/
    		DescOffset = ADI_EMDMA_STD_DESC_OFFSET;

    		if(pStream->eDescReuseMode!=ADI_EMDMA_DESC_REUSE_ALL)
    		{
				/* Prepare TCBs */
				PrepareEmdmaTCBStd(pStream, (ADI_EMDMA_DESC_STANDARD*)pDescList, DescNo, DescOffset);
    		}
    	}
    	/* Code specific to circular mode */
    	else
    	{
			/* Set descriptor offset to the right value*/
    		DescOffset = ADI_EMDMA_STD_CIRC_DESC_OFFSET;

    		/* Enable circular buffering */
    		CtrlValue |= BITM_EMDMA_CTL_CBEN;

    		if(pStream->eDescReuseMode!=ADI_EMDMA_DESC_REUSE_ALL)
    		{
    			/* Prepare TCBs */
    			PrepareEmdmaTCBStdCirc(pStream, (ADI_EMDMA_DESC_STANDARD_CIRCULAR*)pDescList, DescNo, DescOffset);
    		}
    	}
    }

    /* Scatter/Gather mode */
    else if((TransferMode == ADI_EMDMA_SCATTER_MODE)||(TransferMode == ADI_EMDMA_GATHER_MODE))
    {
		/* Set DMA direction to "write" for scatter mode and "read" for gather mode */
		if(TransferMode == ADI_EMDMA_SCATTER_MODE)
		{
			pStream->eXferMode  = ADI_EMDMA_XFER_SCATTER;

			CtrlValue |=  BITM_EMDMA_CTL_TRAN;
		}
		else
		{
			pStream->eXferMode  = ADI_EMDMA_XFER_GATHER;
		}

		/* Common code */
		/* Enable tap list mode */
		CtrlValue |= BITM_EMDMA_CTL_TLEN;

    	/* Code specific to non circular mode */
		if (!bExtCircBuffEnable)
		{
			/* Set descriptor offset to the right value*/
			DescOffset = ADI_EMDMA_SG_DESCOFFSET;

			if(pStream->eDescReuseMode!=ADI_EMDMA_DESC_REUSE_ALL)
			{
				/* Prepare TCBs */
				PrepareEmdmaTCBSG(pStream, (ADI_EMDMA_DESC_SCATTER_GATHER*)pDescList, DescNo, DescOffset);
			}
		}
		/* Code specific to circular mode */
		else
		{
			/* Set descriptor offset to the right value*/
			DescOffset = ADI_EMDMA_SG_CIRC_DESCOFFSET;

			/* Enable circular buffering */
			CtrlValue |= BITM_EMDMA_CTL_CBEN;

    		if(pStream->eDescReuseMode!=ADI_EMDMA_DESC_REUSE_ALL)
    		{
    			/* Prepare TCBs */
    			PrepareEmdmaTCBSGCirc(pStream, (ADI_EMDMA_DESC_SCATTER_GATHER_CIRCULAR*)pDescList, DescNo, DescOffset);
    		}
		}
    }

     /* Delay line mode */
   	else if((TransferMode == ADI_EMDMA_DELAYLINE_MODE))
   	{
		/* Set the transfer mode to delay line */
		pStream->eXferMode  = ADI_EMDMA_XFER_DELAY_LINE;
		
		/* Enable delay line mode*/
		CtrlValue |= BITM_EMDMA_CTL_DLEN;

		/* Set descriptor offset to the right value*/
		DescOffset = ADI_EMDMA_DL_DESCOFFSET;

		/* Enable circular mode if needed */
		if (bExtCircBuffEnable)
		{
			CtrlValue |= BITM_EMDMA_CTL_CBEN;
		}

		if(pStream->eDescReuseMode!=ADI_EMDMA_DESC_REUSE_ALL)
		{
			/* Prepare TCBs */
			PrepareEmdmaTCBDL(pStream, (ADI_EMDMA_DESC_DELAYLINE*)pDescList, DescNo, DescOffset);
		}
   	}
	else
	{
		return (ADI_EMDMA_TRANSFER_MODE_INVALID);
	}
	
    /* Prepare chain pointer register value */
#if defined(__ADSPSC598_FAMILY__) && defined(__ADSPCORTEXA55__)
    ChnptrValue = (uint32_t)(uint64_t)pDescList;
#else
    ChnptrValue = (uint32_t)pDescList;
#endif /* defined(__ADSPSC598_FAMILY__) && defined(__ADSPCORTEXA55__) */

    ChnptrValue += DescOffset;
    ChnptrValue = adi_rtl_internal_to_system_addr(ChnptrValue, 1)>>2;
	ChnptrValue |= OTFDir;

    /* Enter critical region */
    adi_osal_EnterCriticalRegion();

     	/* Initialize chain pointer register */
    	pStream->pStreamInfo->pReg->ChainPointer = ChnptrValue;

    	/* Initialize control register and kick off the DMA */
    	*pStream->pStreamInfo->pControlReg = CtrlValue;

		/* Update DMA data transfer status flag */
    	pStream->bIsXferInProgress  = true;

		/* Update the callback argument to the address of the first descriptor*/
		pStream->pCallbackArg = pDescList;
		
    /* Exit critical region */
    adi_osal_ExitCriticalRegion();

   return eResult;
}

#ifdef _MISRA_RULES
#pragma diag(pop)
#endif /* _MISRA_RULES */


/*****/

/*@}*/
