/*********************************************************************************

Copyright(c) 2019-20 Analog Devices, Inc. All Rights Reserved.

This software is proprietary.  By using this software you agree
to the terms of the associated Analog Devices License Agreement.

*********************************************************************************/

/*!
 * @file:    adi_emdma_def_2156x.h
 * @brief:   Private header file for EMDMA Manager.
 * @version: $Revision: 68342 $
 * @date:    $Date: 2022-09-20 06:27:14 -0400 (Tue, 20 Sep 2022) $
 *
 * @details
 *           This is a private header file for the EMDMA Manager service
 *           which contains the API declarations, data and constant definitions
 *	         used in Memory DMA Manager implementation. 
 */

/*! \addtogroup EMDMA_Manager EMDMA Manager 
 *  @{
 */

#ifndef __ADI_EMDMA_DEF_2156X_H__
#define __ADI_EMDMA_DEF_2156X_H__

#ifdef _MISRA_RULES
#pragma diag(push)
#pragma diag(suppress:misra_rule_5_1:"Allow identifiers to be more than 31 characters")
#pragma diag(suppress:misra_rule_5_3:" ")
#pragma diag(suppress:misra_rule_5_4:" ")
#pragma diag(suppress:misra_rule_5_6:"Allow identifier names to be reused")
#pragma diag(suppress:misra_rule_5_7:"Allow identifier names to be reused")
#pragma diag(suppress:misra_rule_18_4:" ")
#endif /* _MISRA_RULES */

/*=============  I N C L U D E S   =============*/

#include "sys/platform.h"
#include <sys/adi_core.h>

/* OSAL includes */
#include <adi_osal.h>

/* Interrupt Manager includes */
#include <services/int/adi_int.h>
#if defined(__ADSP215xx__)
#if defined(__ADSPARM__)
#include <services/int/adi_gic.h>
#else
/* SHARC core */
#include <services/int/adi_sec.h>
#endif
#endif


/* DMA Manager includes */
#include <services/dma/adi_dma.h>

/* memcpy includes */
#include <string.h>

/* Memory size check */
#include <assert.h>

/*==============  D E F I N E S  ===============*/
/* Total number of EMDMA streams */
#define ADI_EMDMA_TOTAL_STREAMS		   (2)

/* Descriptor offset values for chain pointer register loading */
#if defined(__ADSPSC598_FAMILY__) && defined(__ADSPCORTEXA55__)
/* Offsets for A55 are different than SHARC since the structures are compiled in 64 Bit mode */
#define ADI_EMDMA_STD_DESC_OFFSET  	   	(28)
#define ADI_EMDMA_STD_CIRC_DESC_OFFSET  (36)
#define ADI_EMDMA_SG_DESCOFFSET  		(36)
#define ADI_EMDMA_SG_CIRC_DESCOFFSET  	(44)
#define ADI_EMDMA_DL_DESCOFFSET 		(56)
#else
#define ADI_EMDMA_STD_DESC_OFFSET  	   	(24)	
#define ADI_EMDMA_STD_CIRC_DESC_OFFSET  (32)
#define ADI_EMDMA_SG_DESCOFFSET  		(32)
#define ADI_EMDMA_SG_CIRC_DESCOFFSET  	(40)
#define ADI_EMDMA_DL_DESCOFFSET 		(52)
#endif


/* EMDMA stream structure definition for prototypes*/
struct ADI_EMDMA_STREAM;

/* EMDMA data transfer mode */
typedef enum
{
    ADI_EMDMA_XFER_NONE				 =	(0u),
	ADI_EMDMA_XFER_STANDARD_WRITE	 =  ADI_EMDMA_EVENT_STANDARD_WRITE_DESC_PROCESSED,
	ADI_EMDMA_XFER_STANDARD_READ	 =  ADI_EMDMA_EVENT_STANDARD_READ_DESC_PROCESSED,
	ADI_EMDMA_XFER_STANDARD_OTFDIR	 =  ADI_EMDMA_EVENT_STANDARD_OTFDIR_DESC_PROCESSED,
    ADI_EMDMA_XFER_SCATTER   		 =  ADI_EMDMA_EVENT_SCATTER_DESC_PROCESSED,
	ADI_EMDMA_XFER_GATHER	 		 =  ADI_EMDMA_EVENT_GATHER_DESC_PROCESSED,
	ADI_EMDMA_XFER_DELAY_LINE 		 =  ADI_EMDMA_EVENT_DELAY_LINE_DESC_PROCESSED
} ADI_EMDMA_DATA_XFER_MODE;

#pragma pack(4)
/* EMDMA register structure  */
typedef struct 
{
    volatile uint32_t   Index1;    	 	 /* External start address */
    volatile int32_t	Modify1;   		 /* External modifier */
    volatile uint32_t   Count1;			 /* External count*/
    volatile uint32_t   Index0; 		 /* Internal start address */
    volatile int32_t	Modify0;         /* Internal modifier */
    volatile uint32_t   Count0;        	 /* Internal count */
    volatile uint32_t   ChainPointer;    /* Chain pointer to the next TCB */
    volatile uint32_t   Base1;  		 /* External base address */
    volatile uint32_t   TapListPointer;  /* Pointer to the tap list */
    volatile uint32_t   BuffLen1;        /* External buffer length */
	volatile uint32_t  	Padding0; 		 /* Zero Padding */
    volatile uint32_t   TapListCount;    /* Tap list count */
} ADI_EMDMA_REGS;
#pragma pack()

/* Structure to handle processor specific EMDMA stream information  */
typedef struct ADI_EMDMA_STREAM_INFO
{
	uint8_t				     DataIntSID;			/* Data interrupt ID */
	volatile uint32_t		*pControlReg;			/* EMDMA control register */
	volatile ADI_EMDMA_REGS *pReg;					/* Other EMDMA registers */
	struct ADI_EMDMA_STREAM	*pEmdmaStream;			/* Pointer to the corresponding EMDMA stream */
} ADI_EMDMA_STREAM_INFO;

/* Structure to handle EMDMA Stream */
typedef struct ADI_EMDMA_STREAM
{
    bool                  	  bIsXferInProgress;      /* TRUE when an EMDMA transfer is currently in progress */
    ADI_EMDMA_DATA_XFER_MODE  eXferMode; 	          /* EMDMA transfer mode */
	ADI_EMDMA_DESC_REUSE_MODE eDescReuseMode;		  /* Descriptor reuse mode */	
	bool					  bIntPerDesc;			  /* Interrupt enabled per descriptor */
    ADI_CALLBACK           	  pfCallback;             /* Client supplied callback function */
    void                   	 *pCBParam;               /* Client supplied callback parameter */
    void	 				 *pCallbackArg;			  /* Stores the callback argument for different modes */
	ADI_EMDMA_STREAM_INFO	 *pStreamInfo;			  /* Pointer to the processor specific stream information */
} ADI_EMDMA_STREAM;


#if defined(__ADSPSC573_FAMILY__)||defined(__ADSP21569_FAMILY__)||defined(__ADSPSC594_FAMILY__)||defined(__ADSPSC598_FAMILY__)
/* Processor specific EMDMA stream information */
static ADI_EMDMA_STREAM_INFO EMDMAStreamInfo[ADI_EMDMA_TOTAL_STREAMS] =
{
   {INTR_EMDMA0_DONE, (volatile uint32_t *)REG_EMDMA0_CTL,(volatile ADI_EMDMA_REGS *)REG_EMDMA0_INDX1, NULL},
   {INTR_EMDMA1_DONE, (volatile uint32_t *)REG_EMDMA1_CTL,(volatile ADI_EMDMA_REGS *)REG_EMDMA1_INDX1, NULL}
};
#endif /* defined(__ADSPSC573_FAMILY__)||defined(__ADSP21569_FAMILY__)||defined(__ADSPSC594_FAMILY__)||defined(__ADSPSC598_FAMILY__) */


#ifdef _MISRA_RULES
#pragma diag(pop)
#endif /* _MISRA_RULES */

#endif  /* __ADI_EMDMA_DEF_2156X_H__ */
/*****/

/*@}*/
