/*********************************************************************************

Copyright(c) 2019-21 Analog Devices, Inc. All Rights Reserved.

This software is proprietary.  By using this software you agree
to the terms of the associated Analog Devices License Agreement.

*********************************************************************************/
/** @addtogroup MDMA_Manager Memory DMA Manager
 *  @{

Memory DMA Manager

- Memory DMA open/close/query/disable APIs
    - adi_mdma_Open()
    - adi_mdma_Close()
    - adi_mdma_IsCopyInProgress()
	- adi_mdma_Disable()
	
- Memory copy APIs
    - adi_mdma_Copy1D()
	- adi_mdma_Copy1DAuto()
    - adi_mdma_Copy2D()
    - adi_mdma_CopyList()
    - adi_mdma_Copy1DArray()
    - adi_mdma_Copy2DArray()
	
- Memory DMA channel specific APIs
	- adi_mdma_EnableDescListReuse()
	- adi_mdma_GetChannelSID()
	- adi_dma_ConfigTriggerIn()
	- adi_dma_ConfigTriggerOut()
	- adi_dma_UpdateCallback()
	- adi_dma_GetCurrentDesc()
	- adi_dma_GetCurrentAddr()
	- adi_dma_GetCurrentXCount()
	- adi_dma_GetCurrentYCount()
	- adi_mdma_BWLimit()
	- adi_mdma_BWMonitor() 
	- adi_mdma_BWMeasure()

Following are some common important points to be noted when using Memory DMA Manager APIs: 

- It is mandatory to pass both source and destination channel information when using memory copy APIs. 
- Both source and destination channels can be used in the same DMA mode (one-shot/list/array) at a time.
- Both source and destination channel error callbacks are supported.
- Only destination channel DMA done callback is supported. For one shot modes, the callback argument given to the application is 
  start address of the destination buffer. For descriptor list mode, the callback argument is either the completed destination channel descriptor 
  address in case multiple callbacks are enabled (by the macro ADI_MDMA_CFG_LIST_MULTIPLE_CALLBACKS_ENABLE) or the address of the
  first descriptor in the list. For descriptor array modes, the callback argument is always the address of the first descriptor completed.
- Note that only single callback is supported for destination channel transfer done and source/destination error interrupts. The callback argument passed by the 
  driver to the application is the value of the bit field BITM_DMA_STAT_ERRC of the source/DMA channel which resulted in the error condition. The meaning 
  of different values of this bit field is as follows.

	0: Configuration Error
	1: Illegal Write Occurred While Channel Running
	2: Address Alignment Error
	3: Memory Access or Fabric Error
	4: Reserved
	5: Trigger Overrun
	6: Bandwidth Monitor Error
	7: Reserved
- The trigger in/out settings for one-shot modes can be configured by the APIs adi_dma_ConfigTriggerIn() and adi_dma_ConfigTriggerOut().
  For descriptor based modes, adi_dma_ConfigTriggerIn() can be used to wait for a trigger before the first descriptor load.
  For other descriptors, the trigger in setting should be part of the descriptor itself.
- When the processor's cache is enabled, the receive data buffers must be aligned on cache line boundaries. If they are not then any data access by 
  the core may pull a buffer into the cache while the DMA is operating on the data buffer. Both the core and the DMA are masters on the bus. The core 
  will be performing cache operations. The DMA engine will not be. So it is important for the application to align data that will be operated on by the
  DMA and to separate this data from other application data.                                       
 */

/*!
 * @file      adi_mdma_2156x.c
 * @brief     Memory DMA Manager for ADSP-2156x processor family.
 * @version:  $Revision: 68342 $
 * @date:     $Date: 2022-09-20 06:27:14 -0400 (Tue, 20 Sep 2022) $
 *
 * @details
 *            This is the the primary source file for Memory DMA Manager Service
 *            for ADSP-2156x processor.
 *
 */

#ifdef _MISRA_RULES
#pragma diag(push)
#pragma diag(suppress:misra_rule_8_5:" ")
#pragma diag(suppress:misra_rule_10_1_a:" ")
#pragma diag(suppress:misra_rule_10_3:" ")
#pragma diag(suppress:misra_rule_17_4:" ")
#pragma diag(suppress:misra_rule_12_7:" ")
#pragma diag(suppress:misra_rule_14_7:" ")
#pragma diag(suppress:misra_rule_8_7:"Objects shall be defined at block scope if they are only accessed from within a single function")
#endif /* _MISRA_RULES */

/*=============  I N C L U D E S   =============*/
#include "adi_mdma_def_2156x.h"
#include <services/pwr/adi_pwr.h>

#if defined(__ADSP215xx__)
#if defined(__ADSPARM__)
/* Cortex-A5 core */
#include <services/int/adi_gic.h>
#include <runtime/cache/adi_cache.h>
#else
/* SHARC core */
#include <sys/cache.h>
#endif
#endif

/*==============  D E F I N E S  ===============*/
/* DMA Config register value for 1D list descriptor for Source channel */
#define ADI_MDMA_DMA_CFG_SRC_LLIST_1D       (ENUM_DMA_CFG_DSCLIST | ENUM_DMA_CFG_FETCH05  | ENUM_DMA_CFG_EN)
/* DMA Config register value for 2D list descriptor for Source channel */
#define ADI_MDMA_DMA_CFG_SRC_LLIST_2D       (ENUM_DMA_CFG_DSCLIST | ENUM_DMA_CFG_FETCH07 | ENUM_DMA_CFG_EN | ENUM_DMA_CFG_ADDR2D)
/* DMA Config register value for 1D list descriptor for Destination channel */
#define ADI_MDMA_DMA_CFG_DEST_LLIST_1D      (ENUM_DMA_CFG_DSCLIST | ENUM_DMA_CFG_FETCH05 | ENUM_DMA_CFG_WRITE | ENUM_DMA_CFG_EN)
/* DMA Config register value for 2D list descriptor for Destination channel */
#define ADI_MDMA_DMA_CFG_DEST_LLIST_2D      (ENUM_DMA_CFG_DSCLIST | ENUM_DMA_CFG_FETCH07 | ENUM_DMA_CFG_WRITE | ENUM_DMA_CFG_EN | ENUM_DMA_CFG_ADDR2D)
/* DMA Config register value for 1D Array descriptor */
#define ADI_MDMA_DMA_CFG_1D_ARRAY           (ENUM_DMA_CFG_DSCARRAY | ENUM_DMA_CFG_FETCH04 | ENUM_DMA_CFG_EN)
/* DMA Config register value for 2D Array descriptor */
#define ADI_MDMA_DMA_CFG_2D_ARRAY           (ENUM_DMA_CFG_DSCARRAY | ENUM_DMA_CFG_ADDR2D | ENUM_DMA_CFG_FETCH06 | ENUM_DMA_CFG_EN)
/* DMA Config register value for 1D stop flow mode */
#define ADI_MDMA_DMA_CFG_1D_STOP_FLOW       (ENUM_DMA_CFG_EN)
/* DMA Config register value for 2D stop flow mode */
#define ADI_MDMA_DMA_CFG_2D_STOP_FLOW       (ENUM_DMA_CFG_ADDR2D | ENUM_DMA_CFG_EN)

/*=============  L O C A L    F U N C T I O N    P R O T O T Y P E S =============*/
/* Resets DMA registers */
static void ResetDmaRegs (volatile ADI_DMA_REGS *pReg);

/* Disables MDMA transfer */
static void DisableXfer (ADI_MDMA_STREAM *pStream);

/* Gets the value of the Memory Transfer Size from the configuration register */
static  uint32_t MdmaConfigToMemSize (uint32_t  ConfigValue);

 /* DMA data interrupt handler */
 static void DmaDataIntHandler(uint32_t SID, void *pCBParam);

/* DMA error interrupt handler */
static void DmaErrorIntHandler(uint32_t SID, void *pCBParam);

/* Prepares descriptors for List mode */
static void PrepareDescList(
	ADI_DMA_CHANNEL_HANDLE 			hChannel,
	bool 							bIsSrcChnl,
	ADI_DMA_DESC_LIST_MODE 			*pWorkingDesc);
	
/* Prepares descriptors for Array 1D mode */	
static ADI_DMA_RESULT PrepareDescArray1D(
	ADI_DMA_CHANNEL_HANDLE 			hChannel,
	bool 							bIsSrcChnl,
	ADI_DMA_DESC_ARRAY_1D_MODE	   *pWorkingDesc,
	uint16_t                         NumEntries);
	
/* Prepares descriptors for Array 2D mode */
static ADI_DMA_RESULT PrepareDescArray2D(
	ADI_DMA_CHANNEL_HANDLE 			hChannel,
	bool 							bIsSrcChnl,
	ADI_DMA_DESC_ARRAY_2D_MODE	   *pWorkingDesc,
	uint16_t                         NumEntries);	
	
/*=============  D E B U G    F U N C T I O N    P R O T O T Y P E S  =============*/
/* IF (Debug information enabled) */
#if defined(ADI_DEBUG)

/* Validate MDMA stream handle */
static ADI_DMA_RESULT ValidateStreamHandle (ADI_DMA_STREAM_HANDLE   hStream);

/* Validate MDMA channel handle */
static ADI_DMA_RESULT ValidateChannelHandle (ADI_DMA_CHANNEL_HANDLE    hChannel);

/* Validates buffer address and its corresponding DMA register values */
static ADI_DMA_RESULT   ValidateBuffer (
    uint32_t            StartAddress,
    uint32_t            Config,
    uint32_t            Status,
    uint32_t            XCount,
    int32_t             XModify,
    uint32_t            YCount,
    int32_t             YModify);

/* Validate descriptor list */
static ADI_DMA_RESULT ValidateDescList (
	ADI_MDMA_STREAM                 *pStream,
    ADI_DMA_DESC_LIST_MODE          *pDestDesc,
    ADI_DMA_DESC_LIST_MODE          *pSrcDesc);

#endif /* ADI_DEBUG */


/*=============  C O D E  =============*/

/*=============  L O C A L   F U N C T I O N S =============*/

/*
 * Resets DMA channel registers.
 *
 * Parameters
 *  - [in]  pReg   Pointer to DMA Channel registers to reset.
 *
 * Returns
 *  - None
 *
 */
static void ResetDmaRegs (volatile ADI_DMA_REGS *pReg)
{
    /* Reset DMA channel registers */
    pReg->Config        = 0u;
    pReg->NextDescPtr   = 0u;
    pReg->StartAddr     = 0u;
    pReg->XCount        = 0u;
    pReg->XModify       = 0;
    pReg->YCount        = 0u;
    pReg->YModify       = 0;
    pReg->Status        = ADI_DMA_CLEAR_INTERRUPTS; /* Clear any pending interrupts */
    pReg->CurrDescPtr	= 0u;
    pReg->BwLimit		= 0;
    pReg->BwMonitor		= 0;
}

/*
 * Disable MDMA transfer.
 *
 * Parameters
 *  - [in]  pStream  DMA Stream to work on.
 *
 * Returns
 *  - None
 *
 */
static void DisableXfer (ADI_MDMA_STREAM *pStream)
{
    /* Disable the MDMA channel. Both Src and Dst channels.*/
	pStream->DestCh.pChInfo->pReg->Config =0u;
	pStream->SrcCh.pChInfo->pReg->Config =0u;

    /* Clear Interrupt sticky bits */
	pStream->DestCh.pChInfo->pReg->Status =ADI_DMA_CLEAR_INTERRUPTS;
	pStream->SrcCh.pChInfo->pReg->Status =ADI_DMA_CLEAR_INTERRUPTS;

    /* Set DMA data transfer status as stopped */
	pStream->bIsXferInProgress = false;

    /* Clear DMA Channel data transfer type */
	pStream->eXferMode  = ADI_MDMA_XFER_NONE;
}

/* Get the value of the Memory Transfer Size from the configuration register
 *
 * Parameters
 *  - [in]  ConfigValue  This contains the value stored in the configuration register
 *
 *  Returns
 *  - The memory transfer size value
 * */
static  uint32_t MdmaConfigToMemSize (uint32_t  ConfigValue)
{
    uint32_t MemCopySize;
    uint32_t TempConfig;
	
    /* Get the configuration for Memory size transfer */
    TempConfig = (ConfigValue & BITM_DMA_CFG_MSIZE) >> BITP_DMA_CFG_MSIZE;
	
    /* Stores the value of the memory copy size */
    MemCopySize = (uint32_t)(1u << TempConfig);
	
    /*returns the value of memory transfer size*/
    return MemCopySize;
}

/* IF (DMA cache management enabled) */
#if (__ADI_MDMA_CACHE_MANAGEMENT == 1)
/*
 * @brief Invalidates memory region to be accessed by DMA from cache.
 *
 * @param [in]  pStartAddr      Buffer start address to invalidate.
 * @param [in]  XCount          DMA Xcount.
 * @param [in]  XModify         DMA YModify.
 * @param [in]  YCount          DMA Ycount.
 * @param [in]  YModify         DMA YModify.
 * @param [in]  eFlushMode      ADI_FLUSH_DATA_NOINV or ADI_FLUSH_DATA_INV.
 *
 * @return  Status
 *          - #ADI_DMA_SUCCESS:    Success.
 *          - #ADI_DMA_BAD_XCOUNT: Invalid X Count.
 *          - #ADI_DMA_BAD_YCOUNT: Invalid Y Count.
 *
 */
ADI_DMA_RESULT adi_dma_FlushCache (
    void                *pStartAddr,
    uint32_t            XCount,
    int32_t             XModify,
    uint32_t            YCount,
    int32_t             YModify,
    uint32_t            TransferSize,
    int32_t             eFlushMode)
{
    /*XFrame is the size of one X-Frame*/
    int32_t                XFrame = (int32_t)(XCount-1u) * XModify;
    /*Threshold variable is to differentiate between two scenarios*/
    int32_t                Threshold = XFrame + YModify;
    uint8_t                *pFlushStartaddr;
    uint8_t                *pFlushEndaddr;

    /* IF (Debug information enabled) */
#if defined (ADI_DEBUG)

    /* IF XCount has an invalid value */
    if (XCount <= 0)
    {
        return ADI_DMA_BAD_XCOUNT;
    }
    /* IF YCount has an invalid value */
    if (YCount <= 0)
    {
        return ADI_DMA_BAD_YCOUNT;
    }

#endif /* ADI_DEBUG */

#if !defined(__ADSPARM__)
    pStartAddr = (void*)(adi_rtl_system_to_internal_addr((uint32_t)pStartAddr));

    if (((uint32_t)pStartAddr < ADI_SC5XX_SHARC_L1_START_ADDR) || ((uint32_t)pStartAddr > ADI_SC5XX_SHARC_L1_END_ADDR))
#endif /* !defined(__ADSPARM__) */
    {
        if((XModify >= 0) && (YModify >= 0))
        {
            /* Store the Buffer pointer's starting address as cache flushing starting address*/
            pFlushStartaddr = (uint8_t *) pStartAddr;
            /* End address invalidate */
            pFlushEndaddr = (uint8_t *) pStartAddr + (XFrame * (int32_t)YCount) + ((int32_t)(YCount - 1u) * YModify) + (int32_t)(TransferSize - 1u);
        }
        else if((XModify < 0) && (YModify < 0))
        {
            /* Calculating the start address of cache to be flushed */
            pFlushStartaddr = (uint8_t *) pStartAddr + (XFrame * (int32_t)YCount) + ((int32_t)(YCount - 1u) * YModify);
            /* End address invalidate */
            pFlushEndaddr = (uint8_t *) pStartAddr + (int32_t)(TransferSize - 1u);
        }
        else if((XModify < 0) && (YModify >= 0))
        {
            if(Threshold >= 0)
            {
                /* Calculating the start address of cache to be flushed */
                pFlushStartaddr = (uint8_t *) pStartAddr + XFrame;
                /* End address invalidate */
                pFlushEndaddr = (uint8_t *) pStartAddr + (XFrame * (int32_t)(YCount - 1u)) + ((int32_t)(YCount - 1u) * YModify) + (int32_t)(TransferSize - 1u);
            }
            else
            {
                /* Calculating the start address of cache to be flushed */
                pFlushStartaddr = (uint8_t *) pStartAddr + (XFrame * (int32_t)YCount) + ((int32_t)(YCount - 1u) * YModify);
                /* End address invalidate */
                pFlushEndaddr  = (uint8_t *) pStartAddr + (TransferSize - 1u);
            }
        }
        else
        {
            if(Threshold >= 0)
            {
                /* Calculating the start address of cache to be flushed */
                pFlushStartaddr = (uint8_t *) pStartAddr;
                /* End address invalidate */
                pFlushEndaddr = (uint8_t *) pStartAddr + (XFrame * (int32_t)YCount) + ((int32_t)(YCount - 1u) * YModify) + (int32_t)(TransferSize - 1u);
            }
            else
            {
                /* Calculating the start address of cache to be flushed */
                pFlushStartaddr = (uint8_t *) pStartAddr + (XFrame * (int32_t)(YCount - 1u)) + ((int32_t)(YCount - 1u) * YModify);
                /* End address invalidate */
                pFlushEndaddr = (uint8_t *) pStartAddr + XFrame + (int32_t)(TransferSize - 1u);
            }
        }
        /* Flush data buffer */
        flush_data_buffer(pFlushStartaddr, pFlushEndaddr, eFlushMode);
    }

    /* Return */
    return ADI_DMA_SUCCESS;
}
#endif /* __ADI_DMA_CACHE_MANAGEMENT */

/*
 * DMA data interrupt handler.
 *
 * Parameters
 *  - [in]   SID         System interrupt ID.
 *  - [in]   pCBParam    Callback parameter provided by Memory DMA manager
 *                       while installing interrupt handler.
 *
 * Returns
 *  None
 *
 */
static void DmaDataIntHandler(uint32_t SID, void *pCBParam)
{
	/* Get the DMA channel info to which the interrupt belongs */
	ADI_MDMA_CHANNEL    *pDmaChInfo = (ADI_MDMA_CHANNEL *) pCBParam;

	/* Stores the callback event information */
	uint32_t   Event;

    /* Clear the DMA DONE interrupt sticky bit */
	pDmaChInfo->pChInfo->pReg->Status = ENUM_DMA_STAT_IRQDONE;

	/* IF (One shot mode) */
	if(pDmaChInfo->pChInfo->pMdmaStream->eXferMode==ADI_MDMA_XFER_ONE_SHOT)
	{
		Event = ADI_DMA_EVENT_BUFFER_PROCESSED;
	}
	else
	{
		Event =  pDmaChInfo->pChInfo->pMdmaStream->eXferMode;
	}

	/* Only if multiple callbacks in descriptor list is enabled */
#if	(ADI_MDMA_CFG_LIST_MULTIPLE_CALLBACKS_ENABLE == 1)
    /* IF (Transfer mode is list mode) */
    if(pDmaChInfo->pChInfo->pMdmaStream->eXferMode==ADI_MDMA_XFER_LARGE_LIST)
	{
		/* IF (Callback for a descriptor is enabled)*/
      	if(((ADI_DMA_DESC_LIST_MODE*)pDmaChInfo->pCallbackArg)->bCallbackWhenDone)
    	{
        	/* IF (We've a valid callback function) */
            if (pDmaChInfo->pChInfo->pMdmaStream->pfCallback != NULL)
            {
				/* Call the callback function */
                (pDmaChInfo->pChInfo->pMdmaStream->pfCallback ) (pDmaChInfo->pChInfo->pMdmaStream->pCBParam,
                										Event,
                                                        pDmaChInfo->pCallbackArg);

														            	/*If (Last descriptor in the list) */
            	if(((ADI_DMA_DESC_LIST_MODE*)pDmaChInfo->pCallbackArg)->Reserved==0)
        		{
            		/* Disable MDMA transfer */
            		DisableXfer (pDmaChInfo->pChInfo->pMdmaStream);
            	}

				/* Fetch the next descriptor address as callback argument for which callback is enabled */
               	pDmaChInfo->pCallbackArg = ((ADI_DMA_DESC_LIST_MODE*)pDmaChInfo->pCallbackArg)->pNext;
            }
    	}
	}
    else
    {
#endif /* ADI_MDMA_CFG_LIST_MULTIPLE_CALLBACKS_ENABLE */

    	/* IF (We've a valid callback function) */
        if (pDmaChInfo->pChInfo->pMdmaStream->pfCallback != NULL)
        {
			/* Call the callback function */
            (pDmaChInfo->pChInfo->pMdmaStream->pfCallback ) (pDmaChInfo->pChInfo->pMdmaStream->pCBParam,
            										Event,
                                                    pDmaChInfo->pCallbackArg);
        }

	  	/* Disable MDMA transfer */
    	DisableXfer (pDmaChInfo->pChInfo->pMdmaStream);

#if (ADI_MDMA_CFG_LIST_MULTIPLE_CALLBACKS_ENABLE == 1)
    }
#endif /* ADI_MDMA_CFG_LIST_MULTIPLE_CALLBACKS_ENABLE */
}

/*
 * DMA error interrupt handler.
 *
 * Parameters
 *  - [in]   SID         System interrupt ID.
 *  - [in]   pCBParam    Callback parameter provided by Memory DMA manager
 *                       while installing interrupt handler.
 *
 * Returns
 *  None
 *
 */
static void DmaErrorIntHandler(uint32_t SID, void *pCBParam)
{
    /* Get the DMA channel info to which the interrupt belongs */
	ADI_MDMA_CHANNEL    *pDmaChInfo = (ADI_MDMA_CHANNEL *) pCBParam;

    /* IF (We've a valid callback function) */
    if (pDmaChInfo->pChInfo->pMdmaStream->pfCallback != NULL)
    {
#if defined(__ADSPSC598_FAMILY__) && defined(__ADSPCORTEXA55__)
        /* Post callback to report DMA error, pass error cause field value as event specific parameter */
        (pDmaChInfo->pChInfo->pMdmaStream->pfCallback ) (pDmaChInfo->pChInfo->pMdmaStream->pCBParam,
                                                ADI_DMA_EVENT_ERROR_INT,
                                                (void *)(uintptr_t)(pDmaChInfo->pChInfo->pReg->Status & BITM_DMA_STAT_ERRC));
#else
        /* Post callback to report DMA error, pass error cause field value as event specific parameter */
        (pDmaChInfo->pChInfo->pMdmaStream->pfCallback ) (pDmaChInfo->pChInfo->pMdmaStream->pCBParam,
                                                ADI_DMA_EVENT_ERROR_INT,
                                                (void *)(pDmaChInfo->pChInfo->pReg->Status & BITM_DMA_STAT_ERRC));
#endif /* defined(__ADSPSC598_FAMILY__) && defined(__ADSPCORTEXA55__) */

    }

    /* Clear the DMA ERROR Interrupt sticky bit */
    pDmaChInfo->pChInfo->pReg->Status = ENUM_DMA_STAT_IRQERR;
}

/*
 * Prepares descriptors for the MDMA channel in list mode.
 *
 * Parameters
 *  - [in]  hChannel        MDMA channel handle to work on.
 *  - [in]	bIsSrcChnl		'true' for source channel and 'false' for destination channel
 *  - [in]  pWorkingDesc    Pointer to the source/destination descriptor list

 * Returns
 *  None
 *
 */
static void PrepareDescList(
	ADI_DMA_CHANNEL_HANDLE 			hChannel,
	bool 							bIsSrcChnl,
	ADI_DMA_DESC_LIST_MODE 			*pWorkingDesc)
{
	/* DMA Channel to work on */
	ADI_MDMA_CHANNEL    *pDmaChInfo = (ADI_MDMA_CHANNEL *)hChannel;

	/* DMA Stream to work on */
	/* ADI_MDMA_STREAM    *pStream = pDmaChInfo->pChInfo->pMdmaStream; */

	/* To store the pointer to the first descriptor temporarily */
	ADI_DMA_DESC_LIST_MODE *pTemp=pWorkingDesc;

#if	(ADI_MDMA_CFG_LIST_MULTIPLE_CALLBACKS_ENABLE == 1)
	/*To store the pointer to the first callback enabled descriptor */
	ADI_DMA_DESC_LIST_MODE *pTemp1;

	/* Pointer used to create a callback enabled descriptor queue */
	ADI_DMA_DESC_LIST_MODE *pCallbackTrace;
#endif /* (ADI_MDMA_CFG_LIST_MULTIPLE_CALLBACKS_ENABLE == 1) */

	/* Stores the DMA direction */
	uint32_t DMADir;

	/* Stores memory transfer size */
	uint32_t MemTransferSize;

    /* Configure the DMA direction to read(source)/write(destination) */
	if(bIsSrcChnl)
	{
		DMADir = ENUM_DMA_CFG_READ;
	}
	else
	{
		DMADir = ENUM_DMA_CFG_WRITE;
	}

	/* Initialize the DMA data interrupt callback argument to NULL */
	pDmaChInfo->pCallbackArg = NULL;

	 /* Based on whether the first descriptor is 1D/2D, configure the corresponding fields in the first descriptor */
	if(pWorkingDesc->YCount>0)
	{
		/* Next descriptor in 2D mode */
		pWorkingDesc->Config|= ENUM_DMA_CFG_ADDR2D;
	}
	else
	{
		/* Next descriptor in 1D mode */
		pWorkingDesc->Config|= ENUM_DMA_CFG_ADDR1D;
	}

	/* Do till the last descriptor */
	while(pWorkingDesc!=NULL)
	{
#if defined(__ADSPSC598_FAMILY__) && defined(__ADSPCORTEXA55__)
		/* Start address address translation(uint32_t) */
		pWorkingDesc->pStartAddress =  adi_rtl_internal_to_system_addr( pWorkingDesc->pStartAddress, /*PortNum*/1);
#else
		/* Start address address translation */
		pWorkingDesc->pStartAddress =  (void *) adi_rtl_internal_to_system_addr( (uint32_t)pWorkingDesc->pStartAddress, /*PortNum*/1);
#endif /* defined(__ADSPSC598_FAMILY__) && defined(__ADSPCORTEXA55__) */


		/* If last descriptor, configure in stop mode */
		if(pWorkingDesc->pNext==NULL)
		{
			/* Tie the next descriptor to NULL */
			pWorkingDesc->Reserved = 0;

			/* Configure in 2D stop mode */
			if(pWorkingDesc->YCount>0)
			{
				pWorkingDesc->Config|= ENUM_DMA_CFG_STOP|DMADir|ENUM_DMA_CFG_ADDR2D|ENUM_DMA_CFG_EN;
				
				/* Enable interrupt only for the destination channel */
				if(!bIsSrcChnl)
				{
					pWorkingDesc->Config|= ENUM_DMA_CFG_YCNT_INT;
				}
			}
			/* Configure in 1D stop mode */
			else
			{
				pWorkingDesc->Config|= ENUM_DMA_CFG_STOP|DMADir|ENUM_DMA_CFG_ADDR1D|ENUM_DMA_CFG_EN;
				 
				/* Enable interrupt only for the destination channel */ 
				if(!bIsSrcChnl)
				{
					pWorkingDesc->Config|= ENUM_DMA_CFG_XCNT_INT;
				}
			 }
		 }
		else
		{
#if defined(__ADSPSC598_FAMILY__) && defined(__ADSPCORTEXA55__)
			/* Configure the next descriptor address */
			pWorkingDesc->Reserved = (uint32_t)(uint64_t)adi_rtl_internal_to_system_addr( pWorkingDesc->pNext, /*PortNum*/1);
#else
			/* Configure the next descriptor address */
			pWorkingDesc->Reserved = adi_rtl_internal_to_system_addr( (uint32_t)pWorkingDesc->pNext, /*PortNum*/1);
#endif /* defined(__ADSPSC598_FAMILY__) && defined(__ADSPCORTEXA55__) */


			/* Initialize the config register to set the DMA direction, set descriptor list mode and enable the DMA.*/ 
			pWorkingDesc->Config|= DMADir|ENUM_DMA_CFG_DSCLIST|ENUM_DMA_CFG_EN;

			/* Based on whether the next descriptor is 1D/2D, configure the corresponding fields in both the current and next descriptor */
			if(pWorkingDesc->pNext->YCount>0)
			{
				/* Fetch next 7 descriptor elements for 2D mode */
				pWorkingDesc->Config|= ENUM_DMA_CFG_FETCH07;

				/* Next descriptor in 2D mode */
				pWorkingDesc->pNext->Config|= ENUM_DMA_CFG_ADDR2D;
			}
			else
			{
				/* Fetch next 5 descriptor elements for 1D mode */
				pWorkingDesc->Config|= ENUM_DMA_CFG_FETCH05;

				/* Next descriptor in 1D mode */
				pWorkingDesc->pNext->Config|= ENUM_DMA_CFG_ADDR1D;
			}
		}

/* IF (DMA cache management enabled) */
#if (__ADI_MDMA_CACHE_MANAGEMENT == 1)

		#if (ADI_MDMA_CFG_LIST_MULTIPLE_CALLBACKS_ENABLE == 1)
		if(bIsSrcChnl)
		{
		#endif
			/*Variable to store memory transfer size*/
			MemTransferSize = MdmaConfigToMemSize(pWorkingDesc->Config);

			/* Flush the descriptor */
			flush_data_buffer((void *)pWorkingDesc,
							(void *)((char *)pWorkingDesc + sizeof(ADI_DMA_DESC_LIST_MODE)),
							ADI_FLUSH_DATA_INV);

			/* Flush the data buffer */
#if defined(__ADSPSC598_FAMILY__) && defined(__ADSPCORTEXA55__)
			adi_dma_FlushCache ((void *)((uint64_t)(pWorkingDesc->pStartAddress)),
#else
			adi_dma_FlushCache ((void *)(pWorkingDesc->pStartAddress),
#endif /* defined(__ADSPSC598_FAMILY__) && defined(__ADSPCORTEXA55__) */
                                pWorkingDesc->XCount,
                                pWorkingDesc->XModify,
                                ((pWorkingDesc->Config & ENUM_DMA_CFG_ADDR2D)
                                 ? pWorkingDesc->YCount : 1u),
                                 pWorkingDesc->YModify,
                                MemTransferSize, ADI_FLUSH_DATA_INV);

		#if (ADI_MDMA_CFG_LIST_MULTIPLE_CALLBACKS_ENABLE == 1)
		}
		#endif
#endif

		/* Move to next descriptor */
		pWorkingDesc= pWorkingDesc->pNext;
		 
		/* IF (First descriptor repeats due to circular chain, come out of the loop) */
		if(pWorkingDesc==pTemp)
		{
			break;
		}
	}

/* If multiple callbacks needed */
#if	(ADI_MDMA_CFG_LIST_MULTIPLE_CALLBACKS_ENABLE == 1)

	/* Should be done only for the destination channel */
	if(!bIsSrcChnl)
	{
		/* Go back to the first descriptor */
		pWorkingDesc=pTemp;

		/* Update callback queue */
		while(pWorkingDesc!=NULL)
		{
			/* IF (Last descriptor or descriptor with callback enabled */
			if((pWorkingDesc->pNext==NULL)||(pWorkingDesc->bCallbackWhenDone))
			{
				/* 2 D mode */
				if(pWorkingDesc->YCount>0)
				{
					pWorkingDesc->Config|= ENUM_DMA_CFG_YCNT_INT;
				}
				 /* 1 D mode */
				 else
				{
					pWorkingDesc->Config|= ENUM_DMA_CFG_XCNT_INT;
				}

				/* Update the pCallbackArg and pCallbackTrace with first callback enabled descriptor */
				if(pDmaChInfo->pCallbackArg == NULL)
				{
					pCallbackTrace = pWorkingDesc;
					pDmaChInfo->pCallbackArg = pCallbackTrace;
					 
					/* Store the address of the first callback enabled  descriptor - useful for circular list*/
					pTemp1 = pCallbackTrace;
				}
				/* Store the address of this descriptor at the pNext field of the previous callback enabled descriptor and
				Update the pCallbackTrace with the new descriptor address */
				else
				{
					pCallbackTrace->pNext = pWorkingDesc; 
					 pCallbackTrace = pWorkingDesc;
				}
			}

	/* IF (DMA cache management enabled) */
	#if (__ADI_MDMA_CACHE_MANAGEMENT == 1)

			 /*Variable to store memory transfer size*/
			MemTransferSize = MdmaConfigToMemSize(pWorkingDesc->Config);

			/* Flush the descriptor */
			flush_data_buffer((void *)pWorkingDesc,
							(void *)((char *)pWorkingDesc + sizeof(ADI_DMA_DESC_LIST_MODE)),
							ADI_FLUSH_DATA_INV);

			/* Flush the data buffer */
			adi_dma_FlushCache ((void *)(pWorkingDesc->pStartAddress),
								pWorkingDesc->XCount,
								pWorkingDesc->XModify,
								((pWorkingDesc->Config & ENUM_DMA_CFG_ADDR2D)
								? pWorkingDesc->YCount : 1u),
								pWorkingDesc->YModify,
								MemTransferSize, ADI_FLUSH_DATA_INV);
	#endif

			/* Move to the next descriptor */
			pWorkingDesc= pWorkingDesc->pNext;
			
			/* IF (First descriptor repeats due to circular chain, come out of the loop) */
			if(pWorkingDesc==pTemp)
			{
				/* Make callback trace circular */
				pCallbackTrace->pNext  = pTemp1;
				
				break;
			}
		}
	}
#else		
		 /* If multiple callbacks are disabled, update pCallbackArg with the address of the first descriptor */
		 pDmaChInfo->pCallbackArg = pTemp;
#endif
}

/*
 * Prepares descriptors for the MDMA channel in array 1D mode.
 *
 * Parameters
 *  - [in]  hChannel        MDMA channel handle to work on.
 *  - [in]	bIsSrcChnl		'true' for source channel and 'false' for destination channel
 *  - [in]  pWorkingDesc    Pointer to the source/destination descriptor list

 * @return  Status
 *          - #ADI_DMA_SUCCESS: Success.
 *          - #ADI_DMA_FAILURE: Failure
 *
 */
static ADI_DMA_RESULT PrepareDescArray1D(
	ADI_DMA_CHANNEL_HANDLE 			hChannel,
	bool 							bIsSrcChnl,
	ADI_DMA_DESC_ARRAY_1D_MODE	   *pWorkingDesc,
	uint16_t                         NumEntries)
{
	/* Return code */ 	
	ADI_DMA_RESULT eResult = ADI_DMA_SUCCESS;

#if defined(ADI_DEBUG)
	/* DMA Channel to work on */
	ADI_MDMA_CHANNEL    *pDmaChInfo = (ADI_MDMA_CHANNEL *)hChannel;
#endif /* defined(ADI_DEBUG) */
	/* Temporary variable to store the descriptor start address */
	ADI_DMA_DESC_ARRAY_1D_MODE *pTemp = pWorkingDesc;
	
	/* Stores the DMA direction */
	uint32_t DMADir;

	/* Stores memory transfer size */
	uint32_t MemTransferSize;

	int i;

    /* Configure the DMA direction to read(source)/write(destination) */
	if(bIsSrcChnl)
	{
		DMADir = ENUM_DMA_CFG_READ;
	}
	else
	{
		DMADir = ENUM_DMA_CFG_WRITE;
	}

	for(i=0;i<NumEntries;i++)
	{
/* IF (Debug information enabled) */
#if defined(ADI_DEBUG)

            /* Validate buffer address */
#if defined(__ADSPSC598_FAMILY__) && defined(__ADSPCORTEXA55__)
            eResult = ValidateBuffer ((uint32_t)(uint64_t) pWorkingDesc->pStartAddress,
                                    pWorkingDesc->Config,
                                    pDmaChInfo->pChInfo->pReg->Status,
                                    pWorkingDesc->XCount,
                                    pWorkingDesc->XModify,
                                    0,
                                    0);
#else
            eResult = ValidateBuffer ((uint32_t) pWorkingDesc->pStartAddress,
                                    pWorkingDesc->Config,
                                    pDmaChInfo->pChInfo->pReg->Status,
                                    pWorkingDesc->XCount,
                                    pWorkingDesc->XModify,
                                    0,
                                    0);
#endif /* defined(__ADSPSC598_FAMILY__) && defined(__ADSPCORTEXA55__) */

			/* IF (Descriptor array is invalid) */
			if (eResult != ADI_DMA_SUCCESS)
			{
				return (eResult);
			}

#endif /* ADI_DEBUG */

/* IF (DMA cache management enabled) */
#if (__ADI_MDMA_CACHE_MANAGEMENT == 1)

			 /*Variable to store memory transfer size*/
			 MemTransferSize = MdmaConfigToMemSize(pWorkingDesc->Config);

			/* Flush the data buffer */
#if defined(__ADSPSC598_FAMILY__) && defined(__ADSPCORTEXA55__)
				adi_dma_FlushCache ((void *)((uint64_t)(pWorkingDesc->pStartAddress)),
#else
				adi_dma_FlushCache ((void *)(pWorkingDesc->pStartAddress),
#endif /* defined(__ADSPSC598_FAMILY__) && defined(__ADSPCORTEXA55__) */
                                    pWorkingDesc->XCount,
                                    pWorkingDesc->XModify,
                                    1,
                                    0,
                                    MemTransferSize, ADI_FLUSH_DATA_INV);
#endif

#if defined(__ADSPSC598_FAMILY__) && defined(__ADSPCORTEXA55__)
			/* Translate the start address to system address space(uint32_t) */
			pWorkingDesc->pStartAddress =  adi_rtl_internal_to_system_addr( pWorkingDesc->pStartAddress, /*PortNum*/1);
#else
			/* Translate the start address to system address space */
			pWorkingDesc->pStartAddress =  (void *) adi_rtl_internal_to_system_addr( (uint32_t)pWorkingDesc->pStartAddress, /*PortNum*/1);
#endif /* defined(__ADSPSC598_FAMILY__) && defined(__ADSPCORTEXA55__) */

		/* Configure all but last descriptors in array mode */
		if(i!=NumEntries-1)
		{
			pWorkingDesc->Config|=ADI_MDMA_DMA_CFG_1D_ARRAY|DMADir;
		}
		/* Configure last descriptor in stop mode */
		else
		{
			pWorkingDesc->Config|=ENUM_DMA_CFG_STOP|DMADir|ENUM_DMA_CFG_EN;
			
			/* Enable interrupt only for the destination channel*/
			if(!bIsSrcChnl)
			{
				pWorkingDesc->Config|=ENUM_DMA_CFG_XCNT_INT;
			}
		}
		/* Move to the next descriptor */
		pWorkingDesc++;
	}

/* IF (DMA cache management enabled) */
#if (__ADI_MDMA_CACHE_MANAGEMENT == 1)
	/* Flush the descriptor */
	flush_data_buffer((void *)pTemp,
					(void *)((char *)pTemp + NumEntries*sizeof(ADI_DMA_DESC_ARRAY_1D_MODE)),
					ADI_FLUSH_DATA_INV);
#endif
	
	return eResult;
}
		
/*
 * Prepares descriptors for the MDMA channel in array 2D mode.
 *
 * Parameters
 *  - [in]  hChannel        MDMA channel handle to work on.
 *  - [in]	bIsSrcChnl		'true' for source channel and 'false' for destination channel
 *  - [in]  pWorkingDesc    Pointer to the source/destination descriptor list

 * @return  Status
 *          - #ADI_DMA_SUCCESS: Success.
 *          - #ADI_DMA_FAILURE: Failure
 *
 */
static ADI_DMA_RESULT PrepareDescArray2D(
	ADI_DMA_CHANNEL_HANDLE 			hChannel,
	bool 							bIsSrcChnl,
	ADI_DMA_DESC_ARRAY_2D_MODE		*pWorkingDesc,
	uint16_t                         NumEntries)		
{
	/* Return code */ 
	ADI_DMA_RESULT eResult = ADI_DMA_SUCCESS;

#if defined(ADI_DEBUG)
	/* DMA Channel to work on */
	ADI_MDMA_CHANNEL    *pDmaChInfo = (ADI_MDMA_CHANNEL *)hChannel;
#endif /* defined(ADI_DEBUG) */

	/* Temporary variable to store the descriptor start address */
	ADI_DMA_DESC_ARRAY_2D_MODE *pTemp = pWorkingDesc;
	
	/* Stores the DMA direction */
	uint32_t DMADir;

	/* Stores memory transfer size */
	uint32_t MemTransferSize;

	int i;

    /* Configure the DMA direction to read(source)/write(destination) */
	if(bIsSrcChnl)
	{
		DMADir = ENUM_DMA_CFG_READ;
	}
	else
	{
		DMADir = ENUM_DMA_CFG_WRITE;
	}

	for(i=0;i<NumEntries;i++)
	{
/* IF (Debug information enabled) */
#if defined(ADI_DEBUG)

		/* Validate buffer address */
#if defined(__ADSPSC598_FAMILY__) && defined(__ADSPCORTEXA55__)
        eResult = ValidateBuffer ((uint32_t)(uint64_t) pWorkingDesc->pStartAddress,
                                  pWorkingDesc->Config,
                                  pDmaChInfo->pChInfo->pReg->Status,
                                  pWorkingDesc->XCount,
                                  pWorkingDesc->XModify,
                                  pWorkingDesc->YCount,
                                  pWorkingDesc->YModify);
#else
        eResult = ValidateBuffer ((uint32_t) pWorkingDesc->pStartAddress,
                                  pWorkingDesc->Config,
                                  pDmaChInfo->pChInfo->pReg->Status,
                                  pWorkingDesc->XCount,
                                  pWorkingDesc->XModify,
                                  pWorkingDesc->YCount,
                                  pWorkingDesc->YModify);
#endif /* defined(__ADSPSC598_FAMILY__) && defined(__ADSPCORTEXA55__) */

		/* IF (Descriptor array is invalid) */
		if (eResult != ADI_DMA_SUCCESS)
		{
			return (eResult);
		}

#endif /* ADI_DEBUG */

/* IF (DMA cache management enabled) */
#if (__ADI_MDMA_CACHE_MANAGEMENT == 1)

		/*Variable to store memory transfer size*/
		MemTransferSize = MdmaConfigToMemSize(pWorkingDesc->Config);

		/* Flush the data buffer */
#if defined(__ADSPSC598_FAMILY__) && defined(__ADSPCORTEXA55__)
		adi_dma_FlushCache ((void *)((uint64_t)(pWorkingDesc->pStartAddress)),
#else
		adi_dma_FlushCache ((void *)(pWorkingDesc->pStartAddress),
#endif /* defined(__ADSPSC598_FAMILY__) && defined(__ADSPCORTEXA55__) */
                                pWorkingDesc->XCount,
                                pWorkingDesc->XModify,
                                pWorkingDesc->YCount,
                                pWorkingDesc->YModify,
                                MemTransferSize, ADI_FLUSH_DATA_INV);
#endif

#if defined(__ADSPSC598_FAMILY__) && defined(__ADSPCORTEXA55__)
		/* Translate the start address to system address space(uint32_t) */
		pWorkingDesc->pStartAddress =  adi_rtl_internal_to_system_addr( pWorkingDesc->pStartAddress, /*PortNum*/1);
#else
		/* Translate the start address to system address space */
		pWorkingDesc->pStartAddress =  (void *) adi_rtl_internal_to_system_addr( (uint32_t)pWorkingDesc->pStartAddress, /*PortNum*/1);
#endif /* defined(__ADSPSC598_FAMILY__) && defined(__ADSPCORTEXA55__) */

	
		/* Configure all but last descriptors in array mode */
		if(i!=NumEntries-1)
		{
			pWorkingDesc->Config|=ADI_MDMA_DMA_CFG_2D_ARRAY|DMADir;
		}
		/* Configure last descriptor in stop mode */
		else
		{
			pWorkingDesc->Config|=ADI_MDMA_DMA_CFG_2D_STOP_FLOW|DMADir|ENUM_DMA_CFG_EN;
			
			/* Enable interrupt only for the destination channel*/
			if(!bIsSrcChnl)
			{
				pWorkingDesc->Config|=ENUM_DMA_CFG_YCNT_INT;
			}
		}
		pWorkingDesc++;
	 }

/* IF (DMA cache management enabled) */
#if (__ADI_MDMA_CACHE_MANAGEMENT == 1)
	/* Flush the descriptor */
	flush_data_buffer((void *)pTemp,
					(void *)((char *)pTemp + NumEntries*sizeof(ADI_DMA_DESC_ARRAY_2D_MODE)),
					ADI_FLUSH_DATA_INV);
#endif /*__ADI_DMA_CACHE_MANAGEMENT */
		
	return eResult;
}

/*=============  D E B U G   F U N C T I O N S =============*/

/* IF (Debug information enabled) */
#if defined(ADI_DEBUG)

/* Validates DMA Channel handle */
static ADI_DMA_RESULT ValidateChannelHandle (ADI_DMA_CHANNEL_HANDLE    hChannel)
{

	/* MDMA channel to work on */
    ADI_MDMA_CHANNEL *pChannel = (ADI_MDMA_CHANNEL *)hChannel;

	/* MDMA stream to work on */
    ADI_MDMA_STREAM     *pStream = pChannel->pChInfo->pMdmaStream;

	/* The channel handle is valid only when the corresponding stream handle is valid */
	if(ValidateStreamHandle(pStream)==ADI_DMA_SUCCESS)
    {
    	 return (ADI_DMA_SUCCESS);
    }

    /* Channel invalid, return failure */
    return (ADI_DMA_FAILURE);
}

/* Validates MDMA Stream handle */
static ADI_DMA_RESULT ValidateStreamHandle (ADI_DMA_STREAM_HANDLE   hStream)
{
    /* Loop variable */
    uint32_t    i;

	/* MDMA stream to work on */
    ADI_MDMA_STREAM     *pStream = (ADI_MDMA_STREAM *) hStream;

    /* FOR (All MDMA Streams) */
    for (i = 0; i < ADI_MDMA_TOTAL_STREAMS; i++)
    {
        /* IF (Stream handle is valid) */
        if ((pStream!=NULL) && ((pStream->SrcCh.pChInfo == (ADI_MDMA_CHANNEL_INFO*)&StreamInfo[i])&&(pStream->SrcCh.pChInfo->pMdmaStream==StreamInfo[i].SrcChInfo.pMdmaStream)))
        {
            /* Stream handle valid, return success */
            return (ADI_DMA_SUCCESS);
        }
    }

    /* Channel invalid, return failure */
    return (ADI_DMA_FAILURE);
}

/*
 * Validates buffer address and its corresponding DMA register values.
 *
 * Parameters
 *  - [in]  StartAddress    Start address of the data buffer to be processed.
 *  - [in]  Config          DMA Config register value to validate.
 *  - [in]  Status          DMA Status register value to be used for validation.
 *  - [in]  XCount          Inner loop count start value in number of
 *                          DMA data bus width (MSIZE) transfers.
 *  - [in]  XModify         Inner loop address increment in bytes.
 *  - [in]  YCount          Outer loop count start value in number of rows.
 *  - [in]  YModify         Outer loop address increment in bytes.
 *
 * Returns  Status
 *  - ADI_DMA_SUCCESS: Supplied List buffer is valid.
 *  - ADI_DMA_BAD_PSIZE: Supplied PSize is more than its allowed value.
 *  - ADI_DMA_BAD_XCOUNT: Supplied XCount is invalid.
 *  - ADI_DMA_BAD_YCOUNT: Supplied YCount is invalid.
 *  - ADI_DMA_BAD_START_ADDR: Supplied start address is invalid.
 *  - ADI_DMA_ADDR_ALIGNMENT_ERROR: MSize/XModify/YModify doesn't support the supplied
 *                                  buffer address and will cause alignment error.
 *
 *
 * Address Alignment to MSIZE
 * +--------+---------------------------+
 * | MSIZE  |   Start Address (ADDR)    |
 * +--------+---------------------------+
 * |1 Byte  |   No restriction on ADDR  |
 * +--------+---------------------------+
 * |2 Bytes |   ADDR[0] = 0             |
 * +--------+---------------------------+
 * |4 Bytes |   ADDR[1:0] = 0           |
 * +--------+---------------------------+
 * |8 Bytes |   ADDR[2:0] = 0           |
 * +--------+---------------------------+
 * |16 Bytes|   ADDR[3:0] = 0           |
 * +--------+---------------------------+
 * |32 Bytes|   ADDR[4:0] = 0           |
 * +--------+---------------------------+
 *
 * The above table also true for XModify and YModify (in 2D mode)
 *
 */
static ADI_DMA_RESULT   ValidateBuffer (
    uint32_t            StartAddress,
    uint32_t            Config,
    uint32_t            Status,
    uint32_t            XCount,
    int32_t             XModify,
    uint32_t            YCount,
    int32_t             YModify)
{
    /* Variables to validate buffer address and X/Y modify */
    uint32_t    MSize;

    /* IF (Start address is invalid) */
    if (StartAddress == 0)
    {
    	/* Report failure as supplied start address is invalid */
    	return (ADI_DMA_BAD_START_ADDR);
    }

    /* IF (XCount is invalid) */
    if (XCount == 0)
    {
    	/* Report failure as supplied XCount is invalid */
    	return (ADI_DMA_BAD_XCOUNT);
    }

    /* IF (YCount is invalid) */
    if ((YCount == 0) && (YModify != 0))
    {
    	/* Report failure as supplied YCount is invalid */
        return (ADI_DMA_BAD_YCOUNT);
    }

    /* IF (PSize is more than the allowed value) */
    if (((Config & BITM_DMA_CFG_PSIZE) >> BITP_DMA_CFG_PSIZE) > \
        ((Status & BITM_DMA_STAT_PBWID) >> BITP_DMA_STAT_PBWID))
    {
        /* Return failure as the PSize is invalid */
        return (ADI_DMA_BAD_PSIZE);
    }

    /* MSize must be more than 1-byte, Extract MSize */
    MSize = ((Config & BITM_DMA_CFG_MSIZE) >> BITP_DMA_CFG_MSIZE);

    /* IF (Buffer alignment doesn't work with the MSize) */
    if ((StartAddress & ((1ul << MSize)-1ul)) != 0)
    {
        /* Return failure - address alignment error */
        return (ADI_DMA_ADDR_ALIGNMENT_ERROR);
    }

    /* Return success as the supplied address is valid */
    return (ADI_DMA_SUCCESS);
}


/* Validate descriptor List */
static ADI_DMA_RESULT ValidateDescList (
	ADI_MDMA_STREAM                 *pStream,
    ADI_DMA_DESC_LIST_MODE          *pDestDesc,
    ADI_DMA_DESC_LIST_MODE          *pSrcDesc)
{
    /* Return Code */
    ADI_DMA_RESULT  eResult = ADI_DMA_SUCCESS;

    /* DO (for all descriptors in list) */
    while (pDestDesc != NULL)
    {
        /* Validate buffer address */
#if defined(__ADSPSC598_FAMILY__) && defined(__ADSPCORTEXA55__)
        eResult = ValidateBuffer ((uint32_t)(uint64_t) pDestDesc->pStartAddress,
                                  pDestDesc->Config,
                                  pStream->DestCh.pChInfo->pReg->Status,
                                  pDestDesc->XCount,
                                  pDestDesc->XModify,
                                  pDestDesc->YCount,
                                  pDestDesc->YModify);
#else
        eResult = ValidateBuffer ((uint32_t) pDestDesc->pStartAddress,
                                  pDestDesc->Config,
                                  pStream->DestCh.pChInfo->pReg->Status,
                                  pDestDesc->XCount,
                                  pDestDesc->XModify,
                                  pDestDesc->YCount,
                                  pDestDesc->YModify);
#endif /* defined(__ADSPSC598_FAMILY__) && defined(__ADSPCORTEXA55__) */

        /* IF (Error) */
        if (eResult != ADI_DMA_SUCCESS)
        {
            return (eResult);
        }

        /* Move to next descriptor in chain */
        pDestDesc = pDestDesc->pNext;
    }

    /* DO (for all descriptors in list) */
    while (pSrcDesc != NULL)
    {
        /* Validate buffer address */
#if defined(__ADSPSC598_FAMILY__) && defined(__ADSPCORTEXA55__)
        eResult = ValidateBuffer ((uint32_t)(uint64_t) pSrcDesc->pStartAddress,
                                  pSrcDesc->Config,
                                  pStream->SrcCh.pChInfo->pReg->Status,
                                  pSrcDesc->XCount,
                                  pSrcDesc->XModify,
                                  pSrcDesc->YCount,
                                  pSrcDesc->YModify);
#else
        eResult = ValidateBuffer ((uint32_t) pSrcDesc->pStartAddress,
                                  pSrcDesc->Config,
                                  pStream->SrcCh.pChInfo->pReg->Status,
                                  pSrcDesc->XCount,
                                  pSrcDesc->XModify,
                                  pSrcDesc->YCount,
                                  pSrcDesc->YModify);
#endif /* defined(__ADSPSC598_FAMILY__) && defined(__ADSPCORTEXA55__) */

        /* IF (Error) */
        if (eResult != ADI_DMA_SUCCESS)
        {
            return (eResult);
        }

        /* Move to next descriptor in chain */
        pSrcDesc = pSrcDesc->pNext;
    }

    /* Return success */
    return (ADI_DMA_SUCCESS);
}
#endif /* ADI_DEBUG */

/*=============  P U B L I C   F U N C T I O N S =============*/
/**
 * @brief Opens a Memory DMA stream for use.
 *
 * @param [in]  eStreamID       MDMA Stream ID to open.
 * @param [in]  pStreamMemory   Pointer to ADI_DMA_STREAM_REQ_MEMORY sized memory
 *                              location to handle the DMA stream.
 * @param [out] phStream        Address where the MDMA stream handle will be stored.
 * @param [out] phSrcChannel    Address where the Source DMA channel handle will be stored.
 * @param [out] phDestChannel   Address where the Destination DMA channel handle will be stored.
 * @param [in]  pfCallback      Address of application callback function.
 * @param [in]  pCBParam        Parameter passed back to application callback.
 *
 * @return  Status
 *          - #ADI_DMA_SUCCESS: Successfully opened a MDMA Stream.
 *          - #ADI_DMA_FAILURE: Supplied pointer parameters are invalid.
 *          - #ADI_DMA_IN_USE: Supplied MDMA Stream ID is already in use.
 *          - #ADI_DMA_BAD_STREAM_ID: Supplied MDMA Stream ID is invalid.
 *          - #ADI_DMA_INT_REGISTER_FAILED: Failed to register DMA interrupt handler.
 *			- #ADI_DMA_INT_UNREGISTER_FAILED: Failed to unregister DMA interrupt handler.
 * @note
 *      Source and Destination channel handles are made available to the
 *      programmer so that the handles can be used to query channel specific
 *      information such as current XCount etc., using the APIs to configure trigger settings etc.
 *
 * @sa adi_mdma_Close()
 */
ADI_DMA_RESULT adi_mdma_Open(
    ADI_DMA_STREAM_ID               eStreamID,
    void                            *pStreamMemory,
    ADI_DMA_STREAM_HANDLE           *phStream,
    ADI_DMA_CHANNEL_HANDLE          *phSrcChannel,
    ADI_DMA_CHANNEL_HANDLE          *phDestChannel,
    ADI_CALLBACK                    pfCallback,
    void                            *pCBParam)
{
	/* Return Code */
    ADI_DMA_RESULT      eResult = ADI_DMA_SUCCESS;

    /* Pointer to memory allocated to handle DMA Stream */
    ADI_MDMA_STREAM     *pStream = (ADI_MDMA_STREAM *) pStreamMemory;

    /* Pointer to the processor specific stream information */
    ADI_MDMA_STREAM_INFO *pStreamInfo = &StreamInfo[eStreamID];

/* IF (Debug information enabled) */
#if defined(ADI_DEBUG)

	   /* IF (This is not a valid DMA stream) */
	   if (eStreamID >= ADI_DMA_MEMDMA_END_COUNT)
	   {
	    /* Report failure as bad stream id */
	    return (ADI_DMA_BAD_STREAM_ID);
	   }

	   /* Check if the given pointer parameters are valid */
	   if((pStreamMemory == NULL) || (phStream == NULL) || (phSrcChannel == NULL)  || (phDestChannel == NULL))
	   {
		return (ADI_DMA_FAILURE);
	   }

#endif  /* ADI_DEBUG */

    /* Save callback parameters */
    pStream->pfCallback = pfCallback;
    pStream->pCBParam   = pCBParam;

    /* Set MDMA transfer mode to none*/
    pStream->eXferMode = ADI_MDMA_XFER_NONE;

    /* Pointer to the stream/channel info */
    pStream->SrcCh.pChInfo = &(pStreamInfo->SrcChInfo);
    pStream->DestCh.pChInfo = &(pStreamInfo->DestChInfo);

	/* IF (DMA stream already in use) */
	if ((pStream->SrcCh.pChInfo->pMdmaStream != NULL)||(pStream->DestCh.pChInfo->pMdmaStream != NULL))
	{
		/* Report failure as the stream is already in use */
		 eResult = ADI_DMA_IN_USE;
	}
	
	/*Reset callback argument and common configuration values */
	pStream->SrcCh.pCallbackArg = NULL;
	pStream->DestCh.pCallbackArg = NULL;
	pStream->SrcCh.CommonConfig = 0u;
	pStream->DestCh.CommonConfig = 0u;

	/* Set descriptor reuse to false */
	pStream->SrcCh.bDescReuse = false;
	pStream->DestCh.bDescReuse = false;

    /* IF (DMA Stream is available for use) */
    if (eResult == ADI_DMA_SUCCESS)
    {
		/* Entering critical region, disable interrupts */
		adi_osal_EnterCriticalRegion();
		
			/* Save the Memory address allocated to handle this MDMA stream */
			pStream->SrcCh.pChInfo->pMdmaStream=pStream;
			pStream->DestCh.pChInfo->pMdmaStream=pStream;

			/* Data transfer is not in progress */
			pStream->bIsXferInProgress  = false;

			/* Reset the DMA registers of source and destination channels */
			ResetDmaRegs(pStream->SrcCh.pChInfo->pReg);
			ResetDmaRegs(pStream->DestCh.pChInfo->pReg);
		
		/* Re-enable interrupts */
		adi_osal_ExitCriticalRegion();

	   /* Install Source DMA Error interrupt handler */
		if(adi_int_InstallHandler((uint32_t)pStream->SrcCh.pChInfo->ErrorIntSID, DmaErrorIntHandler, &(pStream->SrcCh), true) != ADI_INT_SUCCESS)
		{
			/* Failed to register DMA interrupt handler */
			eResult = ADI_DMA_INT_REGISTER_FAILED;
		}

		/* Install Destination DMA Error interrupt handler */
		if(adi_int_InstallHandler((uint32_t)pStream->DestCh.pChInfo->ErrorIntSID, DmaErrorIntHandler, &(pStream->DestCh), true) != ADI_INT_SUCCESS)
		{
			/* Failed to register DMA interrupt handler */
			eResult = ADI_DMA_INT_REGISTER_FAILED;
		}

		/* Install Destination DMA Data Interrupt handler */
		if (adi_int_InstallHandler ((uint32_t)pStream->DestCh.pChInfo->DataIntSID, DmaDataIntHandler, &(pStream->DestCh), true) != ADI_INT_SUCCESS)
		{
			/* Uninstall destination error interrupt handler if data interrupt handler installation fails */
			if (adi_int_UninstallHandler ((uint32_t)pStream->DestCh.pChInfo->ErrorIntSID) != ADI_INT_SUCCESS)
			{
				/* Return failure as we failed to un-register the interrupt */
				eResult = ADI_DMA_INT_UNREGISTER_FAILED;
			}
			
			/* Uninstall source error interrupt handler if data interrupt handler installation fails */
			if (adi_int_UninstallHandler ((uint32_t)pStream->SrcCh.pChInfo->ErrorIntSID) != ADI_INT_SUCCESS)
			{
				/* Return failure as we failed to un-register the interrupt */
				eResult = ADI_DMA_INT_UNREGISTER_FAILED;
			}
		}
    }

    /* IF (Successfully registered MDMA stream with Interrupt dispatcher) */
    if (eResult == ADI_DMA_SUCCESS)
    {
        /* Pass a valid handle to this MDMA Stream */
        *phStream = (ADI_DMA_STREAM_HANDLE)pStream;

        /* Pass channel handle for source and destination channels */
        *phSrcChannel  = (ADI_DMA_CHANNEL_HANDLE) &(pStream->SrcCh);
        *phDestChannel = (ADI_DMA_CHANNEL_HANDLE) &(pStream->DestCh);
    }
    else
    {
    	/* Clear the memory address allocated to handle the MDMA stream */
    	pStream->SrcCh.pChInfo->pMdmaStream=NULL;
    	pStream->DestCh.pChInfo->pMdmaStream=NULL;
    }

	/* Return */
	return (eResult);
}

/**
 * @brief Closes a Memory DMA stream.
 *
 * @param [in]  hStream     MDMA Stream handle to be closed.
 * @param [in]  bWaitFlag   Wait for transfers to complete flag
 *                          (TRUE to wait, FALSE to force close).
 *
 * @return  Status
 *          - #ADI_DMA_SUCCESS: Successfully closed a MDMA Stream.
 *          - #ADI_DMA_NOT_OPEN: The MDMA Stream is not open yet.
 *          - #ADI_DMA_BAD_HANDLE: Supplied MDMA Stream handle is invalid.
 *          - #ADI_DMA_INT_UNREGISTER_FAILED: Failed to unregister interrupt.
 *
 * @sa		adi_mdma_open()
 */
ADI_DMA_RESULT adi_mdma_Close(
    ADI_DMA_STREAM_HANDLE           hStream,
    bool                            bWaitFlag)
{
    /* Return Code */
    ADI_DMA_RESULT  eResult = ADI_DMA_SUCCESS;

    /* MDMA Stream to work on */
    ADI_MDMA_STREAM     *pStream = (ADI_MDMA_STREAM *) hStream;

/* IF (Debug information enabled) */
#if defined(ADI_DEBUG)

	/* IF (MDMA Stream Handle is invalid) */
	if (ValidateStreamHandle (hStream) != ADI_DMA_SUCCESS)
	{
		return (ADI_DMA_BAD_HANDLE);
	}

	/* IF (Stream is not in use) */
	if (pStream->DestCh.pChInfo->pMdmaStream == NULL)
	{
		return (ADI_DMA_NOT_OPEN);
	}

#endif /* ADI_DEBUG */

    /* IF (wait for DMA to complete transfer) */
    if (bWaitFlag)
    {
        /* wait until DMA is idle */
        while ((pStream->DestCh.pChInfo->pReg->Status & BITM_DMA_STAT_RUN) != ENUM_DMA_STAT_STOPPED)
        {
        }
    }

	/* Un-register DMA data interrupt handler for destination channel */
	if (adi_int_UninstallHandler ((uint32_t)pStream->DestCh.pChInfo->DataIntSID) != ADI_INT_SUCCESS)
	{
		/* Return failure as we failed to un-register the interrupt */
		return (ADI_DMA_INT_UNREGISTER_FAILED);
	}

   /* Un-register DMA error interrupt handler for source channel */
	if (adi_int_UninstallHandler ((uint32_t)pStream->SrcCh.pChInfo->DataIntSID) != ADI_INT_SUCCESS)
	{
		/* Return failure as we failed to un-register the interrupt */
		return (ADI_DMA_INT_UNREGISTER_FAILED);
	}

	/* Un-register DMA error interrupt handler for destination channel */
	if (adi_int_UninstallHandler ((uint32_t)pStream->DestCh.pChInfo->ErrorIntSID) != ADI_INT_SUCCESS)
	{
		/* Return failure as we failed to un-register the interrupt */
		return (ADI_DMA_INT_UNREGISTER_FAILED);
	}

    /* Entering critical region, disable interrupts */
    adi_osal_EnterCriticalRegion();

	    /* Reset Destination DMA registers */
		ResetDmaRegs (pStream->DestCh.pChInfo->pReg);

		/* Reset Source DMA registers */
		ResetDmaRegs (pStream->SrcCh.pChInfo->pReg);

		/* Clear the memory address allocated to handle the MDMA Stream */
		pStream->SrcCh.pChInfo->pMdmaStream=NULL;
		pStream->DestCh.pChInfo->pMdmaStream=NULL;

		/* Clearing the pointer to channel info */
		pStream->SrcCh.pChInfo=NULL;
		pStream->DestCh.pChInfo=NULL;
		
    /* Re-enable interrupts */
    adi_osal_ExitCriticalRegion();

    return (eResult);
}

/**
 * @brief Gets the current data copy status of a Memory DMA stream.
 *
 * @param [in]  hStream             MDMA stream handle to query.
 * @param [out] pbCopyInProgress    Location to store current data copy status.
 *
 * @return  Status
 *          - #ADI_DMA_SUCCESS: Successfully queried MDMA dataflow status.
 *          - #ADI_DMA_NOT_OPEN: The MDMA Stream is not open yet.
 *          - #ADI_DMA_BAD_HANDLE: Supplied MDMA Stream handle is invalid.
 *
 */
ADI_DMA_RESULT adi_mdma_IsCopyInProgress(
    ADI_DMA_STREAM_HANDLE           hStream,
    bool                            *pbCopyInProgress)
{
    /* MDMA Stream to work on */
	ADI_MDMA_STREAM     *pStream = (ADI_MDMA_STREAM *) hStream;

	/* IF (Debug information enabled) */
#if defined(ADI_DEBUG)

	/* IF (MDMA Stream Handle is invalid) */
	if (ValidateStreamHandle (hStream) != ADI_DMA_SUCCESS)
	{
		return (ADI_DMA_BAD_HANDLE);
	}

	/* IF (Stream is not in use) */
	if (pStream->DestCh.pChInfo->pMdmaStream == NULL)
	{
		return (ADI_DMA_NOT_OPEN);
	}

#endif /* ADI_DEBUG */

    /* Get the current data transfer status of the DMA channel */
    *pbCopyInProgress = pStream->bIsXferInProgress;

    /* Report success */
    return (ADI_DMA_SUCCESS);
}

/**
 * @brief Disables/terminates Memory DMA copy that is currently in progress.
 *
 * @param [in]  hStream     MDMA stream handle to work on.
 *
 * @return  Status
 *          - #ADI_DMA_SUCCESS: Successfully disabled MDMA dataflow.
 *          - #ADI_DMA_NOT_OPEN: The MDMA Stream is not open yet.
 *          - #ADI_DMA_BAD_HANDLE: Supplied MDMA Stream handle is invalid.
 *
 */
ADI_DMA_RESULT adi_mdma_Disable (
    ADI_DMA_STREAM_HANDLE           hStream)
{
    /* MDMA Stream to work on */
	ADI_MDMA_STREAM     *pStream = (ADI_MDMA_STREAM *) hStream;

/* IF (Debug information enabled) */
#if defined(ADI_DEBUG)

    /* IF (MDMA Stream Handle is invalid) */
    if (ValidateStreamHandle (hStream) != ADI_DMA_SUCCESS)
    {
        return (ADI_DMA_BAD_HANDLE);
    }

	/* IF (Stream is not in use) */
	if (pStream->DestCh.pChInfo->pMdmaStream == NULL)
	{
		return (ADI_DMA_NOT_OPEN);
	}

#endif /* ADI_DEBUG */

    /* Entering critical region, disable interrupts */
     adi_osal_EnterCriticalRegion();

    /* Disable MDMA transfer */
    DisableXfer (pStream);

	/* Re-enable interrupts */
   adi_osal_ExitCriticalRegion();
   
    /* Report success */
    return (ADI_DMA_SUCCESS);
}

/**
 * @brief Copies data between memory in One-Dimensional mode.
 *
 * @param [in]  hStream         MDMA Stream handle to work on.
 * @param [in]  pMemDest        Memory destination address.
 * @param [in]  pMemSrc         Memory source address.
 * @param [in]  eElementWidth   Data element width in terms of ADI_DMA_MSIZE.
 * @param [in]  ElementCount    Number of data elements to copy.
 *
 * @return  Status
 *          - #ADI_DMA_SUCCESS: Successfully submitted 1D Memory transfer info.
 *          - #ADI_DMA_NOT_OPEN: The MDMA Stream is not open yet.
 *          - #ADI_DMA_BAD_HANDLE: Supplied MDMA Stream handle is invalid.
 *          - #ADI_DMA_ALREADY_RUNNING: Can't submit new buffer when the DMA dataflow is already enabled.
 *
 * @note	When the configuration macro ADI_MDMA_CFG_MSIZE_AUTO is enabled (default),
 * 			this API selects the most optimum MSIZE value automatically (and overrides the 
 *          user supplied MSIZE value) based on the DMA start address and count value. 
 *
 * @sa	adi_mdma_Copy1DAuto(), adi_dma_ConfigTriggerIn(), adi_dma_ConfigTriggerOut(), adi_mdma_BWLimit(), adi_mdma_BWMonitor(), and adi_mdma_BWMeasure()
*/
ADI_DMA_RESULT adi_mdma_Copy1D(
    ADI_DMA_STREAM_HANDLE           hStream,
    void                            *pMemDest,
    void                            *pMemSrc,
    ADI_DMA_MSIZE                   eElementWidth,
    uint32_t                        ElementCount)
{
    /* Return code */
    ADI_DMA_RESULT  eResult = ADI_DMA_SUCCESS;

	/* MDMA stream to work on */
    ADI_MDMA_STREAM     *pStream = (ADI_MDMA_STREAM *) hStream;

	/* Other variables */
    uint32_t  MSize, MSizePSizeCfg, PSize,  StartAddr;
#if (ADI_MDMA_CFG_MSIZE_AUTO == 1)
    uint32_t ElementCountInBytes, Mask, Value;
#endif /* (ADI_MDMA_CFG_MSIZE_AUTO == 1) */

    int32_t Xmod;

/* IF (Debug information enabled) */
#if defined(ADI_DEBUG)
	/* IF (MDMA Stream Handle is invalid) */
	if (ValidateStreamHandle (hStream) != ADI_DMA_SUCCESS)
	{
		return (ADI_DMA_BAD_HANDLE);
	}

    /* IF (Stream is not in use) */
    if (pStream->DestCh.pChInfo->pMdmaStream == NULL)
    {
        return (ADI_DMA_NOT_OPEN);
    }

	/* Element width less than 4 bytes are not allowed for MDMA streams 2 and 3 */
	if((pStream->SrcCh.pChInfo->ChannelID==CHID_MDMA2_SRC)||(pStream->SrcCh.pChInfo->ChannelID==CHID_MDMA3_SRC))
	{
		if((eElementWidth==ADI_DMA_MSIZE_1BYTE)||(eElementWidth==ADI_DMA_MSIZE_2BYTES))
		{
			return (ADI_DMA_BAD_MSIZE);
		}
	}
#endif /* ADI_DEBUG */

    /* IF (DMA channel is already in the middle of a data transfer) */
    if (pStream->bIsXferInProgress)
    {
        /* Report failure as DMA channel already running */
        return (ADI_DMA_ALREADY_RUNNING);
    }

	/* Get MSize value */
    MSize = ((uint32_t)eElementWidth >> BITP_DMA_CFG_MSIZE);

    /* Select the optimum MSIZE value automatically based on the source and destination start address and count value */
#if	(ADI_MDMA_CFG_MSIZE_AUTO == 1)
    /* Initialize Mask */
     Mask = ADI_DMA_MSIZE_MASK;

     /* Get byte equivalent of MSize and calculate Element count in bytes */
     ElementCountInBytes = (ElementCount * (1 << MSize));

#if defined(__ADSPSC598_FAMILY__) && defined(__ADSPCORTEXA55__)
     /* Calculate the value that will be used to get Max MSize */
     Value = (ElementCountInBytes | (uint32_t)(uint64_t)pMemDest | (uint32_t)(uint64_t)pMemSrc);
#else
     /* Calculate the value that will be used to get Max MSize */
     Value = (ElementCountInBytes | (uint32_t)pMemDest | (uint32_t)pMemSrc);
#endif /* defined(__ADSPSC598_FAMILY__) && defined(__ADSPCORTEXA55__) */

     /* FOR (Valid MSize is reached or MSize is zero) */
     for (MSize = ADI_DMA_MAX_MSIZE_VAL; MSize > 0 ; MSize--)
     {
         /* Validate the OR'd result of element count and buffer address */
         if ((Value & Mask) == 0u)
         {
            break;
         }
		/* Update Mask */
         Mask >>= 1;
     }

    /* Recalculate ElementCount value */
    ElementCount = ElementCountInBytes >> MSize;

#endif /* ADI_DMA_CFG_MSIZE_AUTO */

	/* PSIZE should be less or equal to DMA_STAT.PBWID, keeping it maximum */
    PSize = ((pStream->SrcCh.pChInfo->pReg->Status)&BITM_DMA_STAT_PBWID)>>BITP_DMA_STAT_PBWID;

    /* PSIZE should be less than or equal to MSIZE */
    if(PSize>MSize)
    {
    	PSize=MSize;
    }

    /* Calculate MSIZE and PSIZE field values of the config register */
    MSizePSizeCfg = (MSize<<BITP_DMA_CFG_MSIZE)|(PSize<<BITP_DMA_CFG_PSIZE);

    /* Calculate the modifier value */
 	Xmod = (int32_t)(1 << (int32_t)MSize);

/* IF (Debug information enabled) */
#if defined(ADI_DEBUG)

	if (pMemDest != NULL)
    {
#if defined(__ADSPSC598_FAMILY__) && defined(__ADSPCORTEXA55__)
        eResult = ValidateBuffer ((uint32_t)(uint64_t)pMemDest,
                            MSizePSizeCfg,
                            pStream->DestCh.pChInfo->pReg->Status,
                            ElementCount,
                            (1u << (int32_t)MSize),
                            0,
                            0);
#else
        eResult = ValidateBuffer ((uint32_t)pMemDest,
                            MSizePSizeCfg,
                            pStream->DestCh.pChInfo->pReg->Status,
                            ElementCount,
                            (1u << (int32_t)MSize),
                            0,
                            0);
#endif /* defined(__ADSPSC598_FAMILY__) && defined(__ADSPCORTEXA55__) */

	}
	
     /* IF (Error) */
     if (eResult != ADI_DMA_SUCCESS)
     {
         return (eResult);
     }

	if (pMemSrc != NULL)
	{
#if defined(__ADSPSC598_FAMILY__) && defined(__ADSPCORTEXA55__)
     eResult = ValidateBuffer ((uint32_t)(uint64_t)pMemSrc,
                            MSizePSizeCfg,
                            pStream->SrcCh.pChInfo->pReg->Status,
                            ElementCount,
                            (1u << (int32_t)MSize),
                            0,
                            0);
#else
     eResult = ValidateBuffer ((uint32_t)pMemSrc,
                            MSizePSizeCfg,
                            pStream->SrcCh.pChInfo->pReg->Status,
                            ElementCount,
                            (1u << (int32_t)MSize),
                            0,
                            0);
#endif /* defined(__ADSPSC598_FAMILY__) && defined(__ADSPCORTEXA55__) */
	}
	
     /* IF (Error) */
     if (eResult != ADI_DMA_SUCCESS)
     {
         return (eResult);
     }

#endif /* ADI_DEBUG */

    /* Entering critical region, disable interrupts */
     adi_osal_EnterCriticalRegion();

		/* Destination channel configuration */
		/* IF (we've a valid destination data buffer address) */
		/* Note: Even though we don't allow the external users to pass NULL source/destination descriptor, we are allowing in internally for the CRC driver */
		if (pMemDest != NULL)
		{
#if defined(__ADSPSC598_FAMILY__) && defined(__ADSPCORTEXA55__)
			/* Convert start address to system address space */
			StartAddr = (uint32_t)(uint64_t)adi_rtl_internal_to_system_addr(pMemDest, 1);
#else
			/* Convert start address to system address space */
			StartAddr = adi_rtl_internal_to_system_addr((uint32_t)pMemDest, 1);
#endif /* defined(__ADSPSC598_FAMILY__) && defined(__ADSPCORTEXA55__) */

/* IF (DMA cache management enabled) */
#if (__ADI_MDMA_CACHE_MANAGEMENT == 1)

#if defined(__ADSPSC598_FAMILY__) && defined(__ADSPCORTEXA55__)
			/* Flush the data buffer */
			adi_dma_FlushCache ((void *)(uint64_t)StartAddr,
								ElementCount,
								Xmod,
								1,
								0,
								Xmod, ADI_FLUSH_DATA_INV);
#else
			/* Flush the data buffer */
			adi_dma_FlushCache ((void *)StartAddr,
								ElementCount,
								Xmod,
								1,
								0,
								Xmod, ADI_FLUSH_DATA_INV);
#endif /* defined(__ADSPSC598_FAMILY__) && defined(__ADSPCORTEXA55__) */

#endif
		/* Write the DMA registers other than the config register */
		pStream->DestCh.pCallbackArg=pMemDest;
		pStream->DestCh.pChInfo->pReg->StartAddr=StartAddr;
		pStream->DestCh.pChInfo->pReg->XCount=ElementCount;
		pStream->DestCh.pChInfo->pReg->XModify=Xmod;

 /* Configure BW monitor register if bandwidth functions are enabled*/
#if	(ADI_MDMA_CFG_ENABLE_BANDWIDTH_FUNCTIONS == 1)
		if(pStream->DestCh.pChInfo->pReg->BwMonitor==0)
		{
			pStream->DestCh.pChInfo->pReg->BwMonitor=0x7FFFFFFF;
		}
#endif
		}

		/* Source channel configuration */
		/* IF (we've a valid source data buffer address) */
		/* Note: Even though we don't allow the external users to pass NULL source/destination descriptor, we are allowing in internally for the CRC driver */
		if (pMemSrc != NULL)
		{
#if defined(__ADSPSC598_FAMILY__) && defined(__ADSPCORTEXA55__)
			/* Convert start address to system address space */
			StartAddr = (uint32_t)(uint64_t)adi_rtl_internal_to_system_addr(pMemSrc, 1);
#else
			/* Convert start address to system address space */
			StartAddr = adi_rtl_internal_to_system_addr((uint32_t)pMemSrc, 1);
#endif /* defined(__ADSPSC598_FAMILY__) && defined(__ADSPCORTEXA55__) */


 /* IF (DMA cache management enabled) */
 #if (__ADI_MDMA_CACHE_MANAGEMENT == 1)
#if defined(__ADSPSC598_FAMILY__) && defined(__ADSPCORTEXA55__)
			/* Flush the data buffer */
			adi_dma_FlushCache ((void *)(uint64_t)StartAddr,
								ElementCount,
								Xmod,
								1,
								0,
								Xmod, ADI_FLUSH_DATA_INV);
#else
			/* Flush the data buffer */
			adi_dma_FlushCache ((void *)StartAddr,
								ElementCount,
								Xmod,
								1,
								0,
								Xmod, ADI_FLUSH_DATA_INV);
#endif /* defined(__ADSPSC598_FAMILY__) && defined(__ADSPCORTEXA55__) */

#endif
			/* Write the DMA registers other than the config register */
			pStream->SrcCh.pCallbackArg=pMemSrc;
			pStream->SrcCh.pChInfo->pReg->StartAddr=StartAddr;
			pStream->SrcCh.pChInfo->pReg->XCount=ElementCount;
			pStream->SrcCh.pChInfo->pReg->XModify=2048;

/* Configure BW monitor register if bandwidth functions are enabled*/
#if	(ADI_MDMA_CFG_ENABLE_BANDWIDTH_FUNCTIONS == 1) 
			if(pStream->SrcCh.pChInfo->pReg->BwMonitor==0)
			{
				pStream->SrcCh.pChInfo->pReg->BwMonitor=0x7FFFFFFF;
			}
#endif
		}
					
		/* IF (we've a valid Destination data buffer address) */
		if (pMemDest != NULL)
		{
			/* Kick off destination DMA by writing to the config register */
			pStream->DestCh.pChInfo->pReg->Config=pStream->DestCh.CommonConfig|ADI_MDMA_DMA_CFG_1D_STOP_FLOW|ENUM_DMA_CFG_WRITE|ENUM_DMA_CFG_EN|MSizePSizeCfg|ENUM_DMA_CFG_XCNT_INT;
		}

		/* IF (we've a valid Source data buffer address) */
		if (pMemSrc != NULL)
		{
			/* Kick off source DMA by writing to the config register */
			pStream->SrcCh.pChInfo->pReg->Config=pStream->SrcCh.CommonConfig|ADI_MDMA_DMA_CFG_1D_STOP_FLOW|ENUM_DMA_CFG_READ|ENUM_DMA_CFG_EN|MSizePSizeCfg;
		}
		/* Set the transfer mode to one shot mode */
		pStream->eXferMode = ADI_MDMA_XFER_ONE_SHOT;

		/* Update DMA data transfer status flag */
		pStream->bIsXferInProgress = true;

    /* Re-enable interrupts */
   adi_osal_ExitCriticalRegion();

    return eResult;
}

/**
 * @brief Copies data between memory in a 1D fashion by using multiple descriptors and MSIZE=32 bytes for major portion of the transfer.
 *  
 * @details Generally, MDMA transfers with MSIZE=32 bytes (specially with larger DMA count) are more efficient in terms of performance.
 *			When using the API adi_mdma_Copy1D(), it may not be possible to use MSIZE=32 bytes due to start address and count alignment
 * 			restrictions w.r.t. MSIZE value even if we need to transfer a large block of data. This may result in loss of MDMA performance.
 *			For such cases, this API can be used to divide a single MDMA transfer to more than one (maximum three) descriptor based
 *			transfers which contain one descriptor with MSIZE=32 bytes transferring majority of the data. The remaining non 32 byte
 *			aligned data can be transferred with one (at start or end) or two (both at start and end) descriptors. 
 *
 * @param [in]  hStream         MDMA Stream handle to work on.
 * @param [in]  pMemDest        Memory destination address.
 * @param [in]  pMemSrc         Memory source address.
 * @param [in]  eElementWidth   Data element width in terms of ADI_DMA_MSIZE.
 * @param [in]  ElementCount    Number of data elements to copy.
 * @param [in]  pTempmem		Temporary memory storage required to create descriptors. The memory size needed is 96 bytes and the start address of the memory should be 32 bit aligned.
 * @return  Status
 *          - #ADI_DMA_SUCCESS: Successfully submitted 1D Memory transfer info.
 *          - #ADI_DMA_NOT_OPEN: The MDMA Stream is not open yet.
 *          - #ADI_DMA_BAD_HANDLE: Supplied MDMA Stream handle is invalid.
 *          - #ADI_DMA_ALREADY_RUNNING: Can't submit new buffer when the DMA dataflow is already enabled.
 *
 * @sa	adi_mdma_Copy1D(), adi_dma_ConfigTriggerIn(), adi_dma_ConfigTriggerOut()
 */
ADI_DMA_RESULT adi_mdma_Copy1DAuto(
    ADI_DMA_STREAM_HANDLE           hStream,
    void                            *pMemDest,
    void                            *pMemSrc,
	ADI_DMA_MSIZE                   eElementWidth,
    uint32_t                        ElementCount,
	void	 						*pTempmem
	)
{
	/* Return code */
    ADI_DMA_RESULT  eResult = ADI_DMA_SUCCESS;

	/* MDMA stream to work on */
    ADI_MDMA_STREAM     *pStream = (ADI_MDMA_STREAM *) hStream;
	
    /* Some temporary variables */
    uint32_t  MSize, MSizeCfg, PSizeCfg, PSize, Psizemax, ElementCountInBytes, StartAdd, DMADir, IntCfg, DMACount1, DMACount2, DMACount3, Trigin, Trigout;
	
	/* Start address for the source descriptors */
	uint32_t *pSrcDesc = (uint32_t*)pTempmem+12;
	
	/* Start address for the destination descriptors */
	uint32_t *pDestDesc = (uint32_t*)pTempmem;
	
	/* Pointer to the user passed TCB memory */
	uint32_t *pTCBMem=(uint32_t*)pTempmem;

	int  i, j;

	/* IF (Debug information enabled) */
#if defined(ADI_DEBUG)

	/* IF (MDMA Stream Handle is invalid) */
	if (ValidateStreamHandle (hStream) != ADI_DMA_SUCCESS)
	{
		return (ADI_DMA_BAD_HANDLE);
	}

    /* IF (Stream is not in use) */
    if (pStream->DestCh.pChInfo->pMdmaStream == NULL)
    {
        return (ADI_DMA_NOT_OPEN);
    }

	/* IF (Source address is NULL), return error */
	if (pMemSrc == NULL)
	{
		return (ADI_DMA_INVALID_NULL_POINTER);
	}
	 
	/* Element width less than 4 bytes are not allowed for MDMA streams 2 and 3 */
	if((pStream->SrcCh.pChInfo->ChannelID==CHID_MDMA2_SRC)||(pStream->SrcCh.pChInfo->ChannelID==CHID_MDMA3_SRC))
	{
		if((eElementWidth==ADI_DMA_MSIZE_1BYTE)||(eElementWidth==ADI_DMA_MSIZE_2BYTES))
		{
			return (ADI_DMA_BAD_MSIZE);
		}
	}
#endif /* ADI_DEBUG */

    /* IF (DMA channel is already in the middle of a data transfer) */
    if (pStream->bIsXferInProgress)
    {
        /* Report failure as DMA channel already running */
        return (ADI_DMA_ALREADY_RUNNING);
    }

	 /* Get MSize value */
    MSize = ((uint32_t)eElementWidth >> BITP_DMA_CFG_MSIZE);
	
	/* PSIZE should be less or equal to DMA_STAT.PBWID, keeping it maximum */
    Psizemax = ((pStream->SrcCh.pChInfo->pReg->Status)&BITM_DMA_STAT_PBWID)>>BITP_DMA_STAT_PBWID;

	PSize = Psizemax;
	
    /* PSIZE should be less than or equal to MSIZE */
    if(PSize>MSize)
    {
    	PSize=MSize;
    }

    /* Calculate MSIZE and PSIZE field values of the config register */
    MSizeCfg = (MSize<<BITP_DMA_CFG_MSIZE);
    PSizeCfg = (PSize<<BITP_DMA_CFG_PSIZE);

	/* Get byte equivalent of MSize and calculate Element count in bytes */
    ElementCountInBytes = (ElementCount * (uint32_t)(1u << MSize));
	
	/* Reduce the iterations to 1 if destination address is not passed */
	if (pMemDest == NULL)
	{
		j=1;
	}
	else
	{
		j=2;
	}
		
    /* Configure source and destination descriptors */
	for(i=0;i<j;i++)
	{
		/* Set start address, DMA direction and interrupt/trigger out configuration for source/destination channel */
		if(i == 0)
		{
#if defined(__ADSPSC598_FAMILY__) && defined(__ADSPCORTEXA55__)
			StartAdd = (uint32_t)(uint64_t)pMemDest;
#else
			StartAdd = (uint32_t)pMemDest;
#endif /* defined(__ADSPSC598_FAMILY__) && defined(__ADSPCORTEXA55__) */

			DMADir = 	ENUM_DMA_CFG_WRITE;
			IntCfg = ENUM_DMA_CFG_XCNT_INT;
			Trigout= pStream->SrcCh.CommonConfig&(BITM_DMA_CFG_TRIG);
		}
		else
		{
#if defined(__ADSPSC598_FAMILY__) && defined(__ADSPCORTEXA55__)
			StartAdd = (uint32_t)(uint64_t)pMemSrc;
#else
			StartAdd = (uint32_t)pMemSrc;
#endif /* defined(__ADSPSC598_FAMILY__) && defined(__ADSPCORTEXA55__) */

			DMADir	= 	ENUM_DMA_CFG_READ;
			IntCfg = 0;
			Trigout= pStream->SrcCh.CommonConfig&(BITM_DMA_CFG_TRIG);
		}
		
		/* First descriptor start address */
		pTCBMem[0]=adi_rtl_internal_to_system_addr(StartAdd, 1);
		
		/* If (Total number of bytes < 32) - need only one descriptor */
		if((ElementCountInBytes>>5)==0) 		
		{
			/* Configure in stop mode */
			pTCBMem[1]=ADI_MDMA_DMA_CFG_1D_STOP_FLOW|DMADir|ENUM_DMA_CFG_EN|MSizeCfg|PSizeCfg|IntCfg|Trigout;
			pTCBMem[2]=ElementCount;
			pTCBMem[3]=(uint32_t)(1u << MSize);
		}

		/* Else IF(Start address is 32 bytes aligned, need <3 descriptors)*/
		else if((StartAdd&0x0000001F)==0)
		{
			/* No. of bytes for the first descriptor */
			DMACount1=(ElementCountInBytes>>5);
			
			/* Configure first descriptor start address, count and modifier */
			pTCBMem[2]=DMACount1;
			pTCBMem[3]=32;

			/* Calculate the remaining bytes for the second descriptor  */
			DMACount1=DMACount1<<5;
			DMACount2=ElementCountInBytes-DMACount1;
			
			/* IF (No remaining bytes), configure first descriptor in stop mode */
			if(DMACount2==0)
			{
				pTCBMem[1]=ADI_MDMA_DMA_CFG_1D_STOP_FLOW|DMADir|ENUM_DMA_CFG_EN|PSizeCfg|ENUM_DMA_CFG_MSIZE32|IntCfg|Trigout;
			}
			/* Else, configure first descriptor in list mode and second descriptor in stop mode */
			else
			{
				pTCBMem[1]=ADI_MDMA_DMA_CFG_1D_ARRAY|DMADir|ENUM_DMA_CFG_MSIZE32|PSizeCfg;
				pTCBMem[4]=adi_rtl_internal_to_system_addr((StartAdd+DMACount1),1);
				pTCBMem[5]=ADI_MDMA_DMA_CFG_1D_STOP_FLOW|DMADir|ENUM_DMA_CFG_EN|PSizeCfg|MSizeCfg|IntCfg|Trigout;
				pTCBMem[6]=DMACount2>>MSize;
				pTCBMem[7]=(uint32_t)(1u << MSize);
			}
		}

		/* Else, need at least two descriptors */
		else
		{
			/* No. of bytes for the first descriptor */
			DMACount1=32-((StartAdd)&0x0000001F);

			/* Remaining bytes */
			DMACount2=ElementCountInBytes-DMACount1;

			/* Third descriptor not needed if remaining bytes <=32 or if the DMA count is 32 byte aligned */
			if((DMACount2<32)||((DMACount2&0x0000001F)==0))
			{
				pTCBMem[1]=ADI_MDMA_DMA_CFG_1D_ARRAY|DMADir|PSizeCfg|MSizeCfg;
				pTCBMem[2]=DMACount1>>MSize;
				pTCBMem[3]=(uint32_t)(1u << MSize);

				pTCBMem[4]=adi_rtl_internal_to_system_addr((StartAdd+DMACount1),1);
				pTCBMem[5]=ADI_MDMA_DMA_CFG_1D_STOP_FLOW|DMADir|ENUM_DMA_CFG_EN|PSizeCfg|ENUM_DMA_CFG_MSIZE32|IntCfg|Trigout;
				pTCBMem[6]=DMACount2>>5;
				pTCBMem[7]=32;
			}
			/* Else, remaining bytes are transferred using third descriptor */
			else
			{
				/* No. of bytes for the third descriptor */
				DMACount3=DMACount2&0x0000001F;

				DMACount2=DMACount2-DMACount3;

				pTCBMem[1]=ADI_MDMA_DMA_CFG_1D_ARRAY|DMADir|PSizeCfg|MSizeCfg;
				pTCBMem[2]=DMACount1>>MSize;
				pTCBMem[3]=(uint32_t)(1u << MSize);

				pTCBMem[4]=adi_rtl_internal_to_system_addr((StartAdd+DMACount1),1);
				pTCBMem[5]=ADI_MDMA_DMA_CFG_1D_ARRAY|DMADir|ENUM_DMA_CFG_MSIZE32|PSizeCfg;
				pTCBMem[6]=DMACount2>>5;
				pTCBMem[7]=32;

				pTCBMem[8]=adi_rtl_internal_to_system_addr((StartAdd+DMACount1+DMACount2),1);
				pTCBMem[9]=ADI_MDMA_DMA_CFG_1D_STOP_FLOW|DMADir|ENUM_DMA_CFG_EN|PSizeCfg|MSizeCfg|IntCfg|Trigout;
				pTCBMem[10]=DMACount3>>MSize;
				pTCBMem[11]=(uint32_t)(1u << MSize);
			}
		}
		/*Increment the pointer to point to source descriptors */
		pTCBMem+=12;
	}

 /* IF (DMA cache management enabled) */
 #if (__ADI_MDMA_CACHE_MANAGEMENT == 1)

#if defined(__ADSPSC598_FAMILY__) && defined(__ADSPCORTEXA55__)
	/* Flush the data buffer */
	adi_dma_FlushCache ((void *)adi_rtl_internal_to_system_addr(pMemSrc, 1),
						ElementCount,
						(int32_t)(1u << MSize),
						1,
						0,
						(uint32_t)(1u << MSize), ADI_FLUSH_DATA_INV);

	adi_dma_FlushCache ((void *)adi_rtl_internal_to_system_addr(pMemDest, 1),
		   			    ElementCount,
					    (int32_t)(1u << MSize),
					    1,
					    0,
					    (uint32_t)(1u << MSize), ADI_FLUSH_DATA_INV);

	adi_dma_FlushCache ((void *)adi_rtl_internal_to_system_addr(pTempmem, 1),
						96,
						1,
						1,
						0,
						1, ADI_FLUSH_DATA_INV);
#else
	/* Flush the data buffer */
	adi_dma_FlushCache ((void *)adi_rtl_internal_to_system_addr((uint32_t)pMemSrc, 1),
						ElementCount,
						(int32_t)(1u << MSize),
						1,
						0,
						(uint32_t)(1u << MSize), ADI_FLUSH_DATA_INV);

	adi_dma_FlushCache ((void *)adi_rtl_internal_to_system_addr((uint32_t)pMemDest, 1),
		   			    ElementCount,
					    (int32_t)(1u << MSize),
					    1,
					    0,
					    (uint32_t)(1u << MSize), ADI_FLUSH_DATA_INV);

	adi_dma_FlushCache ((void *)adi_rtl_internal_to_system_addr((uint32_t)pTempmem, 1),
						96,
						1,
						1,
						0,
						1, ADI_FLUSH_DATA_INV);
#endif /* defined(__ADSPSC598_FAMILY__) && defined(__ADSPCORTEXA55__) */

#endif

	/* Entering critical region, disable interrupts */
    adi_osal_EnterCriticalRegion();

	/* Destination DMA start */
	
		/* IF (we've a valid Destination data buffer address) */
		if (pMemDest != NULL)
		{
			/* Store the callback argument value as start address of the buffer */
			pStream->DestCh.pCallbackArg=pMemDest;

			/* Trigger in setting */
			Trigin = pStream->DestCh.CommonConfig&(ENUM_DMA_CFG_TRGWAIT|ENUM_DMA_CFG_TOV_EN);
			
#if defined(__ADSPSC598_FAMILY__) && defined(__ADSPCORTEXA55__)
			/* Kick off the destination DMA by writing to the current descriptor pointer and config register */
			pStream->DestCh.pChInfo->pReg->CurrDescPtr=(uint32_t)(uint64_t)adi_rtl_internal_to_system_addr(pDestDesc,1);
#else
			/* Kick off the destination DMA by writing to the current descriptor pointer and config register */
			pStream->DestCh.pChInfo->pReg->CurrDescPtr=adi_rtl_internal_to_system_addr((uint32_t)pDestDesc,1);
#endif /* defined(__ADSPSC598_FAMILY__) && defined(__ADSPCORTEXA55__) */

			pStream->DestCh.pChInfo->pReg->Config=Trigin|ADI_MDMA_DMA_CFG_1D_ARRAY|ENUM_DMA_CFG_WRITE|PSizeCfg|ENUM_DMA_CFG_MSIZE04|Trigin;
		}

		/* Source DMA start */
		/* Store the callback argument value as start address of the buffer */
		pStream->SrcCh.pCallbackArg=pMemSrc;

		/* Trigger in setting */
		Trigin = pStream->SrcCh.CommonConfig&(ENUM_DMA_CFG_TRGWAIT|ENUM_DMA_CFG_TOV_EN);
		
#if defined(__ADSPSC598_FAMILY__) && defined(__ADSPCORTEXA55__)
		/* Kick off the source DMA by writing to the current descriptor pointer and config register */
		pStream->SrcCh.pChInfo->pReg->CurrDescPtr=(uint32_t)(uint64_t)adi_rtl_internal_to_system_addr(pSrcDesc,1);
#else
		/* Kick off the source DMA by writing to the current descriptor pointer and config register */
		pStream->SrcCh.pChInfo->pReg->CurrDescPtr=adi_rtl_internal_to_system_addr((uint32_t)pSrcDesc,1);
#endif /* defined(__ADSPSC598_FAMILY__) && defined(__ADSPCORTEXA55__) */

		pStream->SrcCh.pChInfo->pReg->Config=ADI_MDMA_DMA_CFG_1D_ARRAY|ENUM_DMA_CFG_READ|PSizeCfg|ENUM_DMA_CFG_MSIZE04|Trigin;

		/* Set the transfer mode to descriptor array 1D mode */
	 	pStream->eXferMode = ADI_MDMA_XFER_ONE_SHOT;

		/* Update DMA data transfer status flag */
		pStream->bIsXferInProgress = true;

    /* Re-enable interrupts */
   adi_osal_ExitCriticalRegion();

   return (eResult);
}
/**
 * @brief Copies data between memory in Two-Dimensional mode.
 *
 * @param [in]  hStream         MDMA Stream handle to work on.
 * @param [in]  eElementWidth   Data element width in terms of ADI_DMA_MSIZE.
 * @param [in]  pDestDesc       Address of an instance that holds Destination DMA channel
 *                              specific information to perform a Memory DMA copy.
 * @param [in]  pSrcDesc        Address of an instance that holds Source DMA channel
 *                              specific information to perform a Memory DMA copy.
 *
 * @return  Status
 *          - #ADI_DMA_SUCCESS: Successfully submitted 2D Memory transfer info.
 *          - #ADI_DMA_NOT_OPEN: The MDMA Stream is not open yet.
 *          - #ADI_DMA_BAD_HANDLE: Supplied MDMA Stream handle is invalid.
 *          - #ADI_DMA_ALREADY_RUNNING: Can't submit new buffer when the DMA dataflow is already enabled.
 *
 * @sa		adi_dma_ConfigTriggerIn(), adi_dma_ConfigTriggerOut(), adi_mdma_BWLimit(), adi_mdma_BWMonitor(), and adi_mdma_BWMeasure()
 */
ADI_DMA_RESULT adi_mdma_Copy2D(
    ADI_DMA_STREAM_HANDLE           hStream,
    ADI_DMA_MSIZE                   eElementWidth,
    ADI_DMA_2D_MEM_TRANSFER         *pDestDesc,
    ADI_DMA_2D_MEM_TRANSFER         *pSrcDesc)
{
    /* MSize/PSize Configuration */
    uint32_t  MSize, MSizePSizeCfg, PSize, StartAddr;

    /* Return code */
    ADI_DMA_RESULT  eResult = ADI_DMA_SUCCESS;

	/*MDMA stream to work on */
    ADI_MDMA_STREAM     *pStream = (ADI_MDMA_STREAM *) hStream;

/* IF (Debug information enabled) */
#if defined(ADI_DEBUG)

	/* IF (MDMA Stream Handle is invalid) */
	if (ValidateStreamHandle (hStream) != ADI_DMA_SUCCESS)
	{
		return (ADI_DMA_BAD_HANDLE);
	}

    /* IF (Stream is not in use) */
    if (pStream->DestCh.pChInfo->pMdmaStream == NULL)
    {
        return (ADI_DMA_NOT_OPEN);
    }

	/* IF (Source address is NULL), return error */
	 if (pSrcDesc == NULL)
	 {
		 return (ADI_DMA_INVALID_NULL_POINTER);
	 }

	/* Element width less than 4 bytes are not allowed for MDMA streams 2 and 3 */
	if((pStream->SrcCh.pChInfo->ChannelID==CHID_MDMA2_SRC)||(pStream->SrcCh.pChInfo->ChannelID==CHID_MDMA3_SRC))
	{
		if((eElementWidth==ADI_DMA_MSIZE_1BYTE)||(eElementWidth==ADI_DMA_MSIZE_2BYTES))
		{
			return (ADI_DMA_BAD_MSIZE);
		}
	}
#endif /* ADI_DEBUG */

    /* IF (DMA channel is already in the middle of a data transfer) */
    if (pStream->bIsXferInProgress)
    {
        /* Report failure as DMA channel already running */
        return (ADI_DMA_ALREADY_RUNNING);
    }

    /* Get MSize value */
    MSize = ((uint32_t)eElementWidth >> BITP_DMA_CFG_MSIZE);

	/* PSIZE should be less than or equal to DMA_STAT.PBWID, keeping it maximum */
    PSize = ((pStream->SrcCh.pChInfo->pReg->Status)&BITM_DMA_STAT_PBWID)>>BITP_DMA_STAT_PBWID;

    /* PSIZE should be less than or equal to MSIZE */
    if(PSize>MSize)
    {
    	PSize=MSize;
    }

    /* Calculate MSIZE and PSIZE field values of the config register */
    MSizePSizeCfg = (MSize<<BITP_DMA_CFG_MSIZE)|(PSize<<BITP_DMA_CFG_PSIZE);

/* IF (Debug information enabled) */
#if defined(ADI_DEBUG)
#if defined(__ADSPSC598_FAMILY__) && defined(__ADSPCORTEXA55__)
     eResult = ValidateBuffer ((uint32_t)(uint64_t)pDestDesc->pStartAddress,
                            MSizePSizeCfg,
                            pStream->DestCh.pChInfo->pReg->Status,
                            pDestDesc->XCount,
                            pDestDesc->XModify,
                            pDestDesc->YCount,
                            pDestDesc->YModify);
#else
     eResult = ValidateBuffer ((uint32_t)pDestDesc->pStartAddress,
                            MSizePSizeCfg,
                            pStream->DestCh.pChInfo->pReg->Status,
                            pDestDesc->XCount,
                            pDestDesc->XModify,
                            pDestDesc->YCount,
                            pDestDesc->YModify);
#endif /* defined(__ADSPSC598_FAMILY__) && defined(__ADSPCORTEXA55__) */

	 /* IF (Error) */
	 if (eResult != ADI_DMA_SUCCESS)
	 {
		 return (eResult);
	 }

#if defined(__ADSPSC598_FAMILY__) && defined(__ADSPCORTEXA55__)
     eResult = ValidateBuffer ((uint32_t)(uint64_t)pSrcDesc->pStartAddress,
                            MSizePSizeCfg,
                            pStream->SrcCh.pChInfo->pReg->Status,
                            pSrcDesc->XCount,
                            pSrcDesc->XModify,
                            pSrcDesc->YCount,
                            pSrcDesc->YModify);
#else
     eResult = ValidateBuffer ((uint32_t)pSrcDesc->pStartAddress,
                            MSizePSizeCfg,
                            pStream->SrcCh.pChInfo->pReg->Status,
                            pSrcDesc->XCount,
                            pSrcDesc->XModify,
                            pSrcDesc->YCount,
                            pSrcDesc->YModify);
#endif /* defined(__ADSPSC598_FAMILY__) && defined(__ADSPCORTEXA55__) */

	 /* IF (Error) */
	 if (eResult != ADI_DMA_SUCCESS)
	 {
		 return (eResult);
	 }

#endif /* ADI_DEBUG */

    /* Entering critical region, disable interrupts */
     adi_osal_EnterCriticalRegion();

	/* Destination DMA start */
	 
 		/* IF (we've a valid Destination data buffer address) */
		if (pDestDesc != NULL)
		{
#if defined(__ADSPSC598_FAMILY__) && defined(__ADSPCORTEXA55__)
			/* Convert start addresss to system address space(uint32_t) */
			StartAddr = (uint32_t)(uint64_t)adi_rtl_internal_to_system_addr(pDestDesc->pStartAddress, 1);
#else
			/* Convert start addresss to system address space */
			StartAddr = adi_rtl_internal_to_system_addr((uint32_t)pDestDesc->pStartAddress, 1);
#endif /* defined(__ADSPSC598_FAMILY__) && defined(__ADSPCORTEXA55__) */


 /* IF (DMA cache management enabled) */
 #if (__ADI_MDMA_CACHE_MANAGEMENT == 1)
#if defined(__ADSPSC598_FAMILY__) && defined(__ADSPCORTEXA55__)
	         /* Flush the data buffer */
	         adi_dma_FlushCache ((void *)(uint64_t)StartAddr,
	        		 	 	 	pDestDesc->XCount,
								pDestDesc->XModify,
								pDestDesc->YCount,
								pDestDesc->YModify,
								(1<<MSize), ADI_FLUSH_DATA_INV);
#else
	         /* Flush the data buffer */
	         adi_dma_FlushCache ((void *)StartAddr,
	        		 	 	 	pDestDesc->XCount,
								pDestDesc->XModify,
								pDestDesc->YCount,
								pDestDesc->YModify,
								(1<<MSize), ADI_FLUSH_DATA_INV);
#endif /* defined(__ADSPSC598_FAMILY__) && defined(__ADSPCORTEXA55__) */

 #endif

		    /* Write the DMA registers other than the config register */
			pStream->DestCh.pCallbackArg=(void*)pDestDesc->pStartAddress;
			pStream->DestCh.pChInfo->pReg->StartAddr=StartAddr;
			pStream->DestCh.pChInfo->pReg->XCount=pDestDesc->XCount;
			pStream->DestCh.pChInfo->pReg->YCount=pDestDesc->YCount;
			pStream->DestCh.pChInfo->pReg->XModify=pDestDesc->XModify;
			pStream->DestCh.pChInfo->pReg->YModify=pDestDesc->YModify;

/* Configure BW monitor register if bandwidth functions are enabled*/
#if	(ADI_MDMA_CFG_ENABLE_BANDWIDTH_FUNCTIONS == 1)
			if(pStream->DestCh.pChInfo->pReg->BwMonitor==0)
			{
				pStream->DestCh.pChInfo->pReg->BwMonitor=0x7FFFFFFF;
			}
#endif
		}

#if defined(__ADSPSC598_FAMILY__) && defined(__ADSPCORTEXA55__)
		/* Source DMA start */
			/* Convert start addresss to system address space */
			StartAddr = (uint32_t)(uint64_t)adi_rtl_internal_to_system_addr(pSrcDesc->pStartAddress, 1);
#else
		/* Source DMA start */
			/* Convert start addresss to system address space */
			StartAddr = adi_rtl_internal_to_system_addr((uint32_t)pSrcDesc->pStartAddress, 1);
#endif /* defined(__ADSPSC598_FAMILY__) && defined(__ADSPCORTEXA55__) */


/* IF (DMA cache management enabled) */
#if (__ADI_MDMA_CACHE_MANAGEMENT == 1)

#if defined(__ADSPSC598_FAMILY__) && defined(__ADSPCORTEXA55__)
			/* Flush the data buffer */
			adi_dma_FlushCache ((void *)(uint64_t)StartAddr,
							    pSrcDesc->XCount,
								pSrcDesc->XModify,
								pSrcDesc->YCount,
								pSrcDesc->YModify,
								(1<<MSize), ADI_FLUSH_DATA_INV);
#else
			/* Flush the data buffer */
			adi_dma_FlushCache ((void *)StartAddr,
							    pSrcDesc->XCount,
								pSrcDesc->XModify,
								pSrcDesc->YCount,
								pSrcDesc->YModify,
								(1<<MSize), ADI_FLUSH_DATA_INV);
#endif /* defined(__ADSPSC598_FAMILY__) && defined(__ADSPCORTEXA55__) */

#endif

		/* Write the DMA registers other than the config register */
		pStream->SrcCh.pCallbackArg=(void*)pSrcDesc->pStartAddress;
		pStream->SrcCh.pChInfo->pReg->StartAddr=StartAddr;
		pStream->SrcCh.pChInfo->pReg->XCount=pSrcDesc->XCount;
		pStream->SrcCh.pChInfo->pReg->YCount=pSrcDesc->YCount;
		pStream->SrcCh.pChInfo->pReg->XModify=pSrcDesc->XModify;
		pStream->SrcCh.pChInfo->pReg->YModify=pSrcDesc->YModify;

/* Configure BW monitor register if bandwidth functions are enabled*/
#if	(ADI_MDMA_CFG_ENABLE_BANDWIDTH_FUNCTIONS == 1)
		if(pStream->SrcCh.pChInfo->pReg->BwMonitor==0)
		{
			pStream->SrcCh.pChInfo->pReg->BwMonitor=0x7FFFFFFF;
		}
#endif
			
		/* IF (we've a valid Destination data buffer address) */
		if (pDestDesc != NULL)
		{
			/* Kick off destination DMA by writing to the config register */
			pStream->DestCh.pChInfo->pReg->Config=pStream->DestCh.CommonConfig|ADI_MDMA_DMA_CFG_2D_STOP_FLOW|ENUM_DMA_CFG_WRITE|MSizePSizeCfg|ENUM_DMA_CFG_YCNT_INT;
		}

		/* Kick off source DMA by writing to the config register */
		pStream->SrcCh.pChInfo->pReg->Config=pStream->SrcCh.CommonConfig|ADI_MDMA_DMA_CFG_2D_STOP_FLOW|ENUM_DMA_CFG_READ|ENUM_DMA_CFG_EN|MSizePSizeCfg;

		/* Set the transfer mode to one shot mode */
		pStream->eXferMode = ADI_MDMA_XFER_ONE_SHOT;

		/* Update DMA data transfer status flag */
		pStream->bIsXferInProgress = true;

    /* Re-enable interrupts */
    adi_osal_ExitCriticalRegion();

	return eResult;
}
	
/**
 * @brief Copies data between memory using List descriptor(s).
 *
 * @param [in]  hStream     MDMA Stream handle to work on.
 * @param [in]  pDestDesc   Address of the first list descriptor in chain that is to be
 *                          submitted to the Destination DMA channel.
 * @param [in]  pSrcDesc    Address of the first list descriptor in chain that is to be
 *                          submitted to the Source DMA channel.
 *
 * @return  Status
 *          - #ADI_DMA_SUCCESS: Successfully submitted Memory transfer descriptor chain.
 *          - #ADI_DMA_NOT_OPEN: The MDMA Stream is not open yet.
 *          - #ADI_DMA_BAD_HANDLE: Supplied MDMA Stream handle is invalid.
 *          - #ADI_DMA_ALREADY_RUNNING: Can't submit new buffer when the DMA dataflow is already enabled.
 *          - #ADI_DMA_NO_BUFFER_TO_PROCESS: There is no valid data buffer to process.
 * @note 
 *			- By default, this API prepares the hardware usable descriptors for each call. 
 *			  However, API adi_mdma_EnableDescListReuse can be used to skip the descriptor preparation to
 *			  allow reuse of the descriptors already prepared. Note that descriptor reuse for descriptor list mode is not supported
 *			  if the macro ADI_MDMA_CFG_LIST_MULTIPLE_CALLBACKS_ENABLE is enabled.
 *
 *			- By default, the DMA done interrupt in list mode is generated after the end of 
 *  		  the last work unit in the descriptor list irrespective of the bCallbackWhenDone
 *			  descriptor parameter. However, the configuration macro ADI_MDMA_CFG_LIST_MULTIPLE_CALLBACKS_ENABLE 
 *			  can be enabled to allow generation of interrupt for more than one descriptors 
 *			  as per the configuration of bCallbackWhenDone parameter. Note that enabling this macro
 *            might result in a race condition for smaller work units and consequently one or more DMA interrupts
 *            might be missed. It is user's responsibility to take care of such conditions in the application.   
 *
 * @sa		  adi_mdma_EnableDescListReuse()
 */
ADI_DMA_RESULT adi_mdma_CopyList(
    ADI_DMA_STREAM_HANDLE           hStream,
    ADI_DMA_DESC_LIST_MODE          *pDestDesc,
    ADI_DMA_DESC_LIST_MODE          *pSrcDesc)
{
    /* Return code */
	ADI_DMA_RESULT  eResult = ADI_DMA_SUCCESS;

    /* DMA Channel info to work on */
    ADI_MDMA_STREAM     *pStream = (ADI_MDMA_STREAM *) hStream;

    /* PSIZE value for source channel */
    uint32_t PSizeSrcCfg;

    /* PSIZE value for destination channel */
    uint32_t PSizeDestCfg;

 /* IF (Debug information enabled) */
 #if defined(ADI_DEBUG)

	 /* IF (MDMA Stream Handle is invalid) */
	 if (ValidateStreamHandle (hStream) != ADI_DMA_SUCCESS)
	 {
		 return (ADI_DMA_BAD_HANDLE);
	 }

     /* IF (Stream is not in use) */
     if (pStream->DestCh.pChInfo->pMdmaStream == NULL)
     {
         return (ADI_DMA_NOT_OPEN);
     }

	 /* IF (Both Source and Destination descriptor pointers are invalid) */
	 if ((pDestDesc == NULL) && (pSrcDesc == NULL))
	 {
		 /* Return failure as we've no valid buffers to process */
		 return (ADI_DMA_NO_BUFFER_TO_PROCESS);
	 }

	 /* Validate Descriptor List */
	 eResult = ValidateDescList (pStream, pSrcDesc, pDestDesc);

	 /* IF (Descriptor List is invalid) */
	 if (eResult != ADI_DMA_SUCCESS)
	 {
		 return (eResult);
	 }
	 
	 /* IF (Source descriptor is NULL), return error */
	 if (pSrcDesc == NULL)
	 {
		 return (ADI_DMA_INVALID_NULL_POINTER);
	 }

 #endif /* ADI_DEBUG */

	/* IF (DMA channel is already in the middle of a data transfer) */
	if (pStream->bIsXferInProgress)
	{
		/* Report failure as DMA channel already running */
		return (ADI_DMA_ALREADY_RUNNING);
	}

	/* Prepare destination descriptors if needed*/
	if(pStream->DestCh.bDescReuse == false)
	{
		if (pDestDesc != NULL)
		{
			PrepareDescList((ADI_DMA_CHANNEL_HANDLE)&pStream->DestCh, false, pDestDesc);
		}
	}

	/* Prepare source descriptors if needed*/
	if(pStream->SrcCh.bDescReuse == false)
	{
		PrepareDescList((ADI_DMA_CHANNEL_HANDLE)&pStream->SrcCh, true, pSrcDesc);
	}

    /* Copy PSIZE value from the first descriptor */
    PSizeSrcCfg = pSrcDesc->Config&BITM_DMA_CFG_PSIZE;
    PSizeDestCfg =pDestDesc->Config&BITM_DMA_CFG_PSIZE;

    /* Entering critical region, disable interrupts */
    adi_osal_EnterCriticalRegion();

/*Clearing BW limit and monitor registers if BW functions are enabled*/
#if	(ADI_MDMA_CFG_ENABLE_BANDWIDTH_FUNCTIONS == 1)

    	pStream->DestCh.pChInfo->pReg->BwLimit=0;
    	pStream->SrcCh.pChInfo->pReg->BwLimit=0;
    	pStream->DestCh.pChInfo->pReg->BwMonitor=0;
    	pStream->SrcCh.pChInfo->pReg->BwMonitor=0;
#endif


	/* Destination DMA start */
		/* IF (we've a valid Destination data buffer address) */
		if (pDestDesc != NULL)
		{
#if defined(__ADSPSC598_FAMILY__) && defined(__ADSPCORTEXA55__)
			/* Write the next descriptor register with the first descriptor address */
			pStream->DestCh.pChInfo->pReg->NextDescPtr=(uint32_t)(uint64_t)adi_rtl_internal_to_system_addr(pDestDesc,1);
#else
			/* Write the next descriptor register with the first descriptor address */
			pStream->DestCh.pChInfo->pReg->NextDescPtr=adi_rtl_internal_to_system_addr((uint32_t)pDestDesc,1);
#endif /* defined(__ADSPSC598_FAMILY__) && defined(__ADSPCORTEXA55__) */


			/* Kick off the destination DMA in 1D/2D mode */
			if(pDestDesc->YCount>0)
			{
				pStream->DestCh.pChInfo->pReg->Config=pStream->DestCh.CommonConfig|ADI_MDMA_DMA_CFG_DEST_LLIST_2D|PSizeDestCfg|ENUM_DMA_CFG_MSIZE04;
			}
			else
			{
				pStream->DestCh.pChInfo->pReg->Config=pStream->DestCh.CommonConfig|ADI_MDMA_DMA_CFG_DEST_LLIST_1D|PSizeDestCfg|ENUM_DMA_CFG_MSIZE04;
			}
		 }

#if defined(__ADSPSC598_FAMILY__) && defined(__ADSPCORTEXA55__)
		/* Source DMA start */
			/* Write the next descriptor register with the first descriptor address */
			pStream->SrcCh.pChInfo->pReg->NextDescPtr=(uint32_t)(uint64_t)adi_rtl_internal_to_system_addr(pSrcDesc,1);
#else
		/* Source DMA start */
			/* Write the next descriptor register with the first descriptor address */
			pStream->SrcCh.pChInfo->pReg->NextDescPtr=adi_rtl_internal_to_system_addr((uint32_t)pSrcDesc,1);
#endif /* defined(__ADSPSC598_FAMILY__) && defined(__ADSPCORTEXA55__) */


		/* Kick off the source DMA in 1D/2D mode */
		if(pSrcDesc->YCount>0)
		{
			pStream->SrcCh.pChInfo->pReg->Config=pStream->SrcCh.CommonConfig|ADI_MDMA_DMA_CFG_SRC_LLIST_2D|PSizeSrcCfg|ENUM_DMA_CFG_MSIZE04;
		}
		else
		{
			pStream->SrcCh.pChInfo->pReg->Config=pStream->SrcCh.CommonConfig|ADI_MDMA_DMA_CFG_SRC_LLIST_1D|PSizeSrcCfg|ENUM_DMA_CFG_MSIZE04;
		}

		/* Set the transfer mode to descriptor list mode */
		pStream->eXferMode = ADI_MDMA_XFER_LARGE_LIST;

		/* Update DMA data transfer status flag */
		pStream->bIsXferInProgress = true;

    /* Re-enable interrupts */
   adi_osal_ExitCriticalRegion();

   return eResult;
}


/**
 * @brief Copies data between memory using One-Dimensional Array descriptor(s).
 *
 * @param [in]  hStream    		MDMA Stream handle to work on.
 * @param [in]  pDestDesc   	Address of the first array descriptor in chain that is to be
 *                          	submitted to the Destination DMA channel.
 * @param [in]  pSrcDesc    	Address of the first array descriptor in chain that is to be
 *                          	submitted to the Source DMA channel.
 * @param [in]  NumEntriesDest  Number of destination descriptor entries in the array.
 * @param [in]  NumEntriesSrc   Number of source descriptor entries in the array.
 *
 * @return  Status
 *          - #ADI_DMA_SUCCESS: Successfully submitted 1D descriptor array.
 *          - #ADI_DMA_NOT_OPEN: DMA Channel is not open yet.
 *          - #ADI_DMA_BAD_HANDLE: Bad DMA Handle.
 * @note 
 *			- By default, this API prepares the hardware usable descriptors for each call. 
 *			  However, API adi_mdma_EnableDescListReuse can be used to skip the descriptor preparation to
 *			  allow reuse of the descriptors already prepared. 
 * @sa		  adi_mdma_EnableDescListReuse()
 */
ADI_DMA_RESULT adi_mdma_Copy1DArray(
    ADI_DMA_STREAM_HANDLE           hStream,
    ADI_DMA_DESC_ARRAY_1D_MODE      *pDestDesc,
    ADI_DMA_DESC_ARRAY_1D_MODE      *pSrcDesc,
    uint16_t                        NumEntriesDest,
	uint16_t                        NumEntriesSrc)
{
    /* Return code */
	ADI_DMA_RESULT  eResult = ADI_DMA_SUCCESS;

    /* DMA Channel info to work on */
    ADI_MDMA_STREAM     *pStream = (ADI_MDMA_STREAM *) hStream;

    /* PSIZE value for source channel */
    uint32_t PSizeSrcCfg;

    /* PSIZE value for destination channel */
    uint32_t PSizeDestCfg;

/* IF (Debug information enabled) */
#if defined(ADI_DEBUG)

	/* IF (MDMA Stream Handle is invalid) */
	if (ValidateStreamHandle (hStream) != ADI_DMA_SUCCESS)
	{
		return (ADI_DMA_BAD_HANDLE);
	}

	/* IF (Stream is not in use) */
	if (pStream->DestCh.pChInfo->pMdmaStream == NULL)
	{
		return (ADI_DMA_NOT_OPEN);
	}

	/* IF (Descriptor array is invalid) */
	if (eResult != ADI_DMA_SUCCESS)
	{
		return (eResult);
	}

	 /* IF (Source descriptor is NULL), return error */
	 if (pSrcDesc == NULL)
	 {
		 return (ADI_DMA_INVALID_NULL_POINTER);
	 }

#endif /* ADI_DEBUG */

    /* Copy PSIZE value from the first descriptor */
    PSizeSrcCfg = pSrcDesc->Config&BITM_DMA_CFG_PSIZE;
    PSizeDestCfg =pDestDesc->Config&BITM_DMA_CFG_PSIZE;

    /* IF (DMA channel is already in the middle of a data transfer) */
    if (pStream->bIsXferInProgress)
    {
        /* Report failure as DMA channel already running */
        return (ADI_DMA_ALREADY_RUNNING);
    }

    /* Prepare destination descriptors */
	if(pStream->DestCh.bDescReuse == false)
	{
		if (pDestDesc != NULL)
		{
			PrepareDescArray1D((ADI_DMA_CHANNEL_HANDLE)&pStream->DestCh, false, pDestDesc, NumEntriesDest);
		}
	}
	
	/* Prepare source descriptors */
	if(pStream->SrcCh.bDescReuse == false)
	{
		PrepareDescArray1D((ADI_DMA_CHANNEL_HANDLE)&pStream->SrcCh, true, pSrcDesc, NumEntriesSrc);
	}
 
    /* Entering critical region, disable interrupts */
    adi_osal_EnterCriticalRegion();

/*Clearing BW limit and monitor registers if BW functions are enabled*/
#if	(ADI_MDMA_CFG_ENABLE_BANDWIDTH_FUNCTIONS == 1)
		pStream->DestCh.pChInfo->pReg->BwLimit=0;
		pStream->SrcCh.pChInfo->pReg->BwLimit=0;
		pStream->DestCh.pChInfo->pReg->BwMonitor=0;
		pStream->SrcCh.pChInfo->pReg->BwMonitor=0;
#endif

	/* Destination DMA Start */
		/* IF (we've a valid Destination data buffer address) */
		if (pDestDesc != NULL)
		{
			pStream->DestCh.pCallbackArg=pDestDesc;
#if defined(__ADSPSC598_FAMILY__) && defined(__ADSPCORTEXA55__)
			pStream->DestCh.pChInfo->pReg->CurrDescPtr=(uint32_t)(uint64_t)adi_rtl_internal_to_system_addr(pDestDesc,1);
#else
			pStream->DestCh.pChInfo->pReg->CurrDescPtr=adi_rtl_internal_to_system_addr((uint32_t)pDestDesc,1);
#endif /* defined(__ADSPSC598_FAMILY__) && defined(__ADSPCORTEXA55__) */

			pStream->DestCh.pChInfo->pReg->Config=pStream->DestCh.CommonConfig|ADI_MDMA_DMA_CFG_1D_ARRAY|ENUM_DMA_CFG_WRITE|ENUM_DMA_CFG_MSIZE04|PSizeDestCfg;
		}

	/* Source DMA Start */
		pStream->SrcCh.pCallbackArg=pSrcDesc;
#if defined(__ADSPSC598_FAMILY__) && defined(__ADSPCORTEXA55__)
		pStream->SrcCh.pChInfo->pReg->CurrDescPtr=(uint32_t)(uint64_t)adi_rtl_internal_to_system_addr(pSrcDesc,1);
#else
		pStream->SrcCh.pChInfo->pReg->CurrDescPtr=adi_rtl_internal_to_system_addr((uint32_t)pSrcDesc,1);
#endif /* defined(__ADSPSC598_FAMILY__) && defined(__ADSPCORTEXA55__) */

		pStream->SrcCh.pChInfo->pReg->Config=pStream->SrcCh.CommonConfig|ADI_MDMA_DMA_CFG_1D_ARRAY|ENUM_DMA_CFG_READ|ENUM_DMA_CFG_MSIZE04|PSizeSrcCfg;

		/* Set the transfer mode to descriptor array 1D mode */
		pStream->eXferMode = ADI_MDMA_XFER_ARRAY_1D;

		/* Update DMA data transfer status flag */
		pStream->bIsXferInProgress = true;

    /* Re-enable interrupts */
   adi_osal_ExitCriticalRegion();

   return eResult;
}

/**
 * @brief Copies data between memory using Two-Dimensional Array descriptor(s).
 *
 * @param [in]  hStream    		 MDMA Stream handle to work on.
 * @param [in]  pDestDesc  		 Address of the first array descriptor in chain that is to be
 *                         		 submitted to the Destination DMA channel.
 * @param [in]  pSrcDesc         Address of the first array descriptor in chain that is to be
 *                          	 submitted to the Source DMA channel.
 * @param [in]  NumEntriesDest   Number of destination descriptor entries in the array.
 * @param [in]  NumEntriesSrc    Number of source descriptor entries in the array.
 *
 * @return  Status
 *          - #ADI_DMA_SUCCESS: Successfully submitted 2D descriptor array.
 *          - #ADI_DMA_NOT_OPEN: DMA Channel is not open yet.
 *          - #ADI_DMA_BAD_HANDLE: Bad DMA Handle.
 * @note 
 *			- By default, this API prepares the hardware usable descriptors for each call. 
 *			  However, API adi_mdma_EnableDescListReuse can be used to skip the descriptor preparation to
 *			  allow reuse of the descriptors already prepared. 
 *
 * @sa		  adi_mdma_EnableDescListReuse()
 */
ADI_DMA_RESULT adi_mdma_Copy2DArray(
    ADI_DMA_STREAM_HANDLE           hStream,
    ADI_DMA_DESC_ARRAY_2D_MODE      *pDestDesc,
    ADI_DMA_DESC_ARRAY_2D_MODE      *pSrcDesc,
    uint16_t                        NumEntriesDest,
	uint16_t                        NumEntriesSrc)
{
    /* Return code */
	ADI_DMA_RESULT  eResult = ADI_DMA_SUCCESS;

    /* DMA Channel info to work on */
    ADI_MDMA_STREAM     *pStream = (ADI_MDMA_STREAM *) hStream;

    /* PSIZE value for source channel */
    uint32_t PSizeSrcCfg;

    /* PSIZE value for destination channel */
    uint32_t PSizeDestCfg;

/* IF (Debug information enabled) */
#if defined(ADI_DEBUG)

	/* IF (MDMA Stream Handle is invalid) */
	if (ValidateStreamHandle (hStream) != ADI_DMA_SUCCESS)
	{
		return (ADI_DMA_BAD_HANDLE);
	}

	/* IF (Stream is not in use) */
	if (pStream->DestCh.pChInfo->pMdmaStream == NULL)
	{
		return (ADI_DMA_NOT_OPEN);
	}
	
	/* IF (Source descriptor is NULL), return error */
	 if (pSrcDesc == NULL)
	 {
		 return (ADI_DMA_INVALID_NULL_POINTER);
	 }

#endif /* ADI_DEBUG */

    /* Copy PSIZE value from the first descriptor */
     PSizeSrcCfg = pSrcDesc->Config&BITM_DMA_CFG_PSIZE;
     PSizeDestCfg =pDestDesc->Config&BITM_DMA_CFG_PSIZE;

    /* IF (DMA channel is already in the middle of a data transfer) */
    if (pStream->bIsXferInProgress)
    {
        /* Report failure as DMA channel already running */
        return (ADI_DMA_ALREADY_RUNNING);
    }

	/* Prepare destination descriptors */
	if(pStream->DestCh.bDescReuse == false)
	{
		if (pDestDesc != NULL)
		{
			PrepareDescArray2D((ADI_DMA_CHANNEL_HANDLE)&pStream->DestCh, false, pDestDesc, NumEntriesDest);
		}
	}
	
	/* Prepare source descriptors */
	if(pStream->SrcCh.bDescReuse == false)
	{
		PrepareDescArray2D((ADI_DMA_CHANNEL_HANDLE)&pStream->SrcCh, true, pSrcDesc, NumEntriesSrc);
	}
 
    /* Entering critical region, disable interrupts */
    adi_osal_EnterCriticalRegion();

/*Clearing BW limit and monitor registers if BW functions are enabled*/
#if	(ADI_MDMA_CFG_ENABLE_BANDWIDTH_FUNCTIONS == 1)
		pStream->DestCh.pChInfo->pReg->BwLimit=0;
		pStream->SrcCh.pChInfo->pReg->BwLimit=0;
		pStream->DestCh.pChInfo->pReg->BwMonitor=0;
		pStream->SrcCh.pChInfo->pReg->BwMonitor=0;
#endif

	/* Destination DMA Start */
		/* IF (we've a valid Destination data buffer address) */
		if (pDestDesc != NULL)
		{
			pStream->DestCh.pCallbackArg=pDestDesc;
#if defined(__ADSPSC598_FAMILY__) && defined(__ADSPCORTEXA55__)
			pStream->DestCh.pChInfo->pReg->CurrDescPtr=(uint32_t)(uint64_t)adi_rtl_internal_to_system_addr(pDestDesc,1);
#else
			pStream->DestCh.pChInfo->pReg->CurrDescPtr=adi_rtl_internal_to_system_addr((uint32_t)pDestDesc,1);
#endif /* defined(__ADSPSC598_FAMILY__) && defined(__ADSPCORTEXA55__) */
			pStream->DestCh.pChInfo->pReg->Config=pStream->DestCh.CommonConfig|ADI_MDMA_DMA_CFG_2D_ARRAY|ENUM_DMA_CFG_WRITE|ENUM_DMA_CFG_MSIZE04|PSizeDestCfg;
		}

	/* Source DMA Start */
		pStream->SrcCh.pCallbackArg=pSrcDesc;
#if defined(__ADSPSC598_FAMILY__) && defined(__ADSPCORTEXA55__)
		pStream->SrcCh.pChInfo->pReg->CurrDescPtr=(uint32_t)(uint64_t)adi_rtl_internal_to_system_addr(pSrcDesc,1);
#else
		pStream->SrcCh.pChInfo->pReg->CurrDescPtr=adi_rtl_internal_to_system_addr((uint32_t)pSrcDesc,1);
#endif /* defined(__ADSPSC598_FAMILY__) && defined(__ADSPCORTEXA55__) */

		pStream->SrcCh.pChInfo->pReg->Config=pStream->SrcCh.CommonConfig|ADI_MDMA_DMA_CFG_2D_ARRAY|ENUM_DMA_CFG_READ|ENUM_DMA_CFG_MSIZE04|PSizeSrcCfg;

		/* Set the transfer mode to descriptor array 1D mode */
		pStream->eXferMode = ADI_MDMA_XFER_ARRAY_2D;

		/* Update DMA data transfer status flag */
		pStream->bIsXferInProgress = true;

    /* Re-enable interrupts */
   adi_osal_ExitCriticalRegion();

   return eResult;
}

/**
 * @brief Sets the descriptor reuse mode for descriptor based Memory DMA copy transfer modes.
 *
 * @details 
 *			- By default, the descriptor based Memory DMA copy APIs prepare the hardware usable descriptors for each call.
 *			  This API can be used to enable/disable reuse of the descriptors.
 *
 * @param [in]  hChannel    DMA Channel handle to work on.
 * @param [in]  bDescReuse  'true' to enable descriptor reuse.
 *							'false' to disable descriptor reuse.			
 * @return  Status
 *          - #ADI_DMA_SUCCESS: Successfully submitted Memory transfer descriptor chain.
 *          - #ADI_DMA_NOT_OPEN: The MDMA Stream is not open yet.
 *          - #ADI_DMA_BAD_HANDLE: Supplied MDMA Stream handle is invalid.
 *
 *	@note   - Descriptor reuse for descriptor list mode is not supported if the macro ADI_MDMA_CFG_LIST_MULTIPLE_CALLBACKS_ENABLE is enabled.
 *
 *	@sa		  adi_mdma_CopyList(), adi_mdma_Copy1DArray(), adi_mdma_Copy2DArray()
 */
ADI_DMA_RESULT adi_mdma_EnableDescListReuse(
	ADI_DMA_CHANNEL_HANDLE 			hChannel, 
	bool 							bDescReuse)
{
	/* DMA Channel to work on */
	ADI_MDMA_CHANNEL    *pDmaChInfo = (ADI_MDMA_CHANNEL *)hChannel;

/* IF (Debug information enabled) */
#if defined(ADI_DEBUG)

	/* IF (DMA Channel Handle is invalid) */
	if (ValidateChannelHandle (hChannel) != ADI_DMA_SUCCESS)
	{
		return (ADI_DMA_BAD_HANDLE);
	}

	/* IF (Channel is not in use) */
	if (pDmaChInfo->pChInfo->pMdmaStream == NULL)
	{
		return (ADI_DMA_NOT_OPEN);
	}
#endif /* ADI_DEBUG */

	pDmaChInfo->bDescReuse = bDescReuse;
	
	/* Return success */
    return (ADI_DMA_SUCCESS);
}


/**
 * @brief Provides Memory DMA channel data interrupt SID.
 *
 *
 * @param [in]  hChannel    DMA Channel handle to work on.
 * @param [out] pnSID       Pointer to variable where SID of MDMA channel need to be stored.
 *
 * @return  Status
 *          - #ADI_DMA_SUCCESS: Successfully updated DMA buffer repeat settings.
 *          - #ADI_DMA_BAD_HANDLE[D]: Bad Channel handle.
 *          - #ADI_DMA_NOT_OPEN[D]: DMA Channel is not open yet to perform this operation.
 *          - #ADI_DMA_INVALID_NULL_POINTER[D]:  "pnSID" is invalid and pointing to NULL.
 *
 *
 */
ADI_DMA_RESULT adi_mdma_GetChannelSID(
    ADI_DMA_CHANNEL_HANDLE const    hChannel,
    uint32_t *pnSID)
{
    /* DMA Channel to work on */
	ADI_MDMA_CHANNEL    *pDmaChInfo = (ADI_MDMA_CHANNEL *)hChannel;

/* IF (Debug information enabled) */
#if defined(ADI_DEBUG)

	/* IF (DMA Channel Handle is invalid) */
	if (ValidateChannelHandle (hChannel) != ADI_DMA_SUCCESS)
	{
		return (ADI_DMA_BAD_HANDLE);
	}

	/* IF (Channel is not in use) */
	if (pDmaChInfo->pChInfo->pMdmaStream == NULL)
	{
		return (ADI_DMA_NOT_OPEN);
	}

	if (pnSID == NULL)
	{
		return (ADI_DMA_INVALID_NULL_POINTER);
	}

#endif /* ADI_DEBUG */

	*pnSID = pDmaChInfo->pChInfo->DataIntSID;

    /* Return */
    return (ADI_DMA_SUCCESS);
}

/**
 * @brief Configures Trigger Input settings for a DMA channel.
 *
 * @param [in]  hChannel         DMA channel handle to configure.
 * @param [in]  bTriggerWait     'true' to wait for an incoming trigger before starting the next DMA work unit.
 *                               'false' to ignore incoming trigger and begin work unit automatically.
 * @param [in]  bTriggerOvError  'true' to enable and report trigger overrun error.
 *                               'false' to disable trigger overrun error.
 *
 * @return  Status
 *          - #ADI_DMA_SUCCESS: Successfully configured Trigger Output.
 *          - #ADI_DMA_NOT_OPEN: DMA channel is not open.
 *          - #ADI_DMA_BAD_HANDLE: Supplied DMA handle is invalid.
 *
 * @note	For descriptor based modes, this API can be used to wait for a trigger before the first descriptor load.
 *			For other descriptors, the trigger in setting should be part of the descriptor itself. 
 *
 *	@sa		adi_dma_ConfigTriggerOut()
 */
ADI_DMA_RESULT  adi_dma_ConfigTriggerIn (
    ADI_DMA_CHANNEL_HANDLE      hChannel,
    bool                        bTriggerWait,
    bool                        bTriggerOvError)
{
    /* DMA Channel to work on */
	ADI_MDMA_CHANNEL    *pDmaChInfo = (ADI_MDMA_CHANNEL *)hChannel;

    /* Address where the DMA common config information is stored */
    uint32_t        *pCommonConfig;

/* IF (Debug information enabled) */
#if defined (ADI_DEBUG)

    /* IF (DMA Channel Handle is invalid) */
    if (ValidateChannelHandle (hChannel) != ADI_DMA_SUCCESS)
    {
        return (ADI_DMA_BAD_HANDLE);
    }

    /* IF (Channel is not in use) */
    if (pDmaChInfo->pChInfo->pMdmaStream == NULL)
    {
        return (ADI_DMA_NOT_OPEN);
    }

#endif /* ADI_DEBUG */

    /* Get the DMA Common Config element address for this channel */
    pCommonConfig = &(pDmaChInfo->CommonConfig);

    /* IF (Wait for Trigger input) */
    if (bTriggerWait)
    {
        /* Update DMA configuration register */
        *pCommonConfig |= ENUM_DMA_CFG_TRGWAIT;
    }
    /* ELSE (Ignore Trigger input) */
    else
    {
        /* Update DMA configuration register */
        *pCommonConfig &= ~ENUM_DMA_CFG_TRGWAIT;
    }

    /* IF (Enable Trigger overrun error) */
    if (bTriggerOvError)
    {
        /* Update DMA configuration register */
        *pCommonConfig |= ENUM_DMA_CFG_TOV_EN;
    }
    /* ELSE (Ignore Trigger overrun error) */
    else
    {
        /* Update DMA configuration register */
        *pCommonConfig &= ~ENUM_DMA_CFG_TOV_EN;
    }

    /* Return */
    return (ADI_DMA_SUCCESS);
}

/**
 * @brief Configures Trigger Output settings for a DMA channel.
 *
 * @param [in]  hChannel         DMA channel handle to configure.
 * @param [in]  eDmaTrigger      Condition to generate an outgoing trigger.
 *
 * @return  Status
 *          - #ADI_DMA_SUCCESS: Successfully configured Trigger Output.
 *          - #ADI_DMA_NOT_OPEN: DMA channel is not open.
 *          - #ADI_DMA_BAD_HANDLE: Supplied DMA handle is invalid.
 *
 * @note	This API is usable only for one shot modes.
 *
 * @sa		adi_dma_ConfigTriggerIn()
 */
ADI_DMA_RESULT  adi_dma_ConfigTriggerOut (
    ADI_DMA_CHANNEL_HANDLE      hChannel,
    ADI_DMA_TRIGGER_OUT    		eDmaTrigger)
{
    /* DMA Channel to work on */
	ADI_MDMA_CHANNEL    *pDmaChInfo = (ADI_MDMA_CHANNEL *)hChannel;

    /* Address where the DMA common config information is stored */
    uint32_t        *pCommonConfig;

/* IF (Debug information enabled) */
#if defined (ADI_DEBUG)

    /* IF (DMA Channel Handle is invalid) */
    if (ValidateChannelHandle (hChannel) != ADI_DMA_SUCCESS)
    {
        return (ADI_DMA_BAD_HANDLE);
    }

    /* IF (Channel is not in use) */
    if (pDmaChInfo->pChInfo->pMdmaStream == NULL)
    {
        return (ADI_DMA_NOT_OPEN);
    }

#endif /* ADI_DEBUG */

    /* Get the DMA Common Config element address for this channel */
	pCommonConfig = &(pDmaChInfo->CommonConfig);

    /* Clear outgoing trigger configuration field */
    *pCommonConfig &= ~BITM_DMA_CFG_TRIG;

    /* Update DMA configuration register */
    *pCommonConfig |= eDmaTrigger;

    /* Return */
    return (ADI_DMA_SUCCESS);
}

/**
 * @brief Update DMA Channel callback function. 
 *
 * @param [in]  hChannel    DMA channel handle to work on.
 * @param [in]  pfCallback  Address of application callback function.
 * @param [in]  pCBParam    Parameter passed back to application callback.
 *
 * @return  Status
 *          - #ADI_DMA_SUCCESS: Successfully updated DMA channel callback.
 *          - #ADI_DMA_BAD_HANDLE: Bad Channel handle.
 *          - #ADI_DMA_NOT_OPEN: DMA Channel is not open yet to perform this operation.
 *          - #ADI_DMA_ALREADY_RUNNING: Can't update callback function when the DMA data transfer is enabled.
 *
 * @note Note that only single callback is supported for destination channel transfer done and 
 *		 source/destination error interrupts. Calling this API via source/destination channel handle updates the
 *		 callback for all the three conditions. 
 */
ADI_DMA_RESULT adi_dma_UpdateCallback(
    ADI_DMA_CHANNEL_HANDLE const    hChannel,
    ADI_CALLBACK                    pfCallback,
    void *const                     pCBParam)
{
    /* DMA Channel to work on */
	ADI_MDMA_CHANNEL    *pDmaChInfo = (ADI_MDMA_CHANNEL *)hChannel;

/* IF (Debug information enabled) */
#if defined (ADI_DEBUG)

    /* IF (DMA Channel Handle is invalid) */
    if (ValidateChannelHandle (hChannel) != ADI_DMA_SUCCESS)
    {
        return (ADI_DMA_BAD_HANDLE);
    }

    /* IF (Channel is not in use) */
    if (pDmaChInfo->pChInfo->pMdmaStream == NULL)
    {
        return (ADI_DMA_NOT_OPEN);
    }

#endif /* ADI_DEBUG */

    /* IF (DMA channel is already in the middle of a data transfer) */
    if (pDmaChInfo->pChInfo->pMdmaStream->bIsXferInProgress)
    {
        /* Report failure as DMA channel already running */
        return (ADI_DMA_ALREADY_RUNNING);
    }

    /* Save the supplied callback information */
    pDmaChInfo->pChInfo->pMdmaStream->pfCallback = pfCallback;
    pDmaChInfo->pChInfo->pMdmaStream->pCBParam   = pCBParam;

    /* Return */
    return (ADI_DMA_SUCCESS);
}

/**
 * @brief Gets Current Descriptor pointer register value.
 *
 * @param [in]  hChannel        DMA Channel handle to work on.
 * @param [out] pCurrentDesc    Address of the location to store current descriptor address.
 *
 * @return  Status
 *          - #ADI_DMA_SUCCESS: Successfully queried DMA Channel settings.
 *          - #ADI_DMA_BAD_HANDLE: DMA Channel handle is invalid.
 *
 */
ADI_DMA_RESULT adi_dma_GetCurrentDesc(
    ADI_DMA_CHANNEL_HANDLE          hChannel,
    void                            **pCurrentDesc)
{
    /* DMA Channel to work on */
	ADI_MDMA_CHANNEL    *pDmaChInfo = (ADI_MDMA_CHANNEL *)hChannel;

/* IF (Debug information enabled) */
#if defined (ADI_DEBUG)

    /* IF (DMA Channel Handle is invalid) */
    if (ValidateChannelHandle (hChannel) != ADI_DMA_SUCCESS)
    {
        return (ADI_DMA_BAD_HANDLE);
    }

#endif /* ADI_DEBUG */

#if defined(__ADSPSC598_FAMILY__) && defined(__ADSPCORTEXA55__)
    *pCurrentDesc = (void *) (uint64_t)pDmaChInfo->pChInfo->pReg->CurrDescPtr;
#else
    *pCurrentDesc = (void *) pDmaChInfo->pChInfo->pReg->CurrDescPtr;
#endif /* defined(__ADSPSC598_FAMILY__) && defined(__ADSPCORTEXA55__) */

    /* Return success */
    return (ADI_DMA_SUCCESS);
}

/**
 * @brief Gets Address Current Register value.
 *
 * @details Gets present memory transfer address of the active work unit.
 *
 * @param [in]  hChannel        DMA Channel handle to work on.
 * @param [out] pCurrentAddr    Address of the location to store current data address.
 *
 * @return  Status
 *          - #ADI_DMA_SUCCESS: Successfully queried DMA Channel settings.
 *          - #ADI_DMA_BAD_HANDLE: DMA Channel handle is invalid.
 *
 */
ADI_DMA_RESULT adi_dma_GetCurrentAddr(
    ADI_DMA_CHANNEL_HANDLE          hChannel,
    void                            **pCurrentAddr)
{
    /* DMA Channel to work on */
	ADI_MDMA_CHANNEL    *pDmaChInfo = (ADI_MDMA_CHANNEL *)hChannel;

/* IF (Debug information enabled) */
#if defined (ADI_DEBUG)

    /* IF (DMA Channel Handle is invalid) */
    if (ValidateChannelHandle (hChannel) != ADI_DMA_SUCCESS)
    {
        return (ADI_DMA_BAD_HANDLE);
    }

#endif /* ADI_DEBUG */

#if defined(__ADSPSC598_FAMILY__) && defined(__ADSPCORTEXA55__)
    *pCurrentAddr = (void *) (uint64_t)pDmaChInfo->pChInfo->pReg->CurrAddr;
#else
    *pCurrentAddr = (void *) pDmaChInfo->pChInfo->pReg->CurrAddr;
#endif /* defined(__ADSPSC598_FAMILY__) && defined(__ADSPCORTEXA55__) */

    /* Return success */
    return (ADI_DMA_SUCCESS);
}

/**
 * @brief Gets Current Inner loop count (current X count) Register value.
 *
 * @param [in]  hChannel            DMA Channel handle to work on.
 * @param [out] *pCurrentXCount     Location to store Current X Count Register value.
 *
 * @return  Status
 *          - #ADI_DMA_SUCCESS: Successfully queried DMA Channel settings.
 *          - #ADI_DMA_BAD_HANDLE: DMA Channel handle is invalid.
 *
 */
ADI_DMA_RESULT adi_dma_GetCurrentXCount(
    ADI_DMA_CHANNEL_HANDLE          hChannel,
    uint32_t                        *pCurrentXCount)
{
    /* DMA Channel to work on */
	ADI_MDMA_CHANNEL    *pDmaChInfo = (ADI_MDMA_CHANNEL *)hChannel;

/* IF (Debug information enabled) */
#if defined (ADI_DEBUG)

    /* IF (DMA Channel Handle is invalid) */
    if (ValidateChannelHandle (hChannel) != ADI_DMA_SUCCESS)
    {
        return (ADI_DMA_BAD_HANDLE);
    }

#endif /* ADI_DEBUG */

    *pCurrentXCount = pDmaChInfo->pChInfo->pReg->CurrXCount;

    /* Return success */
    return (ADI_DMA_SUCCESS);
}

/**
 * @brief Gets Current Outer loop count (current Y count) Register value.
 *
 *
 * @param [in]  hChannel            DMA Channel handle to work on.
 * @param [out] *pCurrentYCount     Location to store Current Y Count Register value.
 *
 * @return  Status
 *          - #ADI_DMA_SUCCESS: Successfully queried DMA Channel settings.
 *          - #ADI_DMA_BAD_HANDLE: DMA Channel handle is invalid.
 *
 */
ADI_DMA_RESULT adi_dma_GetCurrentYCount(
    ADI_DMA_CHANNEL_HANDLE          hChannel,
    uint32_t                        *pCurrentYCount)
{
    /* DMA Channel to work on */
	ADI_MDMA_CHANNEL    *pDmaChInfo = (ADI_MDMA_CHANNEL *)hChannel;

/* IF (Debug information enabled) */
#if defined (ADI_DEBUG)

    /* IF (DMA Channel Handle is invalid) */
    if (ValidateChannelHandle (hChannel) != ADI_DMA_SUCCESS)
    {
        return (ADI_DMA_BAD_HANDLE);
    }

#endif /* ADI_DEBUG */

    *pCurrentYCount = pDmaChInfo->pChInfo->pReg->CurrYCount;

    /* Return success */
    return (ADI_DMA_SUCCESS);
}

/**
 * @brief Limits the Memory DMA channel bandwidth.
 *
 *
 * @param [in]  hChannel            DMA Channel handle to work on.
 * @param [in]  TargetBW            Target bandwidth in Bytes/s.
 * @param [in]  eElementWidth       Data element width in terms of ADI_DMA_MSIZE.
 *
 * @return  Status
 *			- #ADI_DMA_SUCCESS: Successfully configured BW limit settings.
 *			- #ADI_DMA_BAD_HANDLE: DMA Channel handle is invalid.
 *			- #ADI_DMA_NOT_OPEN: The MDMA Stream is not open yet.
 *			- #ADI_DMA_FAILURE: Power service API couldn't be initialized.
 * @note 
 *			- Ensure that  the configuration macro ADI_MDMA_CFG_ENABLE_BANDWIDTH_FUNCTIONS is enabled to use this API.
 *			- This API can be used only with adi_mdma_Copy1D() and adi_mdma_Copy2D() APIs.
 *			- This API should be called before calling the Memory DMA copy API. 
 *			- This API makes use of power service and assumes that the adi_pwr_Init() is called by the application before making use of this API.

 */
ADI_DMA_RESULT adi_mdma_BWLimit(
		ADI_DMA_CHANNEL_HANDLE hChannel,
		uint32_t TargetBW,
		ADI_DMA_MSIZE eElementWidth)
{
	/* DMA Channel to work on */
	ADI_MDMA_CHANNEL    *pDmaChInfo = (ADI_MDMA_CHANNEL *)hChannel;

	/* Variables for temporary storage */
	uint32_t fsysclk, fsclk0, fsclk1, MSize;

	float mdmaclk; float BWLimit;
	
/* IF (Debug information enabled) */
#if defined(ADI_DEBUG)
	/* IF (DMA Channel Handle is invalid) */
	if (ValidateChannelHandle (hChannel) != ADI_DMA_SUCCESS)
	{
		return (ADI_DMA_BAD_HANDLE);
	}

	/* IF (Channel is not in use) */
	if (pDmaChInfo->pChInfo->pMdmaStream == NULL)
	{
		return (ADI_DMA_NOT_OPEN);
	}
#endif 

	/* Get MSize value */
    MSize = ((uint32_t)eElementWidth >> BITP_DMA_CFG_MSIZE);
    MSize=1<<MSize;

    /* Get the value of SCLK/SYSCK frequency used for the Memory DMA */
	adi_pwr_GetSystemFreq(0, &fsysclk, &fsclk0, &fsclk1);

	mdmaclk = (float)fsysclk;
	
	BWLimit = mdmaclk*(float)MSize/(float)TargetBW;
	
	pDmaChInfo->pChInfo->pReg->BwLimit = (uint32_t)BWLimit;
	
	return (ADI_DMA_SUCCESS);
}

/**
 * @brief Configures Memory DMA channel monitor register.
 *
 * @param [in]  hChannel            DMA Channel handle to work on.
 * @param [in]  TargetBW            Target bandwidth in Bytes/s.
 * @param [in]  ElementCountInBytes Element count value in bytes.
 *
 * @return  Status
 *			- #ADI_DMA_SUCCESS: Successfully configured BW monitor settings.
 *			- #ADI_DMA_BAD_HANDLE: DMA Channel handle is invalid.
 *			- #ADI_DMA_NOT_OPEN: The MDMA Stream is not open yet.
 *			- #ADI_DMA_FAILURE: Power service API couldn't be initialized or BW monitor count expired.
 * @note 
 *			- Ensure that  the configuration macro ADI_MDMA_CFG_ENABLE_BANDWIDTH_FUNCTIONS is enabled to use this API.
 *			- This API can be used only with adi_mdma_Copy1D() and adi_mdma_Copy2D() APIs.
 *			- This API should be called before calling the Memory DMA copy API. 
 *			- This API makes use of power service and assumes that the adi_pwr_Init() is called by the application before making use of this API.
 */
ADI_DMA_RESULT adi_mdma_BWMonitor(
		ADI_DMA_CHANNEL_HANDLE hChannel,
		uint32_t TargetBW,
		uint32_t ElementCountInBytes)
{
	/* DMA Channel to work on */
	ADI_MDMA_CHANNEL    *pDmaChInfo = (ADI_MDMA_CHANNEL *)hChannel;
	
	/* Variables for temporary storage */
	uint32_t fsysclk, fsclk0, fsclk1;
	
	float mdmaclk; float BWMonitor;
	
/* IF (Debug information enabled) */
#if defined(ADI_DEBUG)
	/* IF (DMA Channel Handle is invalid) */
	if (ValidateChannelHandle (hChannel) != ADI_DMA_SUCCESS)
	{
		return (ADI_DMA_BAD_HANDLE);
	}

	/* IF (Channel is not in use) */
	if (pDmaChInfo->pChInfo->pMdmaStream == NULL)
	{
		return (ADI_DMA_NOT_OPEN);
	}
#endif 

    /* Get the value of SCLK/SYSCK frequency used for the Memory DMA */
	adi_pwr_GetSystemFreq(0, &fsysclk, &fsclk0, &fsclk1);

	mdmaclk = (float)fsysclk;

	BWMonitor = mdmaclk*(float)ElementCountInBytes/(float)TargetBW;
	
	pDmaChInfo->pChInfo->pReg->BwMonitor = (uint32_t)BWMonitor;
	
	return (ADI_DMA_SUCCESS);
}

/**
 * @brief Measures the bandwidth of a Memory DMA channel.
 *
 *
 * @param [in]  hChannel            DMA Channel handle to work on.
 * @param [out] pMeasuredBW         Address where the measured bandwidth will be stored in Bytes/s.
 * @param [in]  ElementCountInBytes No of bytes transferred in the last transfer for which the bandwidth measurement is needed.
 *
 * @return  Status
 *			- #ADI_DMA_SUCCESS: Successfully measured Memory DMA channel bandwidth.
 *			- #ADI_DMA_BAD_HANDLE: DMA Channel handle is invalid.
 *			- #ADI_DMA_NOT_OPEN: The MDMA Stream is not open yet.
 *			- #ADI_DMA_FAILURE: Power service API couldn't be initialized or BW monitor count expired.
 *
 * @note 
 *			- Ensure that  the configuration macro ADI_MDMA_CFG_ENABLE_BANDWIDTH_FUNCTIONS is enabled to use this API.
 *			- This API can be used only with adi_mdma_Copy1D() and adi_mdma_Copy2D() APIs.
 *			- This API should be called before calling the Memory DMA copy API. 
 *			- This API makes use of power service and assumes that the adi_pwr_Init() is called by the application before making use of this API.
 *			- The API only measures the hardware DAM channel bandwidth and doesn't include software overheads.
 *
 */
ADI_DMA_RESULT adi_mdma_BWMeasure(
		ADI_DMA_CHANNEL_HANDLE hChannel,
		uint32_t *pMeasuredBW,
		uint32_t ElementCountInBytes)
{
	/* DMA Channel to work on */
	ADI_MDMA_CHANNEL    *pDmaChInfo = (ADI_MDMA_CHANNEL *)hChannel;
	
	/* Variables for temporary storage */
	uint32_t fsysclk, fsclk0, fsclk1, Cycles;

	float mdmaclk; float BW;
	
/* IF (Debug information enabled) */
#if defined(ADI_DEBUG)
	/* IF (DMA Channel Handle is invalid) */
	if (ValidateChannelHandle (hChannel) != ADI_DMA_SUCCESS)
	{
		return (ADI_DMA_BAD_HANDLE);
	}

	/* IF (Channel is not in use) */
	if (pDmaChInfo->pChInfo->pMdmaStream == NULL)
	{
		return (ADI_DMA_NOT_OPEN);
	}
#endif 

   /* Check if BW monitor count expired */
    if(pDmaChInfo->pChInfo->pReg->CurrBwMonitor==0)
    {
    	return (ADI_DMA_FAILURE);
    }

    /* Get the value of SCLK/SYSCK frequency used for the Memory DMA */
	adi_pwr_GetSystemFreq(0, &fsysclk, &fsclk0, &fsclk1);
	
	/* Calculate the number of SCLK/SYSCLK cycles taken by the last transfer */
	Cycles = pDmaChInfo->pChInfo->pReg->BwMonitor - pDmaChInfo->pChInfo->pReg->CurrBwMonitor;
	
	mdmaclk = (float)fsysclk;
	
	BW = mdmaclk*(float)ElementCountInBytes/Cycles;

	*pMeasuredBW = (uint32_t)BW;
	
	return (ADI_DMA_SUCCESS);
}

#ifdef _MISRA_RULES
#pragma diag(pop)
#endif /* _MISRA_RULES */
/*****/

/*@}*/
