/*********************************************************************************

Copyright(c) 2019-21 Analog Devices, Inc. All Rights Reserved.

This software is proprietary.  By using this software you agree
to the terms of the associated Analog Devices License Agreement.

*********************************************************************************/

/*!
 * @file:    adi_mdma_def_2156x.h
 * @brief:   Private header file for for Memory DMA Manager.
 * @version: $Revision: 68342 $
 * @date:    $Date: 2022-09-20 06:27:14 -0400 (Tue, 20 Sep 2022) $
 *
 * @details
 *           This is a private header file for the Memory DMA Manager service
 *           which contains the API declarations, data and constant definitions
 *	         used in Memory DMA Manager implementation
 */

/*! \addtogroup MDMA_Manager Memory DMA Manager
 *  @{
 */

#ifndef __ADI_MDMA_DEF_2156X_H__
#define __ADI_MDMA_DEF_2156X_H__

#ifdef _MISRA_RULES
#pragma diag(push)
#pragma diag(suppress:misra_rule_5_1:"Allow identifiers to be more than 31 characters")
#pragma diag(suppress:misra_rule_5_3:" ")
#pragma diag(suppress:misra_rule_5_4:" ")
#pragma diag(suppress:misra_rule_5_6:"Allow identifier names to be reused")
#pragma diag(suppress:misra_rule_5_7:"Allow identifier names to be reused")
#pragma diag(suppress:misra_rule_18_4:" ")
#endif /* _MISRA_RULES */

/*=============  I N C L U D E S   =============*/
/* This is an issue with sys/adi_core.h which should be including all of its required header files */

#include "sys/platform.h"
#include <sys/adi_core.h>

/* OSAL includes */
#include <adi_osal.h>

/* Interrupt Manager includes */
#include <services/int/adi_int.h>

/* DMA Manager includes */
#include <services/dma/adi_dma.h>

/* memcpy includes */
#include <string.h>

/* Memory size check */
#include <assert.h>

/*==============  D E F I N E S  ===============*/

/* Clear DMA interrupts */
#define ADI_DMA_CLEAR_INTERRUPTS        (ENUM_DMA_STAT_PIRQ | ENUM_DMA_STAT_IRQERR | ENUM_DMA_STAT_IRQDONE)

/* Maximum MSize */
#define ADI_DMA_MAX_MSIZE_VAL               (ENUM_DMA_CFG_MSIZE32 >> BITP_DMA_CFG_MSIZE)

/* Masks for MSIZE and PSIZE values */
#define ADI_DMA_MSIZE_MASK             (0x1Fu)
#define ADI_DMA_PSIZE_MASK             (0x0Fu)

/* Total number of MDMA streams */
#if defined(__ADSPSC573_FAMILY__)||defined(__ADSP21569_FAMILY__)
#define ADI_MDMA_TOTAL_STREAMS		   (4)
#elif defined (__ADSPSC594_FAMILY__) || defined (__ADSPSC598_FAMILY__)
#define ADI_MDMA_TOTAL_STREAMS		   (8)
#endif /* defined(__ADSPSC573_FAMILY__)||defined(defined(__ADSP21569_FAMILY__) */

/* ADSP-SC5xx and ADSP-215xx processor families */
#if defined(__ADSP215xx__)

/* Common addresses
 */

/* Sharc L1 private address space start address */
#define ADI_SC5XX_SHARC_L1_START_ADDR       (0x00240000u)

/* ADSP-SC57x-specific addresses.
 */
#if defined(__ADSPSC573_FAMILY__)

/* Sharc L1 private address space end address */
#define ADI_SC5XX_SHARC_L1_END_ADDR         (0x0038FFFFu)

#endif /* defined(__ADSPSC573_FAMILY__) */

/* ADSP-2156x/SC59x-specific addresses.
 */
#if defined(__ADSP21569_FAMILY__) || defined (__ADSPSC594_FAMILY__) || defined (__ADSPSC598_FAMILY__)

/* Sharc L1 private address space end address */
#define ADI_SC5XX_SHARC_L1_END_ADDR         (0x0039FFFFu)

#endif /* defined(__ADSP21569_FAMILY__) || defined (__ADSPSC594_FAMILY__) || defined (__ADSPSC598_FAMILY__) */

#endif /* defined(__ADSP215xx__) */

/* Memory DMA stream structure definition for prototypes*/
struct ADI_MDMA_STREAM;

/* Memory DMA data transfer mode */
typedef enum
{
    ADI_MDMA_XFER_NONE		 = (0u),                                  /* MDMA is idle */
    ADI_MDMA_XFER_ONE_SHOT	 = ADI_DMA_EVENT_BUFFER_PROCESSED,        /* One shot 1D/2D transfer */
    ADI_MDMA_XFER_LARGE_LIST = ADI_DMA_EVENT_DESC_LIST_PROCESSED,     /* Large model Descriptor type transfer */
    ADI_MDMA_XFER_ARRAY_1D   = ADI_DMA_EVENT_DESC_1D_ARRAY_PROCESSED, /* 1D Array Descriptor type transfer */
    ADI_MDMA_XFER_ARRAY_2D   = ADI_DMA_EVENT_DESC_2D_ARRAY_PROCESSED  /* 2D Array Descriptor type transfer */
} ADI_MDMA_DATA_XFER_MODE;

#pragma pack(4)
/* DMA register structure  */
typedef struct 
{
    volatile uint32_t   NextDescPtr;    /* Pointer to next Initial Descriptor */
    volatile uint32_t   StartAddr;      /* Start address of current buffer */
    volatile uint32_t   Config;         /* Configuration register */
    volatile uint32_t   XCount;         /* XCOUNT - Inner loop count start value in number of MSIZE data transfers*/
    volatile int32_t    XModify;        /* XMODIFY - Inner loop address increment, in bytes */
    volatile uint32_t   YCount;         /* YCOUNT - Outer loop count start value (2D only), in number of rows*/
    volatile int32_t    YModify;        /* YMODIFY - Outer loop address increment (2D only), in bytes*/
    uint32_t            Padding0;
    uint32_t            Padding1;
    volatile uint32_t   CurrDescPtr;    /* Current Descriptor Pointer */
    volatile uint32_t   PrevDescPtr;    /* Previous Initial Descriptor Pointer */
    volatile uint32_t   CurrAddr;       /* Current Address */
    volatile uint32_t   Status;         /* Status Register */
    volatile uint32_t   CurrXCount;     /* Current XCOUNT */
    volatile uint32_t   CurrYCount;     /* Current YCOUNT */
    uint32_t            Padding3;
    volatile uint32_t   BwLimit;        /* Bandwidth Limit Count */
    volatile uint32_t   CurrBwLimit;    /* Bandwidth Limit Count Current */
    volatile uint32_t   BwMonitor;      /* Bandwidth Monitor Count */
    volatile uint32_t   CurrBwMonitor;  /* Bandwidth Monitor Count Current */
} ADI_DMA_REGS;
#pragma pack()

/* Structure to handle processor specific Memory DMA channel information */
typedef struct 
{
	uint32_t					ChannelID;		/* DMA channel ID */
    uint32_t					DataIntSID;		/* DMA channel data interrupt ID */
    uint32_t					ErrorIntSID;	/* DMA channel error interrupt ID */
    volatile ADI_DMA_REGS 	*pReg;			/* DMA channel registers */
    struct ADI_MDMA_STREAM  *pMdmaStream;	/* MDMA stream corresponding to the DMA channel */
} ADI_MDMA_CHANNEL_INFO;

/* Structure to handle a Memory DMA Channel */
typedef struct 
{
    ADI_MDMA_CHANNEL_INFO	*pChInfo;		/* DMA channel information */
    void					*pCallbackArg;	/* Stores the callback argument for different modes */
    uint32_t                CommonConfig;	/* DMA Configuration values common to all transfer modes */
	bool					bDescReuse;		/* Whether or not to reuse the descriptors for this channel */
} ADI_MDMA_CHANNEL;

/* Structure to handle Memory DMA Stream */
typedef struct ADI_MDMA_STREAM
{
    bool                    bIsXferInProgress;      /* TRUE when a DMA data transfer is currently in progress */
    ADI_CALLBACK            pfCallback;             /* Client supplied callback function */
    void                    *pCBParam;              /* Client supplied callback parameter */
    ADI_MDMA_DATA_XFER_MODE eXferMode; 				/* MDMA transfer mode */
    ADI_MDMA_CHANNEL		SrcCh;					/* Source channel information */
    ADI_MDMA_CHANNEL		DestCh;					/* Destination channel information */
} ADI_MDMA_STREAM;

/* Structure to handle processor specific Memory DMA stream information  */
typedef struct ADI_MDMA_STREAM_INFO
{
	ADI_MDMA_CHANNEL_INFO SrcChInfo;	/* Source DMA channel information */
	ADI_MDMA_CHANNEL_INFO DestChInfo;  /* Destination DMA channel information */
} ADI_MDMA_STREAM_INFO;

#if defined(__ADSPSC573_FAMILY__)||defined(__ADSP21569_FAMILY__) || defined (__ADSPSC594_FAMILY__) || defined (__ADSPSC598_FAMILY__)
/* Processor specific Memory DMA stream information */
static ADI_MDMA_STREAM_INFO StreamInfo[ADI_MDMA_TOTAL_STREAMS] =
{
   {{CHID_MDMA0_SRC, INTR_SYS_MDMA0_SRC, INTR_SYS_MDMA0_SRC_ERR, (volatile ADI_DMA_REGS *)REG_DMA8_DSCPTR_NXT, NULL},
   {CHID_MDMA0_DST, INTR_SYS_MDMA0_DST, INTR_SYS_MDMA0_DST_ERR, (volatile ADI_DMA_REGS *)REG_DMA9_DSCPTR_NXT, NULL}},
   {{CHID_MDMA1_SRC, INTR_SYS_MDMA1_SRC, INTR_SYS_MDMA1_SRC_ERR, (volatile ADI_DMA_REGS *)REG_DMA18_DSCPTR_NXT, NULL},
   {CHID_MDMA1_DST, INTR_SYS_MDMA1_DST, INTR_SYS_MDMA1_DST_ERR, (volatile ADI_DMA_REGS *)REG_DMA19_DSCPTR_NXT, NULL}},
   {{CHID_MDMA2_SRC, INTR_SYS_MDMA2_SRC, INTR_SYS_MDMA2_SRC_ERR, (volatile ADI_DMA_REGS *)REG_DMA39_DSCPTR_NXT, NULL},
   {CHID_MDMA2_DST, INTR_SYS_MDMA2_DST, INTR_SYS_MDMA2_DST_ERR, (volatile ADI_DMA_REGS *)REG_DMA40_DSCPTR_NXT, NULL}},
   {{CHID_MDMA3_SRC, INTR_SYS_MDMA3_SRC, INTR_SYS_MDMA3_SRC_ERR, (volatile ADI_DMA_REGS *)REG_DMA43_DSCPTR_NXT, NULL},
   {CHID_MDMA3_DST, INTR_SYS_MDMA3_DST, INTR_SYS_MDMA3_DST_ERR, (volatile ADI_DMA_REGS *)REG_DMA44_DSCPTR_NXT, NULL}}
#if defined (__ADSPSC594_FAMILY__) || defined (__ADSPSC598_FAMILY__)
   ,{{CHID_MDMA4_SRC, INTR_SYS_MDMA4_SRC, INTR_SYS_MDMA4_SRC_ERR, (volatile ADI_DMA_REGS *)REG_DMA45_DSCPTR_NXT, NULL},
    {CHID_MDMA4_DST, INTR_SYS_MDMA4_DST, INTR_SYS_MDMA4_DST_ERR, (volatile ADI_DMA_REGS *)REG_DMA46_DSCPTR_NXT, NULL}},
    {{CHID_MDMA5_SRC, INTR_SYS_MDMA5_SRC, INTR_SYS_MDMA5_SRC_ERR, (volatile ADI_DMA_REGS *)REG_DMA47_DSCPTR_NXT, NULL},
    {CHID_MDMA5_DST, INTR_SYS_MDMA5_DST, INTR_SYS_MDMA5_DST_ERR, (volatile ADI_DMA_REGS *)REG_DMA48_DSCPTR_NXT, NULL}},
	{{CHID_MDMA6_SRC, INTR_SYS_MDMA6_SRC, INTR_SYS_MDMA6_SRC_ERR, (volatile ADI_DMA_REGS *)REG_DMA49_DSCPTR_NXT, NULL},
	{CHID_MDMA6_DST, INTR_SYS_MDMA6_DST, INTR_SYS_MDMA6_DST_ERR, (volatile ADI_DMA_REGS *)REG_DMA50_DSCPTR_NXT, NULL}},
	{{CHID_MDMA7_SRC, INTR_SYS_MDMA7_SRC, INTR_SYS_MDMA7_SRC_ERR, (volatile ADI_DMA_REGS *)REG_DMA51_DSCPTR_NXT, NULL},
	{CHID_MDMA7_DST, INTR_SYS_MDMA7_DST, INTR_SYS_MDMA7_DST_ERR, (volatile ADI_DMA_REGS *)REG_DMA52_DSCPTR_NXT, NULL}}
#endif /* defined (__ADSPSC594_FAMILY__) || defined (__ADSPSC598_FAMILY__) */

};
#endif /* defined(__ADSPSC573_FAMILY__)||defined(defined(__ADSP21569_FAMILY__) || defined (__ADSPSC594_FAMILY__) || defined (__ADSPSC598_FAMILY__) */

#ifdef _MISRA_RULES
#pragma diag(pop)
#endif /* _MISRA_RULES */

#endif  /* __ADI_MDMA_DEF_2156X_H__ */
/*****/

/*@}*/
